<?php

namespace WPEP\Admin\Container;

use WPEP\Entity\Assessment as EntityAssessment;

trait AssessmentBuilder {

  public function display_assessment_for_course( $section_id = 0, $course_id = null ) {
    $course_id         = ( $course_id === null ? get_the_ID() : $course_id );
    $assessment_id     = wpep_get_course_assessment_id( $course_id, $section_id );
    $field_prefix      = 'wpep_assessment[' . $section_id . ']';

    $this->display_assessment_setup_functionality( $assessment_id, $field_prefix, false );
  }

  public function display_assessment_setup_functionality(
    $assessment_id,
    $field_prefix = 'wpep_assessment',
    $on_post_page = false,
    $text_editor_preload = null
  ) {
    if( $text_editor_preload == null )
      $text_editor_preload = wpep_get_setting_flag( 'lesson-wp-editor-preload' );

    $meta_field_prefix = $field_prefix . '[meta_information]';

    echo    '<h3>' . __( "Assessment Settings") . '</h3>';
    echo    '<div class="wpep-form-group wpep-form-group-assessment-general-information">';
    if( $on_post_page == false ) {
      echo      '<div class="wpep-form-group-row">';
      echo        '<label>' . __( "Status", "wpep" ) . '</label>';
      $this->display_select_field(
        'post_status',
        [
          'draft'   => __( 'Draft - Inactive', 'wpep' ),
          'private' => __( 'Private - Admin Available', "wpep" ),
          'publish' => __( 'Active - Available for all students', "wpep" )
        ],
        $field_prefix . '[post_status]',
        ( $assessment_id == 0 ? null : get_post_status( $assessment_id ) )
      );
      echo      '</div>';
      echo      '<div class="wpep-form-group-row">';
      echo        '<label>' . __( "Title", "wpep" ) . '</label>';
      $this->display_text_field(
        'title',
        $field_prefix . '[title]',
        ( $assessment_id == 0 ? '' : get_the_title( $assessment_id ) )
      );
      echo        '<p class="description">' . __( "Leaving the Title empty will result in this assessment not being saved.", "wpep" ) . '</p>';
      echo      '</div>';
    }
    echo      '<div class="wpep-form-group-row">';
    echo        '<label>' . __( "Allowed Minutes", "wpep" ) . '</label>';
    $this->display_text_field(
      'wpep_available_minutes',
      $meta_field_prefix . '[wpep_available_minutes]',
      intval( get_post_meta( $assessment_id, 'wpep_available_minutes', true ) )
    );
    echo        '<p class="description">' . __( "The number of minutes the student may take to complete this assessment, leave 0 for no time restriction.", "wpep" ) . '</p>';
    echo      '</div>';

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo">';
    echo        '<label>' . __( "Submission", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_submission_restraint',
      [
        2   => __( 'Students can always change their answers and may take the Test any number of times.', "wpep" ),
        1   => __( 'Students can change their answers until the Submission is being reviewed by the Teacher.', 'wpep' ),
        0   => __( 'Students can Submit answers only once.', 'wpep' ),
      ],
      $meta_field_prefix . '[wpep_submission_restraint]',
      intval( get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) )
    );
    $this->display_select_field(
      'wpep_paginate_questions',
      [
        0   => __( 'Display all Questions on one Page', 'wpep' ),
        1   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "1" ),
        2   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "2" ),
        3   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "3" ),
      ],
      $meta_field_prefix . '[wpep_paginate_questions]',
      intval( get_post_meta( $assessment_id, 'wpep_paginate_questions', true ) ),
      'wpep-select-field wpep-paginate-questions'
    );
    echo      '</div>';
    //// End Submission Settings

    $display_welcome_page = get_post_meta( $assessment_id, 'wpep_enable_welcome_page', true );

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo">';
    echo        '<label>' . __( "Interaction", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_enable_welcome_page',
      [
        1   => __( 'Display the Welcome Page at the beginning of the Testing Journey.', 'wpep' ),
        0   => __( "Do not Display the Welcome Page at the beginning of the Testing Journey.", 'wpep' ),
      ],
      $meta_field_prefix . '[wpep_enable_welcome_page]',
      ( $display_welcome_page === null ? 1 : $display_welcome_page )
    );
    $this->display_select_field(
      'wpep_enable_view_questions',
      [
        0   => __( "Disable the Questions Summary View at the end of the Testing Journey.", 'wpep' ),
        1   => __( 'Enable the Questions Summary View at the end of the Testing Journey.', 'wpep' ),
      ],
      $meta_field_prefix . '[wpep_enable_view_questions]',
      intval( get_post_meta( $assessment_id, 'wpep_enable_view_questions', true ) )
    );
    echo      '</div>';
    //// End Submission Settings

    $grading_option = get_post_meta( $assessment_id, 'wpep_grading_option', true );

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo">';
    echo        '<label>' . __( "Grading", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_grading_option',
      [
        0   => __( 'Disabled', 'wpep' ),
        1   => __( 'Manual', "wpep" ),
        2   => __( 'Semi Automatic, requires manual grading by the Teacher. (Question Type: Text, Essay, File Upload)', 'wpep' ),
        3   => __( 'Automatic, all answers that cannot be graded are always correct.', "wpep" )
      ],
      $meta_field_prefix . '[wpep_grading_option]',
      ( $grading_option === '' || $grading_option === false ? 3 : $grading_option )
    );

    $grading_requirement = get_post_meta( $assessment_id, 'wpep_grading_requirement', true );

    $this->display_select_field(
      'wpep_grading_requirement',
      wpep_get_indexed_number_list(0, 100, __( "required Score to Pass the Test %value%%", "wpep" ) ),
      $meta_field_prefix . '[wpep_grading_requirement]',
      ( $grading_requirement == null ? 70 : intval( $grading_requirement ) )
    );
    echo      '</div>';
    //// End Submission Settings

    //// Start Shuffle Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-checkbox-combo">';
    echo        '<span>' . __( "Shuffle", "wpep" ) . '</span>';
    echo        '<label class="shuffle-questions-container">';
    $this->display_checkbox_field(
      'shuffle_questions',
      get_post_meta( $assessment_id, 'wpep_shuffle_questions', true ),
      $meta_field_prefix . '[wpep_shuffle_questions]',
      0
    );
    echo        __( "Questions", "wpep" ) . '</label>';

    echo        '<label>';
    $this->display_checkbox_field(
      'shuffle_answers',
      get_post_meta( $assessment_id, 'wpep_shuffle_answers', true ),
      $meta_field_prefix . '[wpep_shuffle_answers]',
      0
    );
    echo        __( "Answers", "wpep" ) . '</label>';
    echo      '</div>';
    echo    '</div>';
    //// End Shuffle Settings

    do_action( "wpep_administration_assessment_meta_options_block", $meta_field_prefix, $assessment_id );

    echo   '<div class="wpep-toggle-container" data-wpep-toggled="success">';
    echo   '<h3>' . vsprintf( __( "Assessment %s / %s Message", "wpep" ), [
        '<span data-wpep-toggle="success">' . __( "Success", "wpep" ) . '</span>',
        '<span data-wpep-toggle="failed">' . __( "Failed", "wpep" ) . '</span>'
      ] ) . '</h3>';

    echo        '<div class="wpep-form-group" data-wpep-toggled="success" data-assessment-id="' . $assessment_id . '" data-assessment-type="success">';
    echo        '<label>';
    $this->display_checkbox_field(
      'wpep_success_message_disable_indicators',
      get_post_meta( $assessment_id, 'wpep_success_message_disable_indicators', true ),
      $meta_field_prefix . '[wpep_success_message_disable_indicators]',
      0
    );
    echo        __( "Hide Visual Success Indicators: Test Result Score & number of correctly answered questions from the Assessment Success Page.", "wpep" ) . '</label>';
    echo        '</div>';

    echo        '<div class="wpep-form-group textarea primary-content-editor" data-wpep-toggled="success" data-assessment-id="' . $assessment_id . '" data-assessment-type="success">';
    if( $text_editor_preload || $on_post_page ) {
      $assessment_content = get_post_meta( $assessment_id, 'wpep_success_message', true );

      if( $assessment_content == '' )
        $assessment_content = wpep_get_setting_translated( 'assessment-success-message' );

      wpep_editor(
        $assessment_content,
        'asm-' . md5( $meta_field_prefix ),
        $meta_field_prefix . '[wpep_success_message]'
      );
    }
    echo        '</div>';

    echo        '<div class="wpep-form-group" data-wpep-toggled="failed" data-assessment-id="' . $assessment_id . '" data-assessment-type="failed">';
    echo        '<label>';
    $this->display_checkbox_field(
      'wpep_failed_message_disable_indicators',
      get_post_meta( $assessment_id, 'wpep_failed_message_disable_indicators', true ),
      $meta_field_prefix . '[wpep_failed_message_disable_indicators]',
      0
    );
    echo        __( "Hide Visual Failure Indicators: Test Result Score & number of correctly answered questions from the Assessment Failure Page.", "wpep" ) . '</label>';
    echo        '</div>';

    echo        '<div class="wpep-form-group textarea primary-content-editor" data-wpep-toggled="failed" data-assessment-id="' . $assessment_id . '" data-assessment-type="failed">';
    if( $text_editor_preload || $on_post_page ) {
      $assessment_content = get_post_meta( $assessment_id, 'wpep_failed_message', true );

      if( $assessment_content == '' )
        $assessment_content = wpep_get_setting_translated( 'assessment-status-failed' );

      wpep_editor(
        $assessment_content,
        'afm-' . md5( $meta_field_prefix ),
        $meta_field_prefix . '[wpep_failed_message]'
      );
    }
    echo        '</div>';
    echo   '</div>';

    echo   '<h3>' . __( "Assessment Questions") . '</h3>';

    echo    '<div class="wpep-assessment-question-container">';

    $assessment_questions = ( is_numeric( $assessment_id ) ? EntityAssessment::instance()->get_questions_by_assessment_id( $assessment_id ) : [] );

    if( !empty( $assessment_questions ) ) {
      foreach( $assessment_questions as $question_id => $question ) {
        $this->_display_assessment_question( $field_prefix, $question );
      }
    } else {
      $this->_display_assessment_question(
        $field_prefix,
        null
      );
    }

    $this->_display_assessment_question(
      str_replace( 'wpep_assessment', 'wpep_assessment_template', $field_prefix ),
      null,
      "wpep-assessment-question-item wpep-assessment-question-item-hidden",
      false
    );

    echo      '<div class="wpep-assessment-question-container-footer">';
    echo        '<span class="button button-primary wpep-add-question">' . __( "Add Question", "wpep" ) . '</span>';
    echo      '</div>';
    echo      '<div class="clear"></div>';
    echo    '</div>';
  }

  /**
   * @param $field_prefix
   * @param null $question
   * @param string $fieldset_class
   * @param null $editor_preload
   */
  private function _display_assessment_question( $field_prefix, $question = null, $fieldset_class = "wpep-assessment-question-item", $editor_preload = null ) {
    $editor_preload = ( $editor_preload === null ? wpep_get_setting_flag( 'lesson-wp-editor-preload' ) : $editor_preload );

    global $wp_version;

    $current_question_id     = ( isset( $question->id ) ? $question->id : uniqid('question_') );
    $current_question_prefix = $field_prefix . '[question]' . '[' . $current_question_id . ']';
    $current_question_type   = ( isset( $question->type ) ? $question->type : wpep_get_assessment_field_type_default() );

    echo '<fieldset class="' . $fieldset_class . '" data-question-id="' . $current_question_id . '" data-field-group-prefix="' . $current_question_prefix . '">';
    echo    '<div class="header-section">';
    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    $this->display_text_field(
      'title',
      $current_question_prefix . '[title]',
      ( isset( $question->title ) ? $question->title : '' ),
      "wpep-question-title",
      __( "New Question", "wpep" )
    );

    echo      '<span class="element-delete"></span>';
    echo      '<span class="element-open"></span>';
    echo      '<span class="element-close"></span>';

    $this->display_select_field(
      'type',
      wpep_get_assessment_field_types(),
      $current_question_prefix . '[type]',
      $current_question_type,
      'wpep-assessment-question-field-type'
    );

    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';
    echo      '<div class="wpep-form-group question-assessment-settings-container">';

    echo      '<div class="wpep-form-group-row">';
    echo        '<label>' . __( "Weight", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_submission_restraint',
      wpep_get_indexed_number_list(0, 15, 'This question is worth %value% points.'),
      $current_question_prefix . '[weight]',
      isset( $question->weight ) ? $question->weight : 1,
      'wpep-select-field wpep-select-field-weight'
    );
    echo      '</div>';
    
    echo        '<div class="wpep-form-group textarea primary-content-editor">';

    if( $editor_preload ) {
      wpep_editor(
        html_entity_decode( isset( $question->content ) ? $question->content : '' ),
        'question-' . $current_question_id,
        $current_question_prefix . '[content]'
      );
    }

    echo        '</div>';
    echo      '</div>';
    echo      '<div class="question-assessment-answers-container ' . ( wpep_get_assessment_field_types()[$current_question_type]['has_answers'] ? 'question-assessment-answers-container-active' : '' ) . '">';

    if( isset( $question->answers ) && !empty( $question->answers ) )
      foreach( $question->answers as $question_answer )
        $this->_display_assessment_question_answer(
          $field_prefix, $current_question_id, $question_answer
        );
    $this->_display_assessment_question_answer(
      ( strpos( $field_prefix, '_template' ) !== false ? $field_prefix :  str_replace( 'wpep_assessment', 'wpep_assessment_template', $field_prefix ) ),
      $current_question_id,
      null,
      "wpep-assessment-question-answer-item wpep-assessment-question-answer-item-hidden"
    );

    echo      '<div class="wpep-assessment-question-answer-container-footer">';
    echo        '<span class="wpep-add-question-answer"></span>';
    echo      '</div>';
    echo      '<div class="clear"></div>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';
  }

  private function _display_assessment_question_answer(
    $field_prefix,
    $question_id,
    $question_answer = null,
    $fieldset_class = "wpep-assessment-question-answer-item"
  ) {
    global $wp_version;

    $current_answer_id     = ( isset( $question_answer->id ) ? $question_answer->id : uniqid('answer_') );
    $current_answer_prefix = $field_prefix . '[question][' . $question_id . '][answer][' . $current_answer_id . ']';
    $is_correct = ( isset( $question_answer->is_correct ) ? $question_answer->is_correct : 0 );

    $fieldset_class .= ( $is_correct ? ' wpep-assessment-question-answer-item-correct' : ' wpep-assessment-question-answer-item-incorrect' );

    echo '<fieldset class="' . $fieldset_class . '" data-question-answer-id="' . $current_answer_id . '" data-field-group-prefix="' . $current_answer_prefix . '">';
    echo    '<div class="header-section">';

    $this->display_hidden_field(
      'is_correct',
      $is_correct,
      $current_answer_prefix . '[is_correct]',
      'wpep-assessment-question-answer-is-correct'
    );

    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    echo      '<span class="element-set-correct"></span>';
    echo      '<span class="element-set-incorrect"></span>';
    $this->display_text_field(
      'title',
      $current_answer_prefix . '[title]',
      ( isset( $question_answer->title ) ? $question_answer->title : '' ),
      "wpep-question-answer-title",
      __( "New Answer", "wpep" )
    );
    echo      '<span class="element-delete"></span>';
    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo '</fieldset>';
  }

  public function handle_assessment_save_functionality( $course_post_id, $entity_id_map = [] ) {
    $this->handle_assessment_delete_functionality();
    
    if( isset( $_POST['wpep_assessment'] ) ) {
      foreach( $_POST['wpep_assessment'] as $assessment_section_id => $assessment_information ) {
        if( !isset( $assessment_information['title'] ) || $assessment_information['title'] == '' )
          continue;

        if( !is_numeric( $assessment_section_id ) && isset( $entity_id_map['section'][$assessment_section_id] ) )
          $assessment_section_id = $entity_id_map['section'][$assessment_section_id];

        // If the assessment was not detected in the external_id_map
        if( !isset( $entity_id_map['section'][$assessment_section_id] ) && $assessment_section_id != 0 ) {
          // Delete the assessment if it existed ?
          continue;
        }

        $assessment_section_id = ( isset( $entity_id_map['section'][$assessment_section_id] ) ? $entity_id_map['section'][$assessment_section_id] : 0 );
        $assessment_id         = wpep_get_course_assessment_id( $course_post_id, $assessment_section_id );
        $assessment_status     = ( isset( $assessment_information['post_status'] ) ? $assessment_information['post_status'] : 'publish' );

        if( $assessment_id == 0 ) {
          $assessment_id = wp_insert_post( [
            'post_title'      => $assessment_information['title'],
            'post_type'       => WPEP_POST_TYPE_ASSESSMENT,
            'post_status'     => $assessment_status,
            'comment_status'  => 'closed'
          ] );
        } else {
          wp_update_post( [
            'ID'          => $assessment_id,
            'post_status' => $assessment_status,
            'post_title'  => $assessment_information['title']
          ] );
        }

        update_post_meta( $assessment_id, 'wpep_course_id', $course_post_id );
        update_post_meta( $assessment_id, 'wpep_section_id', $assessment_section_id );

        foreach( $assessment_information['meta_information'] as $assessment_meta_key => $assessment_meta_value )
          update_post_meta( $assessment_id, $assessment_meta_key, $assessment_meta_value );

        if( !isset( $assessment_information['question'] ) )
          continue;

        $entity_id_map = $this->save_assessment_questions( $assessment_information['question'], $assessment_id, $entity_id_map );
      }
    }

    return $entity_id_map;
  }

  public function handle_assessment_delete_functionality() {
    if( isset( $_POST['wpep_deleted_question'] ) ) {
      foreach( $_POST['wpep_deleted_question'] as $question_id ) {
        if( !is_numeric( $question_id ) )
          continue;

        EntityAssessment::instance()->delete_question_by_id( $question_id );
      }
    }

    if( isset( $_POST['wpep_deleted_question_answer'] ) ) {
      foreach( $_POST['wpep_deleted_question_answer'] as $question_answer_id ) {
        if( !is_numeric( $question_answer_id ) )
          continue;

        EntityAssessment::instance()->delete_answer_by_id( $question_answer_id );
      }
    }
  }

  public function save_assessment_questions( $assessment_questions, $assessment_id, $entity_id_map = [] ) {
    $question_position = 1;

    foreach( $assessment_questions as $question_id => $question_information ) {
      if( !isset( $question_information['title'] ) )
        continue;

      if( !is_numeric( $question_id ) && isset( $entity_id_map['question'][$question_id] ) )
        $question_id = $entity_id_map['question'][$question_id];

      $current_information = [
        'post_id'     => $assessment_id,
        'title'       => stripcslashes(
          $question_information['title'] != ''
            ? $question_information['title']
            : __( "New Question", "wpep" )
        ),
        'type'        => ( isset( $question_information['type'] ) ? $question_information['type'] : wpep_get_assessment_field_type_default() ),
        'order'       => ( isset( $question_information['order'] ) ? intval( $question_information['order'] ) : $question_position ),
        'weight'      => ( isset( $question_information['weight'] ) ? intval( $question_information['weight'] ) : 1 ),
      ];

      if( isset( $question_information['content'] ) )
        $current_information['content'] = stripcslashes( $question_information['content'] );

      if( is_numeric( $question_id ) ) {

        EntityAssessment::instance()->update_question( $current_information, [ 'id' => $question_id ] );

        $entity_id_map['question'][$question_id] = $question_id;

      } else {
        $insert_id = EntityAssessment::instance()->insert_question( $current_information );

        $entity_id_map['question'][$question_id] = $insert_id;

        $question_id = $insert_id;
      }

      $question_position++;

      if( !isset( $question_information['answer'] ) )
        continue;

      $current_answer_position = 0;

      foreach( $question_information['answer'] as $question_answer_id => $question_answer_information ) {
        if( !isset( $question_answer_information['title'] ) )
          continue;

        $question_answer_information['title'] = stripcslashes( $question_answer_information['title'] );

        if( $question_answer_information['title'] === '' )
          continue;

        if( !is_numeric( $question_id ) && isset( $entity_id_map['question_answer'][$question_id] ) )
          $question_answer_id = $entity_id_map['question_answer'][$question_answer_id];

        $current_information = [
          'post_id'     => $assessment_id,
          'question_id' => $question_id,
          'title'       => $question_answer_information['title'],
          'is_correct'  => ( isset( $question_answer_information['is_correct'] ) ? $question_answer_information['is_correct'] : 0 ),
          'order'       => $current_answer_position
        ];

        if( isset( $lesson_information['content'] ) )
          $current_information['content'] = stripcslashes($lesson_information['content']);

        if( is_numeric( $question_answer_id ) ) {
          EntityAssessment::instance()->update_question_answer( $current_information, [ "id" => $question_answer_id ] );

          $entity_id_map['question_answer'][$question_answer_id] = $question_answer_id;
        } else {
          $entity_id_map['question_answer'][$question_answer_id] = EntityAssessment::instance()->insert_question_answer( $current_information );
        }

        $current_answer_position++;
      }
    }

    return $entity_id_map;
  }

}
