<?php

namespace WPEP\Admin\Container;

use WPEP\Entity\Course as EntityCourse;

trait CourseBuilder {

  public $enable_downloads   = true;
  public $enable_layout_content = false;
  public $enable_assessments    = false;
  public $enable_timing_settings= false;
  public $enable_member_magnet                = false;
  public $enable_presentation_card_settings   = true;
  public $enable_presentation_button_settings = true;
  public $enable_prerequisites_settings       = true;
  public $enable_sidebar_settings             = true;
  public $enable_learning_paths               = true;
  public $new_course_sections   = 3;
  public $new_course_lessons    = 5;

  public $section_course_type = [];

  public function __construct() {
    $this->section_course_type    = [
      'course_management'  => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_VIDEO,
      'course_video'       => WPEP_COURSE_TYPE_ONE_OFF,
      'course_downloads'   => WPEP_COURSE_TYPE_ONE_OFF,
      'learning_path'      => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP,
      'section_checkbox'   => WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
      'lesson_video'       => WPEP_COURSE_TYPE_VIDEO
    ];

    $this->enable_downloads       = wpep_get_setting_flag( 'enable-course-deliverables' );
    $this->enable_layout_content  = wpep_get_setting_flag( 'course-layout-content-editor' );
    $this->enable_timing_settings = wpep_get_setting_flag( 'enable-course-timing-option' );
    $this->enable_assessments     = wpep_assessments_enabled();

    $this->enable_member_magnet   = wpep_get_setting_flag( 'enable-post-member-magnet-option' );

    $this->new_course_sections    = wpep_get_setting( 'admin-course-preset-sections' );
    $this->new_course_lessons     = wpep_get_setting( 'admin-course-preset-lessons' );

    $this->section_course_type    = apply_filters( "wpep_administration_course_builder_section_functionality", $this->section_course_type );

    do_action( 'wpep_administration_course_builder_settings_setup', $this );
  }

  public function display_course_builder_tabs() {
    $screen = get_current_screen();

    echo    '<ul id="wpep-tab-links">';

    do_action( "wpep_administration_course_builder_tab_items_before" );

    echo      '<li class="wpep-tab-course-presentation wpep-tab-link active"><h4>' . __( "Present", "wpep" ) . '</h4></li>';
    echo      '<li class="wpep-tab-course-management wpep-tab-link" 
                   data-wpep-course-type="' . $this->section_course_type[ 'course_management' ] . '"
                   data-hidden-callback="#wpep-tab-links > .wpep-tab-course-presentation"
                   data-visible-callback="#wpep-tab-links > .wpep-tab-course-management"
               ><h4>' . __( "Build", "wpep" ) . '</h4></li>';

    if( $this->enable_layout_content )
      echo      '<li class="wpep-tab-link wpep-tab-course-layout"><h4>' . __( "Layout", "wpep" ) . '</h4></li>';

    if( $this->enable_assessments )
      echo      '<li class="wpep-tab-link wpep-tab-course-final-assessment"><h4>' . __( "Assess", "wpep" ) . '</h4></li>';

    do_action( "wpep_administration_course_builder_tab_items_after" );

    echo    '</ul>';

    if ( $screen->action != 'add' ) {
      echo    '<span class="wpep-post-quick-save" data-text="' . __( "Quick Save", "wpep" ) . '"></span>';
    }

    echo    '<span class="wpep-toggle-large-builder"></span>';
    echo    '<div class="clear"></div>';
  }

  public function display_course_builder_content() {
    echo    '<div id="wpep-tabs">';

    do_action( "wpep_administration_course_builder_content_before" );

    $this->_course_builder_tab_presentation();

    $this->_course_builder_tab_manage();

    if( $this->enable_layout_content )
      $this->_builder_tab_layout_content();

    if( $this->enable_assessments ) {
      echo '<div class="wpep-tab-item">';

      $this->display_assessment_for_course( 0, get_the_ID() );

      echo '</div>';
    }

    do_action( "wpep_administration_course_builder_content_after" );

    echo    '</div>';
  }

  private function _course_builder_tab_presentation() {
    echo '<div class="wpep-tab-item wpep-tab-course-presentation wpep-tab-course-description-container">';
    echo    '<h2>' . __( "Description", "wpep" ) . '</h2>';
    echo    '<div class="wpep-form-group wpep-course-description-section">';
              wpep_editor(
                get_post_meta( get_the_ID(), 'wpep_course_description', true ),
                'wpep_course_description'
              );
    echo    '</div>';

    echo    '<h2 data-wpep-course-type="' . $this->section_course_type['course_video'] . '">' . __( "Video Embed", "wpep" ) . '</h2>';
    echo    '<div class="wpep-form-group textarea wpep-course-video-section" data-wpep-course-type="' . $this->section_course_type['course_video'] . '">';
    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Video URL", "wpep" ) . '</label>';
    $this->display_text_field( 'wpep_course_video' );
    echo        '</div>';
    if( wpep_get_setting_flag( 'course-basic-enable-video-description' ) ) {
      echo '<div class="wpep-form-group-row">';
      echo '<label>' . __("Video Description", "wpep") . '</label>';
      $this->display_editor_field('wpep_course_video_description');
      echo '</div>';
    }

    echo        '<div class="wpep-form-group-row">';
    $this->display_attributes_field_jump_links(
      '_wpep_course_video_jump_links',
      false
    );
    echo        '</div>';
    echo    '</div>';

    if( $this->enable_downloads ) {
      echo    '<h2 data-wpep-course-type="' . $this->section_course_type['course_downloads'] . '">' . __( "Available Deliverables", "wpep" ) . '</h2>';
      echo    '<div class="wpep-form-group wpep-course-downloads-section" data-wpep-course-type="' . $this->section_course_type['course_downloads'] . '">';
      echo        '<div class="wpep-form-group-row">';
      $this->display_attributes_field_downloads(
        '_wpep_course_downloads',
        false
      );
      echo        '</div>';
      echo    '</div>';
    }

    if( $this->enable_presentation_card_settings )
      $this->_course_presentation_settings();

    echo '</div>';
  }

  private function _course_presentation_settings() {
    echo    '<div class="wpep-form-group wpep-course-presentation-card">';
    echo       '<h2>' . __( "Content Tab - Presentation Card", "wpep" ) . '</h2>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Description", "wpep" ) . '</label>';
    $this->display_textarea_field(
      'wpep_course_excerpt',
      false,
      '50px',
      wpep_course_excerpt( get_the_ID(), true )
    );
    echo          '<p class="description">' . __( "Leaving this empty will automatically generate one based on the description", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Ribbon Text", "wpep" ) . '</label>';
    $this->display_text_field(
      'wpep_course_ribbon_text'
    );
    echo          '<p class="description">' . __( "Display an Ribbon on the Top Right Side of the Course Image.", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Show or Hide this Course", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_show_on_index',
      [
        1 => __('Show', "wpep"),
        0 => __('Hide', "wpep")
      ],
      false,
      null,
      'wpep-select-field',
      1
    );
    echo          '<p class="description">' . __( "Show or Hide this Course from your Course Index Page.", "wpep" ) . '</p>';
    echo       '</div>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Coming Soon", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_coming_soon',
      wpep_flag_setting_field_options(),
      false
    );
    echo          '<p class="description">' . __( "The Start Course button on the Index Page and Widgets will change to Coming Soon and the link will be removed, the course access will not be restricted.", "wpep" ) . '</p>';
    echo        '</div>';
    echo       '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Top Border Color", "wpep" ) . '</label>';
    $this->display_color_field(
      'identity_color'
    );
    echo          '<p class="description">' . __( 'Optionally select a color for this course, it will be used as the border color on the index page.', "wpep" ) . '</p>';
    echo        '</div>';

    if( $this->enable_presentation_button_settings ) {
      echo       '<h3>' . __( "Button Settings", "wpep" ) . '</h3>';
      echo       '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Text", "wpep" ) . '</label>';
      $this->display_text_field(
        'wpep_post_button_text',
        false,
        null,
        false,
        wpep_get_setting_translated( 'opt-course-start-text' )
      );
      echo          '<p class="description">' . __( 'The text on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo       '</div>';
      echo       '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Color", "wpep" ) . '</label>';
      $this->display_color_field(
        'wpep_post_button_color'
      );
      echo          '<p class="description">' . __( 'A different color on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo        '</div>';
      echo       '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Text Color", "wpep" ) . '</label>';
      $this->display_color_field(
        'wpep_post_button_text_color'
      );
      echo          '<p class="description">' . __( 'A different text color on the Start Course Button, leaving this empty will default to the one from settings.', "wpep" ) . '</p>';
      echo        '</div>';
      echo       '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Custom URL", "wpep" ) . '</label>';
      $this->display_text_field(
        'wpep_post_alternative_link',
        false,
        null,
        false,
        __( "Leave this empty to ignore this feature.", "wpep" )
      );
      echo          '<p class="description">' . __( "URL users will be redirected to when clicking the Start Button on the Course Tab. Links must start with http:// or https://.", "wpep" ) . '</p>';
      echo        '</div>';
    }

    echo    '</div>';
  }

  private function _course_builder_tab_manage() {
    echo '<div class="wpep-tab-item wpep-tab-course-management" data-wpep-course-type="' . $this->section_course_type[ 'course_management' ] . '">';
    echo    '<div id="wpep-section-group-container">';

    $course_sections = EntityCourse::instance()->get_with_lessons();

    if( !empty( $course_sections ) ) {
      foreach( $course_sections as $course_section )
        $this->manage_course_section( $course_section );
    } else {
      for( $i = 1; $i <= $this->new_course_sections; $i++ )
        $this->manage_course_section();
    }

    echo '</div>';

    echo    '<div class="wpep-button-row">';
    echo      '<button type="button" class="button button-primary wpep-add-section">' . __( "Add Section", "wpep") . '</button>';
    echo    '</div>';

    echo '</div>';
  }

  private function _builder_tab_layout_content() {
    global $post;

    echo '<div id="wpep-layout-content-editor-container" class="wpep-tab-item">';

    echo '<p>' . __( "Personalize the Course Layout using page builders or just the editor content.", "wpep" ) . '</p>';
    echo '<p>' . __( "Using the [wpep_the_course] shortcode we'll detect where to place the course content.", "wpep" ) . '</p>';
    echo '<p>' . __( "Omitting the [wpep_the_course] shortcode will cause the layout content builder to be ignored, and just display the course.", "wpep" ) . '</p>';

    if( strpos( $post->post_content, '[wpep_the_course]' ) === false ) {
      echo '<div class="notice notice-warning inline">
                <p>' . __( "Please use the [wpep_the_course] shortcode where you want to display the Course Content.", "wpep" ) . '</p>
              </div>';
    } else {
      echo '<div class="notice notice-success inline">
                <p>' . __( "[wpep_the_course] shortcode has been detected, do not remove it.", "wpep" ) . '</p>
              </div>';
    }


    echo '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . '">';

    $this->display_table_start();
    $this->display_table_checkbox_field( __( "Hide Course Header", "wpep" ), 'wpep_course_layout_hide_header' );
    $this->display_table_end();

    echo '</div>';


    echo '</div>';
  }

  private function manage_course_section( $course_section = null ) {
    $course_section = ( $course_section === null ? new \stdClass() : $course_section );

    if( !isset( $course_section->options ) )
      $course_section->options = new \stdClass();

    global $wp_version;

    $course_section_id     = isset( $course_section->id ) ? $course_section->id : uniqid('section_');
    $current_course_prefix = 'wpep_course[' . $course_section_id . ']';

    echo '<fieldset class="wpep-section-group-item" data-section-id="' . $course_section_id . '" data-field-group-prefix="' . $current_course_prefix . '">';

    echo    '<div class="header-section">';
    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    echo      '<input type="text" name="' . $current_course_prefix . '[title]" class="wpep-section-title" placeholder="' . __( 'New Section', 'wpep' ) . '" value="' . ( isset( $course_section->title ) ? $course_section->title : '' ) . '">';
    echo      '<span class="element-delete"></span>';
    echo      '<span class="element-settings"></span>';
    echo      ( $this->enable_timing_settings ? '<span class="element-time-options"></span>' : '' );
    echo      ( $this->enable_assessments ? '<span class="element-assessment"></span>' : '' );
    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';

    if( $this->enable_timing_settings ) {
      echo      '<div class="section-timing-settings-container">';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Available After", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_course_prefix . '[time_action_display_from]' ,
        ( isset( $course_section->time_action_display_from ) ? $course_section->time_action_display_from : 0 )
      );

      echo          '<p class="description">' . __( "How long it takes for this section to be available after the student has first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Available For", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_course_prefix . '[time_action_display_until]' ,
        ( isset( $course_section->time_action_display_until ) ? $course_section->time_action_display_until : 0 )
      );

      echo          '<p class="description">' . __( "How long will this section be available after the student has first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Notify After", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_course_prefix . '[time_action_notify]' ,
        ( isset( $course_section->time_action_notify ) ? $course_section->time_action_notify : 0 )
      );

      echo          '<p class="description">' . __( "Report students that have not completed this section during this timespan, after they have first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo      '</div>';
    }

    echo      '<div class="section-settings-container">';

    echo      '<div data-wpep-course-type="' . $this->section_course_type[ 'learning_path' ] . '" class="wpep-form-group" style="display:none;">';

    echo        '<h3 data-wpep-description-path="^ .wpep-section-description-description">' . __( "Section Description", "wpep" ) . '</h3>';
    echo        '<div class="wpep-section-description-description" style="display:none;">';
    echo          '<p>' . __( "If you're not using the learning path options, this field can be used simply as a section description.", "wpep" ) . '</p>';
    echo          '<p>' . __( "If you're using the learning path options, this field can address a question or instructions on chosing the learning path.", "wpep" ) . '</p>';
    echo        '</div>';
    echo      '</div>';

    echo      '<div class="wpep-form-group textarea primary-content-editor">';

    if( wpep_get_setting_flag( 'lesson-wp-editor-preload' ) ) {
      wpep_editor(
        ( isset( $course_section->content ) ? $course_section->content : '' ),
        'wpep-section-' . ( isset( $course_section->id ) ? $course_section->id : str_replace( 'section_', '', $course_section_id ) ),
        $current_course_prefix . '[content]'
      );
    }

    echo      '</div>';


    echo      '<div data-wpep-course-type="' . $this->section_course_type[ 'learning_path' ] . '" class="wpep-form-group" style="display:none;">';

    if( $this->enable_learning_paths ) {
      echo '<h3 data-wpep-description-path="^ .wpep-learning-path-options-description">' . __("Learning Path Options") . '</h3>';
      echo '<div class="wpep-learning-path-options-description" style="display:none;">';
      echo '<p>' . __("Allow your students to get personalized section content based on their needs. Before starting a section students are required to choose from a set of answers, based on their decision, they'll follow a personalized learning path.", "wpep") . '</p>';
      echo '<p>' . __("You need to setup at least 2 Learning Paths for this feature to work.", "wpep") . '</p>';
      echo '</div>';

      echo '<div class="wpep-form-group-row">';
      if (isset($course_section->options->question_answers) && is_array($course_section->options->question_answers) && !empty($course_section->options->question_answers)) {

        foreach ($course_section->options->question_answers as $conditional_logic_answer)
          $this->display_text_field(
            'question_answers',
            $current_course_prefix . '[option][question_answers][]',
            $conditional_logic_answer,
            'wpep-section-conditional-field',
            __("Leave empty to ignore", "wpep")
          );

        if (count($course_section->options->question_answers) == 1)
          $this->display_text_field(
            'question_answers',
            $current_course_prefix . '[option][question_answers][]',
            '',
            'wpep-section-conditional-field',
            __("Please complete...", "wpep")
          );

      } else {
        $this->display_text_field(
          'question_answers',
          $current_course_prefix . '[option][question_answers][]',
          '',
          'wpep-section-conditional-field',
          __("Leave empty to ignore", "wpep")
        );

        $this->display_text_field(
          'question_answers',
          $current_course_prefix . '[option][question_answers][]',
          '',
          'wpep-section-conditional-field',
          __("Leave empty to ignore", "wpep")
        );
      }
      echo '</div>';
      echo '<div class="wpep-button-row">';
      echo '<button class="button button-primary wpep-add-section-conditional-field">' . __("Add Learning Path", "wpep") . '</button>';
      echo '</div>';
    }

    echo          '<div class="wpep-form-group-row" data-wpep-course-type="' . $this->section_course_type['section_checkbox'] . '">';
    echo            '<h3>' . __( "Misc Options", "wpep" ) . '</h3>';
    echo            '<label>' . __( "Section Checkbox", "wpep" ) . '</label>';

    $this->display_checkbox_field(
      'display_checkbox',
      ( isset( $course_section->options->display_checkbox ) ? $course_section->options->display_checkbox : 1 ),
      $current_course_prefix . '[option][display_checkbox]'
    );

    echo            '<p class="description">' . __( "Optionally hide the section checkbox, this will work perfectly if you want to connect sections based on learning paths.", "wpep" ) . '</p>';
    echo        '</div>';

    echo     '</div>';

    do_action( "wpep_administration_section_option_field", $current_course_prefix . '[option]', ( isset( $course_section->options ) ? $course_section->options : [] ) );

    echo      '</div>';

    if( $this->enable_assessments ) {
      echo      '<div class="section-assessment-container">';

      $this->display_assessment_for_course( $course_section_id, get_the_ID()  );

      echo      '</div>';
    }

    echo      '<div class="wpep-section-lesson-container">';

    if( !is_numeric( $course_section_id ) ) {
      for( $i = 1; $i <= $this->new_course_lessons; $i++ )
        $this->manage_course_section_lesson( $course_section_id );

    } else if( isset( $course_section->lessons ) && !empty( $course_section->lessons ) ) {
      foreach( $course_section->lessons as $section_lesson )
        $this->manage_course_section_lesson( $course_section_id, $section_lesson, $course_section );
    }

    $this->manage_course_section_lesson(
      $course_section_id, null, null,
      'wpep-section-lesson-item wpep-section-lesson-item-hidden',
      'wpep_course_template',
      false
    );

    echo      '</div>';
    echo      '<div class="wpep-section-lesson-container-footer">';
    echo        '<span class="wpep-add-lesson"></span>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';

  }

  private function manage_course_section_lesson(
    $section_id,
    $section_lesson = null,
    $course_section = null,
    $fieldset_class = 'wpep-section-lesson-item',
    $field_prefix   = 'wpep_course',
    $editor_preload = null
  ) {
    $section_lesson = ( $section_lesson === null ? new \stdClass() : $section_lesson );
    $course_section = ( $course_section === null ? new \stdClass() : $course_section );
    $editor_preload = ( $editor_preload === null ? wpep_get_setting_flag( 'lesson-wp-editor-preload' ) : $editor_preload );

    if( !isset( $section_lesson->options ) )
      $section_lesson->options = new \stdClass();
    else if( is_array( $section_lesson->options ) )
      $section_lesson->options = (object) $section_lesson->options;

    global $wp_version;

    $current_lesson_id     = ( isset( $section_lesson->id ) ? $section_lesson->id : uniqid('lesson_') );
    $current_lesson_prefix = $field_prefix . '[' . $section_id . '][lesson]' . '[' . $current_lesson_id . ']';
    $has_learning_path     = ( isset( $course_section->options->question_answers ) && is_array( $course_section->options->question_answers ) && count( $course_section->options->question_answers ) >= 2 );

    $section_lesson->options->question_answer = ( isset( $section_lesson->options->question_answer ) ? $section_lesson->options->question_answer : -1 );

    echo '<fieldset class="' . $fieldset_class . '" data-lesson-id="' . $current_lesson_id . '" data-field-group-prefix="' . $current_lesson_prefix . '">';
    echo    '<div class="header-section">';
    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    echo      '<span class="learning-path-index wpep-lesson-conditional-setup" 
                     style="' . ( $has_learning_path ? 'display:inline-block;' : 'display:none' ) . '">' .
      ( $has_learning_path ? ( $section_lesson->options->question_answer == -1 ? 'All' : ( "Q : " . ( $section_lesson->options->question_answer + 1 ) ) ) : '' ) .
      '</span>';
    echo        '<input type="text"
                        name="' . $current_lesson_prefix . '[title]' . '"
                        class="wpep-lesson-title"
                        value="' . ( isset( $section_lesson->title ) ? $section_lesson->title : '' ) . '"
                        placeholder="' . __( "New Lesson", "wpep" ) . '">';
    echo      '<span class="element-delete"></span>';
    echo      '<span class="element-duplicate"></span>';
    echo      ( $this->enable_timing_settings ? '<span class="element-time-options"></span>' : '' );
    echo      ( $this->enable_downloads ? '<span class="element-download-options"></span>' : '' );
    echo      '<span class="element-open"></span>';
    echo      '<span class="element-close"></span>';
    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';

    if( $this->enable_timing_settings ) {
      echo      '<div class="lesson-timing-settings-container">';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Available After", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_lesson_prefix . '[time_action_display_from]' ,
        ( isset( $section_lesson->time_action_display_from ) ? $section_lesson->time_action_display_from : 0 )
      );

      echo          '<p class="description">' . __( "How long it takes for this lesson to be available after the student has first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Available For", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_lesson_prefix . '[time_action_display_until]' ,
        ( isset( $section_lesson->time_action_display_until ) ? $section_lesson->time_action_display_until : 0 )
      );

      echo          '<p class="description">' . __( "How long will this lesson be available after the student has first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo        '<div class="wpep-form-group-row">';
      echo          '<label>' . __( "Notify After", "wpep" ) . '</label>';

      $this->display_time_field(
        'time_action_display_from',
        $current_lesson_prefix . '[time_action_notify]' ,
        ( isset( $section_lesson->time_action_notify ) ? $section_lesson->time_action_notify : 0 )
      );

      echo          '<p class="description">' . __( "Report students that have not completed this lesson during this timespan, after they have first seen the course.", "wpep" ) . '</p>';
      echo       '</div>';

      echo      '</div>';
    }

    if( $this->enable_downloads ) {
      echo      '<div class="lesson-downloads-settings-container">';
      echo        '<div class="wpep-form-group wpep-course-downloads-section">';
      echo          '<div class="wpep-form-group-row">';
      $this->display_attributes_field_downloads(
        'course_lesson_downloads',
        $current_lesson_prefix . '[option][downloads_list]',
        isset( $section_lesson->options->downloads_list ) ? $section_lesson->options->downloads_list : []
      );
      echo          '</div>';
      echo        '</div>';
      echo      '</div>';
    }

    echo      '<div class="lesson-content-settings-container">';
    do_action( "wpep_administration_lesson_option_field", $current_lesson_prefix . '[option]', ( isset( $section_lesson->options ) ? $section_lesson->options : [] ) );

    echo      '<div data-wpep-course-type="' . $this->section_course_type['lesson_video'] . '" class="wpep-form-group wpep-form-group-video-details" style="display:none;">';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Video URL", "wpep" ) . '</label>';

    $this->display_text_field(
      'video_embed_source',
      $current_lesson_prefix . '[option][video_embed_source]' ,
      ( isset( $section_lesson->options->video_embed_source ) ? $section_lesson->options->video_embed_source : '' )
    );

    echo       '</div>';
    echo       '<div class="wpep-form-group-row wpep-form-group-row-video-image">';

    $this->display_image_field(
      'video_embed_image',
      __( "Video Image", "wpep" ),
      $current_lesson_prefix . '[option][video_embed_image]' ,
      ( isset( $section_lesson->options->video_embed_image ) ? $section_lesson->options->video_embed_image : '' )
    );

    echo          '<p class="description">' . __( "Chose an image for this video that will appear on the navigation bar of videos.", "wpep" ) . '</p>';
    echo        '</div>';

    echo        '<h3>' . __( "Optional Course Description") . '</h3>';
    echo        '<p>' . __( "Display instructions under the video, in case you want to offer file downloads or extra guidance, this is where you should place it.", "wpep" ) . '</p>';
    echo      '</div>';

    echo      '<div class="wpep-lesson-conditional-setup-container" data-wpep-course-type="' . $this->section_course_type[ 'learning_path' ] . '">';
    echo        '<div class="wpep-form-group wpep-lesson-conditional-setup" style="display:' . ( $has_learning_path ? 'block' : 'none' ). '">';
    echo          '<div class="wpep-form-group-row">';
    echo            '<label>' . __( "Learning Path", "wpep" ) . '</label>';

    $this->display_select_field(
      'question_answer',
      (
        [
          -1 => __( "Available for any answer", "wpep" )
        ] +
        ( isset( $course_section->options->question_answers ) && is_array( $course_section->options->question_answers ) ? $course_section->options->question_answers : [] )
      ),
      $current_lesson_prefix . '[option][question_answer]',
      $section_lesson->options->question_answer
    );

    echo            '<p class="description">' . __( "This section has learning path configured, chose which learning path is required to display this lesson.", "wpep" ) . '</p>';
    echo        '</div>';
    echo        '</div>';
    echo      '</div>';

    echo      '<div class="wpep-form-group textarea primary-content-editor">';

    if( $editor_preload ) {
      wpep_editor(
        ( isset( $section_lesson->content ) ? $section_lesson->content : '' ),
        'lesson-' . ( isset( $section_lesson->id ) ? $section_lesson->id : str_replace( 'lesson_', '', $current_lesson_id ) ),
        $current_lesson_prefix . '[content]'
      );
    }

    echo      '</div>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';

  }

  public function handle_course_save_functionality( $course_post_id, $entity_id_map = [] ) {
    global $wpdb;

    if( isset( $_POST['wpep_deleted_lesson'] ) ) {

      foreach( $_POST['wpep_deleted_lesson'] as $lesson_id ) {
        if( !is_numeric( $lesson_id ) )
          continue;

        $wpdb->delete( $wpdb->prefix . 'wpep_section_lesson', [ 'id' => $lesson_id ] );
      }

    }

    if( isset( $_POST['wpep_deleted_section'] ) ) {

      foreach( $_POST['wpep_deleted_section'] as $section_id ) {
        if( !is_numeric( $section_id ) )
          continue;

        $wpdb->delete( $wpdb->prefix . 'wpep_section', [ 'id' => $section_id ] );
        $wpdb->delete( $wpdb->prefix . 'wpep_section_lesson', [ 'section_id' => $section_id ] );
      }

    }

    if( isset( $_POST['wpep_course'] ) ) {
      $has_learning_path = 0;
      $lesson_position   = 1;
      $section_position  = 1;
      foreach( $_POST['wpep_course'] as $section_id => $section_information ) {
        if( !isset( $section_information['title'] ) )
          continue;

        if( !is_numeric( $section_id ) && isset( $entity_id_map['section'][$section_id] ) )
          $section_id = $entity_id_map['section'][$section_id];

        $section_options = isset( $section_information['option'] ) && is_array( $section_information['option'] ) ? $section_information['option'] : [];

        if( isset( $section_options[ 'question_answers' ] ) ) {
          foreach ($section_options['question_answers'] as $question_answer_index => $question_answer_value)
            if (trim($question_answer_value) == '')
              unset($section_options['question_answers'][$question_answer_index]);

          if( count( $section_options[ 'question_answers' ] ) >= 2 )
            $has_learning_path = 1;
        }

        $current_information = [
          'post_id' => $course_post_id,
          'title'   => stripcslashes( $section_information['title'] != '' ? $section_information['title'] : __( "New Section", "wpep" ) ),
          'order'   => ( isset( $section_information['order'] ) ? intval( $section_information['order'] ) : $section_position ),
          'options' => json_encode( $section_options )
        ];

        if( isset( $section_information['time_action_display_from'] ) )
          $current_information['time_action_display_from'] = $section_information['time_action_display_from'];

        if( isset( $section_information['time_action_display_until'] ) )
          $current_information['time_action_display_until'] = $section_information['time_action_display_until'];

        if( isset( $section_information['time_action_notify'] ) )
          $current_information['time_action_notify'] = $section_information['time_action_notify'];

        if( isset( $section_information['content'] ) )
          $current_information['content'] = stripcslashes( $section_information['content'] );

        if( is_numeric( $section_id ) ) {

          $wpdb->update( $wpdb->prefix . 'wpep_section', $current_information, [ 'id' => $section_id ] );

          $entity_id_map['section'][$section_id] = $section_id;

        } else {

          $wpdb->insert( $wpdb->prefix . 'wpep_section', $current_information );

          $entity_id_map['section'][$section_id] = $wpdb->insert_id;

          $section_id = $wpdb->insert_id;
        }

        $section_position++;

        if( !isset( $section_information['lesson'] ) )
          continue;

        foreach( $section_information['lesson'] as $lesson_id => $lesson_information ) {
          if( !isset( $lesson_information['title'] ) )
            continue;

          if( !is_numeric( $lesson_id ) && isset( $entity_id_map['lesson'][$lesson_id] ) )
            $lesson_id = $entity_id_map['lesson'][$lesson_id];

          $current_information = [
            'post_id'     => $course_post_id,
            'section_id'  => $section_id,
            'title'       => stripcslashes( $lesson_information['title'] != '' ? $lesson_information['title'] : __( "New Lesson", "wpep" ) ),
            'order'       => $lesson_position,
            'options'     => json_encode( isset( $lesson_information['option'] ) && is_array( $lesson_information['option'] ) ? $lesson_information['option'] : [] )
          ];
          
          if( isset( $lesson_information['content'] ) )
            $current_information['content'] = stripcslashes($lesson_information['content']);

          if( isset( $lesson_information['time_action_display_from'] ) )
            $current_information['time_action_display_from'] = $lesson_information['time_action_display_from'];

          if( isset( $lesson_information['time_action_display_until'] ) )
            $current_information['time_action_display_until'] = $lesson_information['time_action_display_until'];

          if( isset( $lesson_information['time_action_notify'] ) )
            $current_information['time_action_notify'] = $lesson_information['time_action_notify'];

          if( is_numeric( $lesson_id ) ) {

            $wpdb->update( $wpdb->prefix . 'wpep_section_lesson', $current_information, [ 'id' => $lesson_id ] );

            $entity_id_map['lesson'][$lesson_id] = $lesson_id;

          } else {

            $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson', $current_information );

            $entity_id_map['lesson'][$lesson_id] = $wpdb->insert_id;

          }

          $lesson_position++;
        }
      }

      if( isset( $_POST['wpep_meta_value']['course_type'] )
            && !in_array( $_POST['wpep_meta_value']['course_type'], [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] ) )
        $has_learning_path = 0;

      update_post_meta( $course_post_id, 'wpep_learning_path_enabled', $has_learning_path );

    }

    return $entity_id_map;
  }

}
