<?php

namespace WPEP\Admin\Container\LicenseAndUpdates;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;

class API {

  use WPEP_Container_Configuration;

  /**
   * @var \WPEP\Admin\Container\LicenseAndUpdates
   */
  protected $_controller;

  public function __construct( $controller_instance ) {
    $this->_controller = $controller_instance;
  }

  public function activate_license() {
    $response = wp_remote_get(
      add_query_arg( $this->get_request_params( 'activate_license' ), WPEP_STORE_LINK ),
      $this->get_remote_request_params()
    );

    if ( is_wp_error($response) )
      return false;

    $license_data = json_decode(wp_remote_retrieve_body($response));

    return $license_data;
  }

  public function get_license_status() {
    $response = wp_remote_get(
      add_query_arg( $this->get_request_params( 'check_license' ), WPEP_STORE_LINK ),
      $this->get_remote_request_params()
    );

    if ( is_wp_error($response) )
      return false;

    $license_data = json_decode(wp_remote_retrieve_body($response));

    return $license_data;
  }

  public function get_request_url() {
    return WPEP_STORE_LINK;
  }

  public function get_request_params( $action = 'check_license' ) {
    return [
      'version'    => $this->_controller->get_plugin_version(),
      'edd_action' => $action,
      'license'    => $this->_controller->get_license(),
      'item_id'    => $this->_controller->get_product_id(),
      'item_name'  => $this->_controller->get_product_name(),
      'url'        => urlencode( home_url() ),
      'author'     => 'admin',
    ];
  }

  public function get_remote_request_params() {
    return [
      'timeout'   => 45,
      'sslverify' => false
    ];
  }

}