<?php

namespace WPEP\Admin\Container\LicenseAndUpdates;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;

class Administration {

  use WPEP_Container_Configuration;

  /**
   * @var \WPEP\Admin\Container\LicenseAndUpdates
   */
  protected $_controller;

  public function __construct( $controller_instance ) {
    $this->_controller = $controller_instance;

    if( defined( 'DOING_AJAX' ) && DOING_AJAX )
      return;

    if( $this->_handle_menu_registration() )
      add_action( 'admin_menu', [ $this, '_register_menu' ] );

    add_action( 'admin_init', [ $this, '_admin_init' ] );
  }

  public function _admin_init() {
    if( $this->_controller->is_valid_license() )
      return;

    if( isset( $_GET['page'] ) && ( $_GET['page'] == $this->_controller->get_plugin_alias() . '-migration' || $_GET['page'] == $this->_controller->get_plugin_alias() . '-license' ) )
      return;

    if( isset( $_GET['post_type'] )
      && in_array( $_GET['post_type'], [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_VIDEO, WPEP_POST_TYPE_E_BOOK, WPEP_POST_TYPE_OFFER, WPEP_POST_TYPE_ASSESSMENT ] ) ) {
      wp_redirect( $this->get_configuration_page() );
      exit;
    }

    add_action('admin_notices', [ $this, 'admin_notices' ] );
  }

  public function _register_menu() {
    add_submenu_page(
      'edit.php?post_type=' . WPEP_POST_TYPE_COURSE,
      sprintf( __( "%s License", 'wpep'), $this->_controller->get_product_name() ),
      __( "License", 'wpep'),
      WPEP_CAP_MANAGE_SETTINGS,
      $this->_controller->get_plugin_alias() . "-license",
      [ $this, "display_page_content" ]
    );
  }

  private function _handle_menu_registration() {
    if( isset( $_GET['page'] ) && $_GET['page'] == $this->_controller->get_plugin_alias() . '-license' )
      return true;

    return !$this->_controller->is_valid_license();
  }

  public function display_page_content() {
    echo '<div class="wrap">';
    echo '<h2>' . sprintf( __( "%s License Key", 'wpep' ), $this->_controller->get_product_name() ) . '</h2>';
    echo '<hr/>';

    echo '<p>';
    echo sprintf(
      esc_html__(
        'Your key also unlocks access to automatic updates and support. You can find your key on Your My Account page on the %sCodeIsCode%s website.',
        'wpep'
      ), '<a href="https://codestore.codeiscode.com/my-account">', '</a>'
    );
    echo '</p>';
    echo '<p>' . sprintf( __( "Enter your %s License Key below.", "wpep" ), $this->_controller->get_product_name() ) . '</p>';

    $this->display_table_start();

    $this->display_table_license_verification( $this->_controller->get_license(), $this->_controller->is_valid_license() );

    $this->display_table_end();

    echo '</div>';
  }

  public function get_configuration_page() {
    return admin_url( '/edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=' . $this->_controller->get_plugin_alias() . '-license' );
  }

  public function admin_notices($hook) {
    echo '<div class="error">';
    echo    '<p>';
    echo      sprintf( __( 'Warning! You have not yet activated the license for %s.', "wpep" ), $this->_controller->get_product_name() );
    echo      ' <strong>' . sprintf( __( 'To activate your license %splease click here%s', "wpep" ), '<a href="' . $this->get_configuration_page() . '">', '</a>' ) . '</strong>';
    echo    '</p>';
    echo '</div>';
  }

}