<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Admin\Container\AssessmentBuilder as WPEP_Container_AssessmentBuilder;

/**
 * declare Editor class.
 *
 * @since        1.0
 */

class Assessment {

  use WPEP_Container_AssessmentBuilder;
  
  public function setup() {
    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

    add_action( 'save_post', [ $this, '_save_post_action' ] );
    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 10, 2 );
  }

  public function _save_post_action( $assessment_post_id ) {
    global $post_type;

    if ( !(is_admin() && ($post_type == WPEP_POST_TYPE_ASSESSMENT) ) )
      return;

    $this->_save_post( $assessment_post_id );
  }

  public function _wpep_admin_save_post_ajax( $entity_id_map = [], $post_id ) {
    return $this->_save_post( $post_id, $entity_id_map );
  }

  private function _save_post( $post_id, $entity_id_map = [] ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return $entity_id_map;

    $this->handle_assessment_delete_functionality();

    $entity_id_map = $this->save_assessment_questions( $_POST['wpep_assessment']['question'], $post_id, $entity_id_map );

    foreach( $_POST['wpep_assessment']['meta_information'] as $assessment_meta_key => $assessment_meta_value )
      update_post_meta( $post_id, $assessment_meta_key, $assessment_meta_value );

    return $entity_id_map;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_ASSESSMENT)
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'course'    => __('Course', "wpep"),
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $new_columns['statistics'] = __('Statistics', "wpep");

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_ASSESSMENT )
      return;

    if( $column == 'course' ) {
      $course_id = wpep_get_assessment_course_id( $post->ID );

      echo '<a href="' . admin_url( 'post.php?post=' . $course_id . '&action=edit' ) . '">' . get_the_title( $course_id ) . '</a>';
    }

    if( $column == 'statistics' )
      echo '<a class="button button-primary" href="' . admin_url('edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-statistics&tab=assessments&assessment_id=' . $post->ID ) . '">' . __( "View Statistics", "wpep" )    . '</a>';

  }

}
