<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Assets {

  public function setup() {
    add_action( 'admin_enqueue_scripts', [ $this, 'admin_assets' ], 5 );
  }

  public function admin_assets($hook) {
    $wpep_admin_dependency  = [ 'jquery', 'wp-color-picker', 'jquery-ui-datepicker', 'jquery-ui-sortable', 'jquery-ui-resizable', 'select2', 'clipboard' ];
    $wpep_course_dependency = [ 'jquery', 'wpep-admin-script' ];

    if( wpep_get_setting_flag( 'admin-editor-integration' ) ) {
      add_action( "admin_head", [ $this, '_tiny_mce_admin_head' ], 50 );
      add_filter( 'mce_buttons', [ $this, '_tiny_mce_buttons' ], 50 );
      add_filter( 'mce_external_plugins', [ $this, '_tiny_mce_extensions' ], 50 );
    } else if( wpep_get_setting( 'wpep-editor-setup' ) == 'medium-editor' ) {
      wp_enqueue_script('medium-editor', WPEP_BASE_URL_PATH . 'assets/medium-editor/medium-editor.min.js', [], '5.23.0', true);
      wp_enqueue_style('medium-editor', WPEP_BASE_URL_PATH . 'assets/medium-editor/medium-editor.min.css', '5.23.0' );
      wp_enqueue_style('medium-editor-theme', WPEP_BASE_URL_PATH . 'assets/medium-editor/medium-editor-beagle.min.css', '5.23.0' );

      $wpep_admin_dependency[]  = 'medium-editor';
      $wpep_course_dependency[] = 'medium-editor';
    }

    wp_register_style('wpep-admin-style', WPEP_BASE_URL_PATH . 'assets/admin/style.css', [], WPEP_VERSION );
    wp_enqueue_style('wpep-admin-style');

    wp_enqueue_style( 'wp-color-picker' );

    wp_enqueue_style ('select2', WPEP_BASE_URL_PATH . 'assets/admin/select2.min.css', [], '4.0.3' );
    wp_enqueue_script('select2', WPEP_BASE_URL_PATH . 'assets/admin/select2.full.min.js', [ 'jquery' ], '4.0.3' , true);

    wp_enqueue_script('clipboard', WPEP_BASE_URL_PATH . 'assets/clipboard.min.js', [], '1.5.16' , true);

    /* enqueue Script. */
    wp_enqueue_script( 'wpep-admin-script', WPEP_BASE_URL_PATH . 'assets/admin/primary-min.js', $wpep_admin_dependency, WPEP_VERSION , true);
    wp_localize_script( 'wpep-admin-script', 'wpep_vars', apply_filters( "wpep_admin_script_variables", [
      'external_embed_helper_script'  => WPEP_BASE_URL_PATH . 'assets/front/embed-support-min.js',
      'can_manage_settings'           => intval( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ),
      'post_type_courses'             => WPEP_POST_TYPE_COURSE
    ], $hook ) );
    wp_enqueue_script( 'wpep-admin-script' );

    if( in_array( $hook, [ 'widgets.php', WPEP_POST_TYPE_COURSE . '_page_wpep-settings' ] ) ) {
      wp_enqueue_media();
      wp_enqueue_script( 'media-grid' );
      wp_enqueue_script( 'media' );
      wp_enqueue_style( 'media');
      wp_enqueue_script( 'media-upload' );
      wp_enqueue_script( 'thickbox' );
    } else if ( in_array( $hook, [ "post-new.php", "post.php" ] ) ) {
      global $post;

      if( isset($post->post_type) && in_array( $post->post_type, [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_ASSESSMENT ] ) ) {

        wp_register_script('wpep-course-edit', WPEP_BASE_URL_PATH . 'assets/admin/course-builder-min.js', $wpep_course_dependency, WPEP_VERSION , true);
        wp_localize_script( 'wpep-course-edit', 'wpep_settings', [
          'open_lesson_on_add'    => wpep_get_setting_flag( 'lesson-wp-editor-enabled-new' ),
          'course_save_alert'     => wpep_get_setting_flag( 'admin-course-save-alert' )
        ] );
        wp_enqueue_script( 'wpep-course-edit' );

        if( wpep_get_setting_flag( 'admin-editor-integration' ) ) {
          add_filter('mce_css', [$this, '_tiny_mce_css_within_builder'], 500);
          add_filter('mce_buttons', [$this, '_tiny_mce_buttons_within_builder'], 500);
        }

        add_action( 'admin_footer', [ $this, '_page_loader' ], 500);
      }
    } else if(
      ( isset( $_GET['post_type'] ) && $_GET['post_type'] === WPEP_POST_TYPE_COURSE )
      && ( isset( $_GET['page'] ) && $_GET['page'] == 'wpep-settings' ) ) {
      if( wpep_get_setting_flag( 'admin-editor-integration' ) ) {
        add_filter('mce_css', [$this, '_tiny_mce_css_within_builder'], 500);
      }
    }
  }

  public function _tiny_mce_admin_head() {
    echo "<script type='text/javascript'>
            /* <![CDATA[ */
            var wpep_toolkit_icon = '" . WPEP_BASE_URL_PATH . "assets/img/logo-white-background.png';
                wpep_toolkit_list = " . json_encode( wpep_toolkit_list_map(), JSON_HEX_TAG ) . ";
            /* ]]> */
          </script>";
  }

  public function _tiny_mce_buttons( $buttons ) {
    array_push( $buttons, 'separator', 'wpep_toolkit', 'separator' );

    return $buttons;
  }

  public function _tiny_mce_extensions( $extensions ) {
    $extensions['wpep_toolkit'] = WPEP_BASE_URL_PATH . 'assets/admin/tinymce-toolkit-min.js?ver=' . WPEP_VERSION;

    return $extensions;
  }

  public function _tiny_mce_css_within_builder( $mce_css ) {
    $mce_css .= ',' . WPEP_BASE_URL_PATH . 'assets/admin/wp-editor.css?ver=' . WPEP_VERSION;

    return $mce_css;
  }

  public function _tiny_mce_buttons_within_builder( $buttons ) {
    $options = [ 'fontselect', 'fontsizeselect', 'styleselect' ];

    foreach( $options as $option )
      if( !in_array( $option, $buttons ) )
        $buttons[] = $option;

    $buttons = array_diff( $buttons, ['wp_more', 'fullscreen'] );

    return $buttons;
  }

  public function _page_loader() {
    echo '<div id="wpep-loader-container">';
    echo    wpep_spinner_loader( __( "Loading the Course Builder", "wpep" ) );
    echo '</div>';
  }

}