<?php
namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;
use WPEP\Admin\Container\AssessmentBuilder as WPEP_Container_AssessmentBuilder;
use WPEP\Admin\Container\CourseBuilder as WPEP_Container_CourseBuilder;

/**
 * declare Builder class.
 *
 * @since        1.0
 */

class Builder {

  use WPEP_Container_Configuration,
      WPEP_Container_AssessmentBuilder,
      WPEP_Container_CourseBuilder;

  public function setup() {
    add_action( 'edit_form_after_title', [ $this, 'response' ] );
    $this->set_field_prefix( 'wpep_meta_value' );
  }

  /**
   * course builder response.
   *
   * @access      public
   * @static
   * @since       1.0
   * @return      void
   */
  public function response() {

    global $post_type;

    if ($post_type == WPEP_POST_TYPE_COURSE )
      $this->_response_course();

    if( $post_type == WPEP_POST_TYPE_ASSESSMENT )
      $this->_response_assessment();
  }

  private function _response_course() {
    $section_settings = '';

    foreach( $this->section_course_type as $key => $value )
      $section_settings .= ' data-option-' . $key . '="' . $value . '"';

    $current_primary_tab = intval( get_post_meta( get_the_ID(), 'wpep_current_primary_tab', true ) );

    echo '<div id="wpep_builder"' . $section_settings . '>';
    echo    '<input type="hidden" id="wpep_current_primary_tab" name="wpep_current_primary_tab" value="' . $current_primary_tab . '"/>';

    $this->display_course_builder_tabs();
    $this->display_course_builder_content();

    echo '</div>';
  }

  private function _response_assessment() {
    echo '<div id="wpep_builder" class="wpep-assessment-builder-wrapper">';

    $this->display_assessment_setup_functionality( get_the_ID(), 'wpep_assessment', true );

    echo '</div>';
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    global $post;

    $field_value = get_post_meta( $post->ID, $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

}