<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

/**
 * declare Editor class.
 *
 * @since        1.0
 */

class EBook {

  public function setup() {
    add_filter('wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );
    add_action('save_post', [ $this, '_save_post' ] );
  }

  public function _save_post( $course_post_id ) {
    global $post_type;

    if ( !(is_admin() && ( $post_type == WPEP_POST_TYPE_E_BOOK ) ) )
      return;

    if( get_post_type( $course_post_id ) != WPEP_POST_TYPE_E_BOOK )
      return;

    if( intval( get_post_meta( $course_post_id, 'wpep_order', true ) ) == false )
      update_post_meta( $course_post_id, 'wpep_order', 0 );
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    $ebook_meta_boxes = [];

    $ebook_meta_boxes['wpep-ebook-settings'] = [
      'id'         => 'wpep_ebook_settings',
      'title'      => __( 'EBook Settings', "wpep" ),
      'post_types' => [ WPEP_POST_TYPE_E_BOOK ],
      'context'    => 'normal',
      'priority'   => 'low',
      'fields'     => [
        [
          'name' => __( "Enter the Download URL for you Ebook here", "wpep" ),
          'id'   => '_ebook_link',
          'type' => 'text',
        ],
        [
          'name'    => __('Button Text', "wpep"),
          'id'      => 'wpep_post_button_text',
          'type'    => 'text',
          'desc'    => __( 'The Button Text on the Index Page, leave this empty for default behaviour.', "wpep" )
        ],
        [
          'name'    => __('Button Color', "wpep"),
          'id'      => 'wpep_post_button_color',
          'type'    => 'colorpicker',
          'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
        ],
        [
          'name'    => __('Button Text Color', "wpep"),
          'id'      => 'wpep_post_button_text_color',
          'type'    => 'colorpicker',
          'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
        ],
      ]
    ];

    $ebook_meta_boxes = apply_filters( "wpep_administration_ebook_metaboxes", $ebook_meta_boxes );

    return $meta_boxes + $ebook_meta_boxes;
  }

}