<?php

namespace WPEP\Admin\Coordinator\LearningCenter;

use WPEP\Controller as WPEP_Controller;

class Content {

  public function display() {
    $information = $this->information();

    if( !is_array( $information ) ) {
      echo '<div class="error notice">';
      echo  '<p>' . __( "Please refresh the page, if this issue persists, please contact support.", 'wpep') . '</p>';
      echo '</div>';

      return;
    }

    if( !isset( $information['block_list'] ) ) {
      echo '<div class="success notice">';
      echo '<p>' . __("It seems like there's no content available right now, please check back later.", 'wpep') . '</p>';
      echo '</div>';

      return;
    }

    $current_tab = isset( $_GET['current-tab'] ) ? $_GET['current-tab'] : false;

    if( !isset( $information[ 'block_list' ][ $current_tab ] ) )
      $current_tab = key( $information[ 'block_list' ] );

    echo '<div class="section-container">';
    echo  '<div class="right-section">';
    $this->_display_primary( $information['block_list'], $current_tab );
    echo  '</div>';

    echo  '<div class="left-section">';
    $this->_display_sidebar( $information['block_list'], $current_tab );
    echo  '</div>';
    echo '</div>';
  }

  private function _display_sidebar( $block_list, $current_tab = false  ) {
    echo '<ul>';

    foreach( $block_list as $block_index => $block_item ) {
      echo '<li class="' . ( $block_index == $current_tab ? 'wpep-active' : 'wpep-not-active' ) . '"><a href="' . admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-learning-center&tab=content&current-tab=' . $block_index ) . '">' . $block_item['title'] . '</a></li>';
    }

    echo '</ul>';
  }

  private function _display_primary( $block_list, $current_tab = false ) {
    $information_block = isset( $block_list[$current_tab] ) ? $block_list[$current_tab] : $block_list[ key( $block_list ) ];

    echo '<div class="wpep-block' . ( isset( $information_block['type'] ) ? ( ' wpep-block-' . $information_block['type'] ) : '' ) . '">';
    if( isset( $information_block ['video'] ) )
      echo '<div class="wpep-video-container">' . wpep_get_video_source_output( $information_block ['video'] ) . '</div>';

    echo '<h2>' . $information_block['title'] . '</h2>';

    if( isset( $information_block['content'] ) )
      echo '<div class="description">' . $information_block['content'] . '</div>';

    echo '</div>';
  }

  public function information() {
    $information = get_transient( 'wpep_learning_center_content' );

    if( $information == false ) {
      $information = wp_remote_post(WPEP_LEARNING_CENTER_JSON_LINK, [ 'timeout' => 15, 'sslverify' => false ] );

      if( is_wp_error( $information ) )
        return false;

      $information = json_decode( wp_remote_retrieve_body( $information ), true );

      set_transient( 'wpep_learning_center_content', $information, 24 * HOUR_IN_SECONDS );
    }

    return $information;
  }

}