<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;

/**
 * declare Editor class.
 *
 * @since        1.0
 */
class MetaBoxes {

  use WPEP_Container_Configuration;

  private $_post_id = 0;
  private $_displayed_metabox_post_id = false;

  public function __construct() {
    $this->set_field_prefix( 'wpep_meta_value' );
  }

  public function setup() {
    add_action( 'add_meta_boxes', [ $this, 'register_meta_boxes'  ] );
    add_action( 'save_post', [ $this, '_handle_save' ], 10 );

    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 20, 2 );
  }

  public function register_meta_boxes() {
    $meta_boxes = apply_filters( 'wpep_meta_boxes', [] );

    foreach( $meta_boxes as $meta_box ) {
      $meta_box['post_types'] = ( isset( $meta_box['post_types'] ) ? $meta_box['post_types'] : $meta_box['pages'] );
      unset($meta_box['pages']);

      add_meta_box(
        $meta_box['id'],
        $meta_box['title'],
        [ $this, '_display_fields' ],
        $meta_box['post_types'],
        $meta_box['context'],
        $meta_box['priority'],
        $meta_box['fields']
      );
    }
  }

  public function _display_fields( $post, $information ) {
    $this->_post_id = $post->ID;

    $remaining_fields = count( $information['args'] );

    if( $this->_displayed_metabox_post_id == false ) {
      $this->display_hidden_field( 'wpep_metabox_post_id', $this->_post_id, 'wpep_metabox_post_id' );
      $this->_displayed_metabox_post_id = true;
    }

    echo '<div class="wpep-meta-box-container">';

    foreach( $information['args'] as $field_information ) {
      if( isset( $field_information[ 'content_before' ] ) )
        echo $field_information[ 'content_before' ];

      if( isset( $field_information['name'] ) )
        echo '<p class="label">' . $field_information['name'] . '</p>';

      if( isset( $field_information['top_description'] ) || isset( $field_information['top_desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['top_description'] ) ? $field_information['top_description'] : $field_information['top_desc'] ) . '</div>';

      if( $field_information['type'] == 'radio_inline' ) {
        $this->display_radio_fields(
          $field_information['id'],
          ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] ),
          ( isset( $field_information['default'] ) ? $field_information['default'] : 0 )
        );
      } else if( $field_information['type'] == 'select' ) {
        $this->display_select_field(
          $field_information['id'],
          ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] ) ,
          false,
          null,
          ( isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field' ),
          ( isset( $field_information['default'] ) ? $field_information['default'] : '' ),
          ( isset( $field_information['attributes'] ) ? $field_information['attributes'] : [] )
        );
      } else if( $field_information['type'] == 'multiple_select' ) {
        $this->display_multiple_select_field( $field_information['id'], ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] ) );
      } else if( $field_information['type'] == 'wysiwyg' || $field_information['type'] == 'editor' ) {
        $this->display_editor_field( $field_information['id'] );
      } else if( $field_information['type'] == 'textarea' ) {
        $this->display_textarea_field(
          $field_information['id'],
          ( isset( $field_information['input_name'] ) ? $field_information['input_name'] : false ),
          ( isset( $field_information['height'] )     ? $field_information['height']     : false )
        );
      } else if( $field_information['type'] == 'colorpicker' || $field_information['type'] == 'input_color' ) {
        $this->display_color_field( $field_information['id'] );
      } else if( $field_information['type'] == 'image' || $field_information['type'] == 'file' ) {
        $this->display_image_field( $field_information['id'], $field_information['name'] );
      } else if( $field_information['type'] == 'input' || $field_information['type'] == 'text' ) {
        $this->display_text_field(
          $field_information['id'],
          false,
          null,
          ( isset( $field_information['class'] ) ? $field_information['class'] : false ),
          ( isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : '' ),
          ( isset( $field_information['data'] ) ? $field_information['data'] : [] ),
          ( isset( $field_information['default'] ) ? $field_information['default'] : '' )
        );
      } else if( $field_information['type'] == 'attributes' ) {
        $this->display_attributes_field( 
          $field_information['id'], 
          false, 
          null, 
          ( isset( $field_information['default'] ) ? $field_information['default'] : [] )
        );
      }

      if( isset( $field_information['description'] ) || isset( $field_information['desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['description'] ) ? $field_information['description'] : $field_information['desc'] ) . '</div>';

      if( $remaining_fields > 1 && $field_information['type'] != 'none' )
        echo '<hr/>';

      if( isset( $field_information[ 'content_after' ] ) )
        echo $field_information[ 'content_after' ];

      if( $field_information['type'] != 'none' )
        $remaining_fields--;
    }

    echo '</div>';
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = get_post_meta( $this->_post_id, $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

  public function _handle_save( $post_id ) {
    if( !isset( $_POST[ $this->field_prefix ] ) )
      return;

    if( !isset( $_POST[ 'wpep_metabox_post_id' ] ) )
      return;

    if( !is_array( $_POST[ 'wpep_metabox_post_id' ] ) && intval( $_POST[ 'wpep_metabox_post_id' ] ) !== intval( $post_id ) )
      return;
    else if( is_array( $_POST[ 'wpep_metabox_post_id' ] ) && !in_array( $post_id, $_POST[ 'wpep_metabox_post_id' ] ) )
      return;

    do_action( "wpep_administration_meta_boxes_save_before" );

    $information = apply_filters( "wpep_administration_meta_boxes_save_information", $_POST[ $this->field_prefix ] );

    foreach( $information as $field_key => $field_value ) {
      if( is_array( $field_value ) ) {
        $single_meta = apply_filters( "wpep_administration_meta_boxes_single_meta", ( strpos( $field_key, '_' ) === 0 ? false : true ), $field_key, $field_value );

        if( $single_meta ) {
          delete_post_meta( $post_id, $field_key );

          foreach( $field_value as $field_val ) {
            // We have our tricks here.
            if( $field_val == 'a:0:{}' )
              continue;

            add_post_meta( $post_id, $field_key, $field_val );
          }
          continue;
        }
      }

      if( $field_value === '' ) {
        delete_post_meta( $post_id, $field_key );

        continue;
      }

      update_post_meta( $post_id, $field_key, $field_value );
    }

    do_action( "wpep_administration_meta_boxes_save_after" );
  }

  public function _wpep_admin_save_post_ajax( $entity_id_map = [], $post_id ) {
    $this->_handle_save( $post_id );

    return $entity_id_map;
  }

}