<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

/**
 * declare Editor class.
 *
 * @since        1.0
 */

class Offer {

  public function setup() {
    add_filter('wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );
    add_action('save_post', [ $this, '_save_post' ] );
  }

  public function _save_post( $course_post_id ) {
    global $post_type;

    if ( !(is_admin() && ( $post_type == WPEP_POST_TYPE_OFFER ) ) )
      return;

    if( get_post_type( $course_post_id ) != WPEP_POST_TYPE_OFFER )
      return;

    if( intval( get_post_meta( $course_post_id, 'wpep_order', true ) ) == false )
      update_post_meta( $course_post_id, 'wpep_order', 0 );
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    $offer_meta_boxes = [];

    $offer_meta_boxes['wpep-offer-settings'] = [
      'id'         => 'wpep_offer_settings',
      'title'      => __( 'Offer Information', "wpep" ),
      'post_types' => [ WPEP_POST_TYPE_OFFER ],
      'context'    => 'normal',
      'priority'   => 'low',
      'fields'     => [
        [
          'name'        => __( 'Sub Title', "wpep" ),
          'id'          => 'offer_subtitle',
          'type'        => 'text',
          'description' => __( 'The sub-title is present under the title, this field is frequently used to display the offer type, for example "Monthly Membership".', "wpep" )
        ],
        [
          'name'        => __( 'Description', "wpep" ),
          'id'          => 'offer_description',
          'type'        => 'textarea',
          'description' => __( 'Text only offer description.', "wpep" )
        ],
        [
          'name'        => __( 'Link', "wpep" ),
          'id'          => 'offer_link',
          'type'        => 'text',
          'description' => __( 'The link where people will be taken when they redeem the offer.', "wpep" )
        ],
        [
          'name'        => __( 'Link Text', "wpep" ),
          'id'          => 'offer_link_text',
          'type'        => 'text',
          'description' => __( 'The text displayed on the button.', "wpep" )
        ],
        [
          'name'        => __( 'Highlighted Information', "wpep" ),
          'id'          => '_offer_information_map',
          'type'        => 'attributes',
          'description' =>
            sprintf(
              '<p>' . __( 'An offer is usually accompanied by a few key details, such as the discount value, the discount coupon, and even the discount availability.', "wpep" ) . '</p>' .
              '<p>' . __( "We allow each offer to have different highlighted information blocks, if you're using an standard format, you can %s", "wpep" ) . '</p>'
              ,
              '<a href="' . admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings&tab=index_page&tab_inner=offer' ) . '">' . __( "manage the Offers Settings", "wpep" ) . '</a>'
            ),
          'default'     => WPEP_Settings::instance()->get_option( 'offer-information-map-default' )
        ],
        [
          'name'    => __('Button Color', "wpep"),
          'id'      => 'wpep_post_button_color',
          'type'    => 'colorpicker',
          'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
        ],
        [
          'name'    => __('Button Text Color', "wpep"),
          'id'      => 'wpep_post_button_text_color',
          'type'    => 'colorpicker',
          'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
        ],
      ]
    ];

    $offer_meta_boxes = apply_filters( "wpep_administration_offer_metaboxes", $offer_meta_boxes );

    return $meta_boxes + $offer_meta_boxes;
  }

}