<?php

namespace WPEP\Admin\Coordinator\PlatformHealth\CleanupUtilities;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class GeneralMaintenance {

  public function display() {
    wpep_administration_platform_health_table(
      $this->_get_column_details(),
      __( "General Maintenance", "wpep" ),
      'wpep-cleanup-utilities-table wpep-cleanup-utilities-table-left'
    );
  }

  private function _get_column_details() {
    $old_options          = WPEP_Settings::instance()->get_legacy_options();
    $old_custom_css_files = count( wpep_get_custom_style_old_files() );
    $empty_metadata       = $this->_get_empty_metadata_field_count();

    $response = [];

    if( empty( $old_options ) ) {
      $response['settings_junk'] = [
        'status'  => 'success',
        'title'   => __( "Settings Junk - No Entries", "wpep" )
      ];
    } else {
      $response['settings_junk'] = [
        'status'        => 'danger',
        'title'         => sprintf( __( "Settings Junk - %s Entries", "wpep" ), '<span>' . count( $old_options ) . '</span>' ),
        'operation'     => wpep_admin_url_platform_health( [ 'tab' => 'cleanup-utilities', 'operation' => 'remove-old-settings' ]  ),
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    if( empty( $empty_metadata ) ) {
      $response['metadata_junk'] = [
        'status'  => 'success',
        'title'   => __( "Empty WPEP Posts Metadata - No Entries", "wpep" )
      ];
    } else {
      $response['metadata_junk'] = [
        'status'        => 'danger',
        'title'         => sprintf( __( "Empty WPEP Posts Metadata - %s Entries", "wpep" ), '<span>' . $empty_metadata . '</span>' ),
        'operation'     => wpep_admin_url_platform_health( [ 'tab' => 'cleanup-utilities', 'operation' => 'remove-empty-post-metadata' ]  ),
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    if( $old_custom_css_files == 0 ) {
      $response['css_file_junk'] = [
        'status'  => 'success',
        'title'   => __( "Custom CSS - No Extra Files", "wpep" )
      ];
    } else {
      $response['css_file_junk'] = [
        'status'        => 'danger',
        'title'         => sprintf( __( "Custom CSS - %s Extra Files", "wpep" ), '<span>' . $old_custom_css_files . '</span>' ),
        'operation'     => wpep_admin_url_platform_health( [ 'tab' => 'cleanup-utilities', 'operation' => 'remove-old-css-files' ]  ),
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    return apply_filters( 'wpep_administration_platform_health_cleanup_maintenance_columns', $response );
  }

  public function process( $operation = false, $operation_confirmation = false ) {
    if( $operation == false )
      return;

    do_action( 'wpep_administration_platform_health_cleanup_maintenance_operation', $operation, $operation_confirmation );

    if( $operation == 'remove-old-settings' ) {
      $old_options = WPEP_Settings::instance()->get_legacy_options();

      if( !empty( $old_options ) ) {
        WPEP_Settings::instance()->delete_setting( $old_options );

        echo '<div class="wpep-notification wpep-notification-success">' . sprintf( __( "Removed %s old WPEP Settings.", "wpep" ), count( $old_options ) ) . '</div>';
      }
    } else if( $operation == 'remove-old-css-files' ) {
      $old_custom_css_files = wpep_get_custom_style_old_files();

      if( count( $old_custom_css_files ) != 0 ) {
        $custom_css_file_directory = wpep_get_upload_dir();

        foreach( $old_custom_css_files as $custom_css_file )
          if( file_exists( $custom_css_file_directory . '/' . $custom_css_file ) )
            unlink( $custom_css_file_directory . '/' . $custom_css_file );

        echo '<div class="wpep-notification wpep-notification-success">' . sprintf(__("Removed %s old WPEP Custom CSS Files.", "wpep"), count( $old_custom_css_files ) ) . '</div>';
      }
    } else if( $operation == 'remove-empty-post-metadata' ) {
      global $wpdb;

      $count = $wpdb->query( 'DELETE FROM `' . $wpdb->postmeta . '` WHERE ' . $this->_get_empty_metadata_sql_where() );

      if( $count == 0 ) {
        echo '<div class="wpep-notification wpep-notification-info">' . __("Empty metadata entries have been already removed.", "wpep") . '</div>';
      } else {
        echo '<div class="wpep-notification wpep-notification-success">' . sprintf( __("Removed %s empty metadata entries.", "wpep"), $count ) . '</div>';
      }
    }
  }

  private function _get_empty_metadata_field_count() {
    global $wpdb;

    return $wpdb->get_var( 'SELECT COUNT(*) FROM `' . $wpdb->postmeta . '` WHERE ' . $this->_get_empty_metadata_sql_where() );
  }

  private function _get_empty_metadata_sql_where() {
    return '( `meta_key` LIKE "wpep_%" AND `meta_value` = "" ) OR ( `meta_key` LIKE "_wpep_%" AND `meta_value` = "" )';
  }

}