<?php

namespace WPEP\Admin\Coordinator;

class Post {

  public function setup() {
    add_filter('wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );

    if( wpep_has_embed_support() ) {
      add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
      add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

      if( in_array( 'page', $this->_external_embed_allowed_post_types() ) ) {
        add_filter('manage_pages_columns', [ $this, '_table_set_columns' ] );
        add_filter('manage_pages_custom_column', [ $this, '_table_custom_columns' ] );
      }
    }
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    if( !wpep_get_setting_flag( 'enable-post-member-magnet-option' ) )
      return $meta_boxes;

    $meta_boxes['wpep_member_magnet']  = [
      'id'         => 'wpep_member_magnet',
      'title'      => __('Member Magnet Option Configuration', "wpep"),
      'post_types' => [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_E_BOOK, WPEP_POST_TYPE_OFFER ],
      'context'    => 'advanced',
      'priority'   => 'low',
      'fields'     => [
        [
          'name'    => __( 'Enable', "wpep" ),
          'desc'    => '<p>' . __( "The Member Magnet option allows you to make it mandatory for users to login before accessing the content.", "wpep" ) . '</p>',
          'id'      => 'wpep_member_magnet_option_status',
          'type'    => 'select',
          'options' => wpep_flag_setting_field_options()
        ],
        [
          'name'    => __( 'Default View', "wpep" ),
          'desc'    => '<p>' . __( "The page that will be displayed on page load, normally login", "wpep" ) . '</p>',
          'id'      => 'wpep_member_magnet_option_form_view',
          'type'    => 'select',
          'options' => [
            'login'          => __( 'Login', "wpep" ),
            'register'       => __( 'Register', "wpep" ),
            'reset_password' => __( 'Reset Password', "wpep" ),
          ]
        ],
        [
          'name'    => __('Content Before Authentication Form', "wpep"),
          'id'      => 'wpep_member_magnet_option_text_before',
          'type'    => 'editor'
        ],
        [
          'name'    => __('Content After Authentication Form', "wpep"),
          'id'      => 'wpep_member_magnet_option_text_after',
          'type'    => 'editor'
        ],
        [
          'name'        => __('Optional - Registration Redirect URL', "wpep"),
          'id'          => 'wpep_member_magnet_option_registration_redirect_url',
          'type'        => 'text',
          'description' => __( "Redirect the Student to a Custom Thank You Page, if an account was just created.", "wpep" )
        ],
      ]
    ];

    return $meta_boxes;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( !in_array( $post_type,  $this->_external_embed_allowed_post_types() ) )
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'external_embed' => __( 'External Embed', "wpep")
    ];

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post,$post_type;

    if ( !in_array( $post_type,  $this->_external_embed_allowed_post_types() ) )
      return;

    if( $column == 'external_embed' ) {
      echo wpep_get_external_page_embed_copy_button( $post->ID );

      return;
    }
  }

  private function _external_embed_allowed_post_types() {
    return [ WPEP_POST_TYPE_COURSE, 'page' ];
  }

}
