<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

/**
 * declare Plugin class.
 *
 * @since        1.0
 */

class Settings {

  /**
   * @var array
   */
  protected $_settingsPages;

  public function setup() {
    add_action( 'admin_menu', [ $this, '_register_menu' ] );

    // We want our functions with inits to also receive fresh information.
    add_action( 'init', [ $this, '_init' ], 0 );
  }

  public function _register_menu() {
    add_submenu_page(
      'edit.php?post_type=' . WPEP_POST_TYPE_COURSE,
      __("WPEP Settings", 'wpep'),
      __( "Settings", 'wpep'),
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-settings",
      [ $this, "display" ]
    );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( !isset( $_POST['wpep_settings_save'] ) )
      return;

    if( isset( $_POST['wpep_settings'] ) )
      WPEP_Settings::instance()->update_settings( $_POST['wpep_settings'] );

    if( isset( $_POST['wpep_settings_post_order'] ) )
      WPEP_Settings::instance()->update_post_order( $_POST['wpep_settings_post_order'] );

    if( isset( $_POST['wpep_settings_taxonomy_order'] ) )
      WPEP_Settings::instance()->update_taxonomy_order( $_POST['wpep_settings_taxonomy_order'] );

    if( isset( $_POST['wpep_add_on_settings'] ) )
      WPEP_Settings::instance()->update_addon_settings( $_POST['wpep_add_on_settings'] );

    $url = add_query_arg( 'wpep-settings-saved', 'true', wpep_get_current_url() );
    $url = remove_query_arg( 'wpep-walk-trough', $url );

    wp_redirect( $url );
    exit;
  }

  public function display() {
    echo '<div class="wrap wpep-settings-container">';
    echo '<h2>' . __( "WPEP Settings", 'wpep') . '</h2>';
    echo '<hr/>';

    if( isset( $_GET['wpep-settings-saved']) ) {
      echo '<div class="updated notice is-dismissible">';
      echo  '<p>' . __('Settings have been updated.', 'wpep') . '</p>';
      echo  '<button class="notice-dismiss" type="button"><span class="screen-reader-text">' . __( "Dismiss this notice.", 'wpep') . '</span></button>';
      echo '</div>';
    }

    $settings_pages = $this->get_settings_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $settings_pages ) );

    if( !isset( $settings_pages[ $current_tab ] ) )
      $current_tab = key( $settings_pages );

    if( $settings_pages[ $current_tab ]->display_as_form() ) {
      echo '<form novalidate="novalidate" method="post">';
    }

    $this->show_tabs( $current_tab );

    $settings_pages[ $current_tab ]->display();

    if( $settings_pages[ $current_tab ]->display_as_form() ) {
      echo    '<p class="submit"><input type="submit" value="Save Changes" class="button button-primary" id="submit" name="wpep_settings_save"></p>';
      echo '</form>';
    }

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-settings-navigation wpep-settings-main-navigation nav-tab-wrapper">';
    foreach( $this->get_settings_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' nav-tab-active' : '';
      echo "<a class='wpep-settings-tab-$tab nav-tab$class' href='?post_type=" . WPEP_POST_TYPE_COURSE . "&page=wpep-settings&tab=$tab'>";
      echo    '<span>' . $setting_page->get_name() . '</span>';
      echo "</a>";

    }
    echo '</h2>';

    echo '<div class="clearfix"></div>';
  }

  public function get_settings_pages() {
    if( !is_array( $this->_settingsPages ) ) {
      $this->_settingsPages = [];

      foreach( [
                 new Settings\Dashboard(),
                 new Settings\General(),
                 new Settings\IndexPage(),
                 new Settings\CoursePage(),
                 new Settings\LearnerProfile(),
                 new Settings\LearnerEngagement(),
                 new Settings\AddOns()
               ] as $current_instance )
        $this->_settingsPages[ $current_instance->get_alias() ] = $current_instance;

      apply_filters( 'wpep_administration_settings_pages', $this->_settingsPages );
    }

    return $this->_settingsPages;
  }

}