<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;

class AddOns {

  public function get_name() {
    return __( 'AddOns Settings', "wpep" );
  }

  public function get_alias() {
    return 'addon';
  }

  public function display_as_form() {
    return true;
  }

  public function display() {
    $addon_list = WPEP_Controller::instance()->addOns->get_add_on_settings_list();

    if( empty( $addon_list ) ) {
      echo '<p>' . __("Nothing to show here.", "wpep") . ' <a href="' . admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-addons' ) . '">' . __("Search for AddOns", 'wpep') . '</a></p>';
      return;
    }

    $current_addon = WPEP_Controller::instance()->addOns->get_add_on_settings( isset( $_GET['addon'] ) ? $_GET['addon'] : '' );

    if( $current_addon === false )
      $current_addon = $addon_list[key($addon_list)];

    $add_on_alias = $current_addon->get_alias();

    $this->display_tabs( $addon_list, $add_on_alias );

    echo    '<table class="form-table">';
    echo      '<tbody>';

    foreach( $current_addon->get_field_list() as $field_key => $field_information ) {
      if( !isset( $field_information['type'] ) ) {
        $current_internal_tab = ( isset( $_GET['addon_tab'] ) ? $_GET['addon_tab'] : false );

        if( !isset( $field_information[$current_internal_tab] ) )
          $current_internal_tab = key( $field_information );

        $this->display_internal_tabs( $add_on_alias, $field_information, $current_internal_tab );

        foreach( $field_information[$current_internal_tab] as  $internal_field_key => $internal_field_information )
          $this->_display_field( $add_on_alias, $internal_field_key, $internal_field_information );

        continue;
      }

      $this->_display_field( $add_on_alias, $field_key, $field_information );
    }

    echo      '</tbody>';
    echo    '</table>';
  }

  private function _display_field( $add_on_alias, $field_key, $field_information ) {
    $input_name  = 'wpep_add_on_settings[' . $add_on_alias . '][' . $field_key . ']';
    $input_id    = 'wpep-addon-' . $add_on_alias . '-' . $field_key;
    $input_value = ( isset( $field_information['value'] ) ? $field_information['value'] : '' );

    echo        '<tr ' . ( $field_information['type'] == 'html' && $field_information['html'] === '<hr/>' ? 'class="wpep-table-separator"' : '' );

    if( isset( $field_information['row_attributes'] ) ) {
      foreach( $field_information['row_attributes'] as $attr_key => $attr_value )
        echo ' ' . $attr_key . '="' . $attr_value . '"';
    }

    echo         '>';
    echo          '<th scope="row"><label for="' . $input_id . '">' . $field_information['name'] . '</label></th>';
    echo          '<td>';

    if( $field_information['type'] == 'text' || $field_information['type'] == 'colorpicker' ) {
      $class = ( $field_information['type'] == 'text' ? 'regular-text' : 'wpep-color-field' ) . ( isset( $field_information[ 'class' ] ) ? ' ' . $field_information[ 'class' ] : '' );
      echo            '<input type="text"
                                class="' . $class . '"
                                id="' . $input_id . '"
                                value="' . $input_value . '"
                                name="' . $input_name . '">';
    } else if( $field_information['type'] == 'checkbox' ) {
      echo            '<input type="hidden"
                              id="' . $input_id . '"
                              value="0"
                              name="' . $input_name . '">';
      echo            '<input type="checkbox"
                              id="' . $input_id . '"
                              value="1"
                              name="' . $input_name . '"
                              ' . checked( $input_value, 1, false ) . '
                              >';
    } else if( $field_information['type'] == 'textarea' ) {
      echo            '<textarea id="' . $input_id . '"
                                   name="' . $input_name . '"
                                   style="width:400px;height:100px;">' . $input_value  . '</textarea>';
    } else if( $field_information['type'] == 'editor' ) {
      wpep_editor( $input_value, $input_id, $input_name );
    } else if( $field_information['type'] == 'select' ) {
      $class = isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field';

      echo '<select id="' . $input_id . '" name="' . $input_name . '" class="' . $class . '"';

      if( isset( $field_information['attributes'] ) ) {
        foreach( $field_information['attributes'] as $attr_key => $attr_value )
          echo ' ' . $attr_key . '="' . $attr_value . '"';
      }

      echo '>';

      foreach( $field_information['values'] as $value => $label )
        echo '<option value="' . $value . '" ' . ( selected( $value, $input_value, false ) ) . '>' . $label . '</option>';

      echo            '</select>';
    } else if( $field_information['type'] == 'multiple_select' ) {
      if( !is_array( $input_value ) )
        $input_value = [];

      echo            '<select id="' . $input_id . '" name="' . $input_name . '[]" class="wpep-js-select-element wpep-select-field" multiple="multiple">';

      foreach( $field_information['values'] as $value => $label )
        echo '<option value="' . $value . '" ' . ( selected( in_array( $value, $input_value ), true, false ) ) . '>' . $label . '</option>';

      echo            '</select>';
    } else if( $field_information['type'] == 'html' ) {
      echo            $field_information['html'];
    } else if( $field_information['type'] == 'text_disabled'  ) {
      echo            '<input type="text"
                                class="regular-text"
                                id="' . $input_id . '"
                                value="' . $input_value . '"
                                disabled="disabled">';
    } else if( $field_information['type'] == 'attributes_field_filters' ) {
      $field_map         = ( !empty( $field_information['value'] ) && is_array( $field_information['value'] ) ? $field_information['value'] : $field_information['default'] );

      echo '<div class="wpep-attributes-field-container" data-name="' . $input_name . '">';
      echo '<table class="wp-list-table fixed widefat">';
      echo  '<thead>';
      echo    '<tr>';
      echo      '<th class="wpep-attributes-field-row-drag"></th>';
      echo      '<th class="wpep-attributes-field-row-key">' . __( "Title", "wpep" ) . '</th>';
      echo      '<th class="wpep-attributes-field-row-value">' . __( "Icon", "wpep" ) . '</th>';
      echo      '<th class="wpep-attributes-field-row-remove"></th>';
      echo    '</tr>';
      echo  '</thead>';
      echo  '<tbody class="wpep-attributes-field-group-container">';

      foreach( $field_map as $key => $value ) {
        echo '<tr class="wpep-attributes-field-group">';
        echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
        echo  '<td class="wpep-attributes-field-row-key">';
        echo    '<input type="hidden" class="wpep-attributes-field" name="' . $input_name . '[' . $key . ']" value="' . $value . '"/>';
        echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
        echo  '</td>';
        echo  '<td class="wpep-attributes-field-row-value">';
        echo    '<input type="text" class="wpep-attributes-field-value wpep-js-select-element-icon" value="' . $value . '"/>';
        echo  '</td>';
        echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
        echo '</tr>';
      }

      echo '<tr class="wpep-attributes-field-group" style="display:none;">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="" value=""/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value wpep-js-select-element-icon" value=""/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';

      echo  '</tbody>';
      echo '</table>';
      echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Filter", "wpep" ) . '</button>';
      echo '<div class="clear"></div>';
      echo '</div>';
    } else if( $field_information['type'] == 'select_sequence' ) {
      $selected_values = ( !empty( $field_information['value'] ) && is_array( $field_information['value'] ) ? $field_information['value'] : $field_information['default'] );

      echo '<div class="wpep-attributes-field-container" data-name="' . $input_name . '">';
      echo '<table class="wp-list-table fixed widefat">';
      echo  '<thead>';
      echo    '<tr>';
      echo      '<th class="wpep-attributes-field-row-drag"></th>';
      echo      '<th class="wpep-attributes-field-row-key">' . __( "Name", "wpep" ) . '</th>';
      echo      '<th class="wpep-attributes-field-row-remove"></th>';
      echo    '</tr>';
      echo  '</thead>';
      echo  '<tbody class="wpep-attributes-field-group-container">';

      foreach( $selected_values as $selected_membership_id ) {
        echo '<tr class="wpep-attributes-field-group">';
        echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';

        echo  '<td class="wpep-attributes-field-row-value">';
        echo    '<input type="hidden" class="wpep-attributes-field" name="' . $input_name . '[' . $selected_membership_id . ']" value="' . $selected_membership_id . '"/>';

        $this->_sequence_field_select( $field_information, $selected_membership_id );

        echo  '</td>';
        echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
        echo '</tr>';
      }

      echo '<tr class="wpep-attributes-field-group" style="display:none;">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="" value=""/>';
      $this->_sequence_field_select( $field_information, 0 );
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';

      echo  '</tbody>';
      echo '</table>';
      echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Field", "wpep" ) . '</button>';
      echo '<div class="clear"></div>';
      echo '</div>';
    }

    if( isset( $field_information['description'] ) && is_string( $field_information['description'] ) )
      echo '<p class="description">' . $field_information['description'] . '</p>';
    else if( isset( $field_information['desc'] ) && is_string( $field_information['desc'] ) )
      echo '<p class="description">' . $field_information['desc'] . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_tabs( $addon_list, $current = false) {
    echo '<h3 class="wpep-settings-navigation wpep-settings-secondary-navigation nav-tab-wrapper">';

    $i = 1; foreach( $addon_list as $addon ){
      $image = '';

      if( $addon->get_controller() != false )
        $image = WPEP_Controller::instance()->adminController->addOnsSection->get_addon_image_by_controller_information( $addon->get_controller()->get_information() );

      $class = ( $addon->get_alias() == $current || ( $current == false && $i == 1) ) ? ' nav-tab-active' : '';

      if( $image != '' )
        $class .= ' wpep-has-image';

      echo '<a class="nav-tab' . $class . '" href="?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings&tab=addon&addon=' . $addon->get_alias() . '">';
      echo    ( $image != '' ? '<img class="wpep-settings-addon-image" src="' . $image . '"/>' : '' );
      echo    $addon->get_name();
      echo '</a>';

      $i++;
    }
    echo '</h3>';

    echo '<div class="clearfix"></div>';
  }

  public function display_internal_tabs( $addon_alias, $internal_tabs = [], $current_tab = false ) {
    echo '<h3 class="nav-tab-wrapper wpep-settings-navigation wpep-settings-tertiary-navigation">';

    $i = 1; foreach( $internal_tabs as $internal_tab_alias => $internal_tab_fields ){
      $class = ( $internal_tab_alias == $current_tab || ( $current_tab == false && $i == 1) ) ? ' nav-tab-active' : '';
      echo '<a class="nav-tab' . $class . '" href="?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings&tab=addon&addon=' . $addon_alias . '&addon_tab=' . $internal_tab_alias . '">' . ucwords( str_replace( '_', ' ', $internal_tab_alias ) ) . '</a>';

      $i++;
    }
    echo '</h3>';

    echo '<div class="clearfix"></div>';
  }

  private function _sequence_field_select( $field_information, $selected_membership_id = 0 ) {
    echo '<select class="wpep-attributes-field-key wpep-attributes-field-value wpep-select-field">';
    echo      '<option value="">' . __( "Select an option", "wpep" ) . '</option>';
    foreach( $field_information['values'] as $membership_id => $membership_title )
      echo    '<option value="' . $membership_id . '" ' . ( selected( $membership_id == $selected_membership_id, true, false ) ) . '>' . $membership_title . '</option>';

    echo '</select>';
  }

}