<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class LearnerEngagement extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'weekly_recap'      => __( 'Weekly Recap', "wpep" ),
      'delivery_settings' => __( 'Delivery Settings', "wpep" ),
      'comments'          => __( "Comments", "wpep" )
    ];

    if( wpep_has_embed_support() )
      $this->tabs[ 'share_embed' ] = __( 'Sharing & Embed', "wpep" );
  }

  public function get_name() {
    return __( 'Learner Engagement', "wpep" );
  }

  public function get_alias() {
    return 'learner_engagement';
  }

  public function display() {
    $current_tab = $this->get_current_tab();

    $this->display_tabs( $current_tab );

    if( $current_tab == 'weekly_recap' )
      $this->display_weekly_recap_options();
    else if( $current_tab == 'share_embed' )
      $this->display_share_embed_options();
    else if( $current_tab == 'comments' )
      $this->display_comments_options();
    else
      $this->display_delivery_settings_options();
  }

  public function get_current_tab() {
    $current_tab = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : key( $this->tabs ) );

    if( !isset( $this->tabs[ $current_tab ] ) )
      $current_tab = key( $this->tabs );

    return $current_tab;
  }

  public function display_weekly_recap_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Status', "wpep" ) ,
      'email-weekly-recap-status',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ]
    );

    $this->display_table_separator(
      __( 'Preview', "wpep" ),
      '<a class="button button-primary" target="_blank" href="' . add_query_arg( 'wpep-preview-weekly-recap', 'true', get_site_url() ) . '">Preview</a>'
    );

    $this->display_table_text_translated_field( __( 'Title', "wpep" ),  'email-weekly-recap-title', __( 'This is the email title, you can optionally use shortcodes, for example [user_name], [date]', "wpep" ) );

    $this->display_table_editor_field(
      __('Content', "wpep"),
      'email-weekly-recap-content',
      '<h2>' . __( "General ShortCodes") . '</h2>' .
      '<ul>' .
        '<li>[user_name] - ' . __( "The name of the person that receives this email.", "wpep" ) . '</li>' .
        '<li>[date] - ' . __( "The name of the person that receives this email.", "wpep" ) . '</li>' .
        '<li>[site_url] - ' . __( "The current website url.", "wpep" ) . '</li>' .
        '<li>[number_lessons_completed] - ' . __( "The number of lessons completed this week.", "wpep" ) . '</li>' .
        '<li>[list_courses_completed] - ' . __( "List of Courses Completed this Week.", "wpep" ) . '</li>' .
        '<li>[list_courses_started] - '  . __( "List of Courses Started this Week.", "wpep" ) . '</li>' .
      '</ul>' .
      '<h2>' . __( "Condition ShortCodes, will display the content within, if the condition is met") . '</h2>' .
      '<ul>' .
        '<li>[condition_has_completed_courses] - ' . __( "The user has to complete at least 1 course.", "wpep" ) . '</li>' .
        '<li>[condition_has_started_courses] - ' . __( "The user has to start at least 1 course.", "wpep" ) . '</li>' .
        '<li>[condition_has_activity] - ' . __( "The user had activity this week.", "wpep" ) . '</li>' .
        '<li>[condition_no_activity] - ' . __( "The user didn't have any activity this week.", "wpep" ) . '</li>' .
      '</ul>'
      ,
      300
    );

    $this->display_table_end();
  }

  public function display_delivery_settings_options() {
    $this->display_table_start();

    $this->display_table_separator( __( 'Branding', "wpep" ) );

    $this->display_table_image_field(
      __( 'Email Logo', "wpep" ),
      'email-logo',
      __( 'The Logo used in the Email Template', "wpep" ),
      false,
      null,
      [
        'width'  => WPEP_EMAIL_LOGO_WIDTH,
        'height' => WPEP_EMAIL_LOGO_HEIGHT
      ]
    );

    $this->display_table_text_field( __( 'Company Name', "wpep" ), 'email-company-name' );
    $this->display_table_text_field( __( 'Company Address', "wpep" ), 'email-company-address' );
    $this->display_table_text_field( __( 'Copyright', "wpep" ), 'email-copyright-text' );

    $this->display_table_separator( __( 'Email Delivery', "wpep" ) );

    $this->display_table_text_field( __( 'From Name', "wpep" ), 'email-from-name', __( 'The default name will be used if left empty.', "wpep" ) );
    $this->display_table_text_field( __( 'From Email', "wpep" ), 'email-from-email',
      __( 'The default email address will be used if left empty.', "wpep" ) . '<br/>' .
      __( 'Note : To avoid getting the emails flagged as spam, you must use an address from the current domain.', 'wpep' )
    );

    $this->display_table_end();
  }

  public function display_comments_options() {
    $this->display_table_start();

    $this->display_table_separator( __( 'List', "wpep" ) );

    $this->display_table_text_field( __( 'Search', "wpep" ), 'comments-search-label' );
    $this->display_table_text_field( __( 'Comments Per Page', "wpep" ), 'comments-per-page' );
    $this->display_table_text_field( __( 'Reply', "wpep" ), 'comments-button-reply' );
    $this->display_table_text_field( __( 'Previous', "wpep" ), 'comments-navigation-previous' );
    $this->display_table_text_field( __( 'Next', "wpep" ), 'comments-navigation-next' );

    $this->display_table_separator( __( 'Form', "wpep" ) );

    $this->display_table_text_field( __( 'Content Required', "wpep" ), 'comments-notification-content-empty' );
    $this->display_table_text_field( __( 'Cancel Reply', "wpep" ), 'comments-button-cancel-reply' );
    $this->display_table_text_field( __( 'Submit Comment', "wpep" ), 'comments-button-submit' );

    $this->display_table_separator( __( 'Advanced Settings', "wpep" ) );
    $this->display_table_text_field( __( "Sync Comments", "wpep" ), 'comments-refresh-ms-interval',
      __( "In seconds, how frequent WPEP should check if new comments were posted while the page is open for a student, minimum 3.", "wpep" ) . '<br/>' .
      __( "Pro Tip : This process is highly optimized to not overlap requests.", "wpep" )
    );

    $this->display_table_end();
  }

  public function display_share_embed_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Allow Students to Embed Content', "wpep" ),
      'user-content-embed-box-enabled',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ]
    );

    $this->display_table_separator( __( 'Call To Action', "wpep" ) );

    $this->display_table_text_field( __( 'Title', "wpep" ), 'user-content-embed-box-title' );
    $this->display_table_editor_field( __( 'Description', "wpep" ), 'user-content-embed-box-description' );
    $this->display_table_text_field( __( 'Button Text', "wpep" ), 'user-content-embed-box-button' );
    $this->display_table_text_field( __( 'Button Text Clicked', "wpep" ), 'user-content-embed-box-button-clicked' );
    $this->display_table_select_field(
      __( 'Show Inside Embeds', "wpep" ),
      'user-content-embed-box-inside-embeds',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ]
    );

    $this->display_table_end();
  }

}
