<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class LearnerProfile extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'general'         => __( 'General', "wpep" ),
      'tab_management'  => __( 'Tab Management', "wpep" ),
      'public'          => __( 'Public Profiles', "wpep" ),
      'social-networks' => __( 'Social Networks', "wpep" )
    ];
  }

  public function get_name() {
    return __( 'Learner Profile', "wpep" );
  }

  public function get_alias() {
    return 'learner_profile';
  }

  public function display() {
    $current_tab = $this->get_current_tab();

    $this->display_tabs( $current_tab );

    if( $current_tab == 'public' )
      $this->display_public_settings();
    else if( $current_tab == 'social-networks' )
      $this->display_social_network_settings();
    else if( $current_tab == 'tab_management' )
      $this->display_tab_management_settings();
    else
      $this->display_general_settings();
  }

  public function get_current_tab() {
    $current_tab = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : key( $this->tabs ) );

    if( !isset( $this->tabs[ $current_tab ] ) )
      $current_tab = key( $this->tabs );

    return $current_tab;
  }

  public function display_general_settings() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Courses Per Row', "wpep"),
      'user-profile-courses-per-row',
      wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE ),
      __( "How many courses per row will be displayed", "wpep" )
    );

    $this->display_table_text_translated_field(
      __('Change Picture', "wpep"),
      'user-profile-change-picture-button'
    );

    $this->display_table_text_translated_field(
      __('Change Profile Picture', "wpep"),
      'user-profile-change-picture',
      __( "Displayed during the upload & file selection modal / popup")
    );

    $this->display_table_text_translated_field(
      __('Change Cover Picture', "wpep"),
      'user-profile-change-picture-cover',
      __( "Displayed during the upload & file selection modal / popup")
    );
    
    $this->display_table_text_translated_field(
      __('Update Button', "wpep"),
      'user-profile-update-button'
    );

    $this->display_table_separator( __( 'Marketing Content', "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'user-profile-marketing-content-status',
      wpep_flag_setting_field_options(),
      __( "Display the Marketing Content from the Authentication Form on the left side of the Profile Page", "wpep" )
    );

    $this->display_table_editor_field(
      __( 'Content', "wpep"),
      'user-profile-marketing-content'
    );

    $this->display_table_separator( __( 'Default Settings', "wpep" ) );

    $this->display_table_image_field(
      __( "Profile Picture", "wpep" ), 'user-profile-default-profile-picture', false,
      false, null, [
        'width' => '256px',
        'height'=> '256px'
      ],
      false
    );
    $this->display_table_image_field(
      __( "Cover Picture", "wpep" ), 'user-profile-default-cover-picture', false,
      false, null, [], false
    );

    $this->display_table_separator( __( 'Email Notifications', "wpep" ) );

    $this->display_table_text_translated_field(
      __('Weekly Recap', "wpep"),
      'user-profile-email-notification-weekly-recap'
    );

    $this->display_table_end();
  }

  public function display_public_settings() {
    $this->display_table_start();

    $profile_link_example = wpep_user_profile_public_link();

    $this->display_table_checkbox_field(
      __('Enable', "wpep"),
      'user-profile-public-active',
      (
      wpep_public_profiles_enabled() ?
        (
        wpep_link_is_404( $profile_link_example )
          ? (
          '<div class="wpep-notification wpep-notification-danger">' .
          sprintf( __( "There was an error while configuring the user profiles, %s" ), '<a href="' . wpep_admin_url_platform_health( [ 'operation' => 'reset-permalinks' ]  ) . '">' . __( "Click here to resolve it.", "wpep" ) . '</a>' ) .
          '</div>'
        ) : sprintf(
          __( "Your Public Profile :  %s", "wpep" ),
          '<a target="_blank" href="' . $profile_link_example . '">' . $profile_link_example . '</a>'
        )
        )
        :
        __( "Activating the Public Profiles will automatically provide every WordPress User with a Profile Page.", "wpep" )
      ),
      NULL,
      wpep_public_profiles_enabled() && wpep_get_setting_flag( 'user-profile-public-active' ) == false
    );

    $this->display_table_text_translated_field(
      __( 'Profile Page Title', "wpep"),
      'user-profile-heading',
      __( "Adding the %s identifier will replace it with the First Name & Last Name of the student", "wpep" )
    );

    $this->display_table_text_field(
      __('Profile Slug', "wpep"),
      'user-profile-public-slug',
      '',
      false,
      WPEP_Controller::instance()->content->userProfileCoordinator->user_identifier
    );

    $this->display_table_checkbox_field(
      __( 'Course Progress Bars', "wpep"),
      'user-profile-course-public-progress-bars'
    );

    $this->display_table_checkbox_field(
      __( 'Email Address', "wpep"),
      'user-profile-course-public-email-address'
    );

    $this->display_table_end();
  }

  public function display_social_network_settings() {
    $this->display_table_start();

    foreach( wpep_user_profile_social_network_platform_list() as $social_platform_key => $social_platform_information ) {
      $this->display_table_checkbox_array_field(
        '<span class="' . $social_platform_information['icon']. '"></span>' . $social_platform_information['name'],
        'user-profile-enabled-social-networks',
        $social_platform_key
      );
    }

    $this->display_table_end();
  }

  public function display_tab_management_settings() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Options", "wpep" ) );

    $this->display_table_select_field(
      __('Tab Links', "wpep"),
      'user-profile-tab-link-format',
      [
        'label'   => __( "Tab Title", "wpep" ),
        'alias'   => __( "Tab Alias", "wpep" ),
        'disabled'=> __( "Disable Navigation", "wpep" )
      ],
      __( "The URL Format of the links generated for the tabs when clicking on them.", "wpep" )
    );

    $this->display_table_separator( __( "Default Tabs", "wpep" ) );

    $this->display_table_text_translated_field(
      __('Progressing Courses', "wpep"),
      'user-profile-tab-progressing-courses'
    );
    $this->display_table_text_translated_field(
      __('Completed Courses', "wpep"),
      'user-profile-tab-completed-courses'
    );
    $this->display_table_text_translated_field(
      __('Bookmarked Courses', "wpep"),
      'user-profile-tab-bookmarked-courses'
    );

    $this->display_table_separator( __( "Custom Tabs", "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'user-profile-custom-tabs-enabled',
      wpep_flag_setting_field_options()
    );

    $this->display_table_select_field(
      __( 'Position', "wpep"),
      'user-profile-custom-tabs-position',
      [
        'right' => __( "Right Side", "wpep" ),
        'left'  => __( "Left Side", "wpep" )
      ]
    );

    $this->display_table_attributes_field_content_editor(
      __( 'Configure Tabs', "wpep"),
      'user-profile-custom-tabs',
      false,
      false,
      [],
      [
        'Example Tab' => __( "You can add anything here, and it will be displayed within this tab.", "wpep" )
      ]
    );

    $this->display_table_end();
  }

}