<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;

class ShortCodeBuilder {

  use WPEP_Container_Configuration;

  public function setup() {
    if( isset( $_GET['wpep-shortcode-builder'] ) )
      add_action( 'admin_init', [ $this, '_ajax_display' ], 100 );
  }
  
  public function _ajax_display() {
    $this->display_builder(
      $_GET['wpep-shortcode-builder'],
      (
        isset( $_GET['wpep-sb-options'] )
          ? json_decode( stripslashes(urldecode( $_GET['wpep-sb-options'] ) ), true )
          : ( isset( $_GET['wpep-sb-options-base64'] ) ? json_decode( base64_decode( $_GET['wpep-sb-options-base64'] ), true ) : [] )
      ),
      ( isset( $_GET['wpep-sb-init'] )               ? intval( $_GET['wpep-sb-init'] ) : true ),
      ( isset( $_GET['wpep-sb-button-text'] )        ? urldecode( $_GET['wpep-sb-button-text'] ) : false ),
      ( isset( $_GET['wpep-sb-button-description'] ) ? urldecode( $_GET['wpep-sb-button-description'] ) : false ),
      ( isset( $_GET['wpep-sb-layout-options'] )     ? $_GET['wpep-sb-layout-options'] : true )
    );

    exit;
  }

  public function display_builder( $tag, $options = [], $init = true, $button_text = false, $button_description = false, $layout_options = true ) {
    $options = $this->_process_options( $options );

    echo '<form id="wpep-shortcode-builder" data-short-code="' . $tag . '">';

    $this->display_table_start();

    $this->display_table_textarea_field( __( "Shortcode", 'wpep' ), 'shortcode', false, 'wpep_shortcode' );

    $this->display_table_separator(
      __( "Options", 'wpep' ),
      '<p class="submit">
                    <span class="button button-primary wpep-shortcode-copy-and-close">' .
                      ( $button_text == false ? __( "Copy & Close", "wpep" ) : $button_text ) .
                    '</span>' .
                    ' - ' . ( $button_description == false ? __( " The shortcode will be copied to your clipboard", "wpep" ) : $button_description ) .
                 '</p>'
    );

    if( !is_array( $options ) || empty( $options ) ) {
      echo '<div class="wpep-notification" style="margin-top: 15px !important;">' . __( "There are no short-code specific options available.", 'wpep' ) . '</div>';
    } else {
      foreach( $options as $option_key => $option_value ) {
        if( !is_array( $option_value )  ) {
          $this->display_table_hidden_field( $option_key, $option_value );

          continue;
        }

        $description = ( isset( $option_value['description'] ) ? $option_value['description'] : false );

        $label = ( isset( $option_value['name'] ) ? $option_value['name'] : wpep_alias_to_label( $option_key ) );

        if( $option_value['type'] == 'shortcode_content' )
          $this->display_table_text_field(
            $label,
            $option_key,
            $description,
            'shortcode_inner_content',
            ( isset( $option_value['default'] ) ? $option_value['default'] : '' )
          );
        else if( $option_value['type'] == 'checkbox' )
          $this->display_table_checkbox_field( $label, $option_key, $description, ( isset( $option_value['is_checked'] ) ? $option_value['is_checked'] : null ) );
        else if( $option_value['type'] == 'select' )
          $this->display_table_select_field( $label, $option_key, $option_value['options'], $description );
        else if( $option_value['type'] == 'searchable_select' )
          $this->display_table_select_field( $label, $option_key, $option_value['options'], $description, 'wpep-select-field wpep-searchable-select-shortcode-builder' );
        else if( $option_value['type'] == 'multiple_select' )
          $this->display_table_multiple_select_field(
            $label, $option_key, $option_value['options'], $description, 'wpep-multiple-select-shortcode-builder'
          );
        else if( $option_value['type'] == 'color' )
          $this->display_table_color_field( $label, $option_key, $description );
        else if( $option_value['type'] == 'text' )
          $this->display_table_text_field( $label, $option_key, $description );
      }
    }

    if( $layout_options )
      $this->display_table_wrapper_options();

    $this->display_table_end();

    echo '</form>';

    if( $init )
      echo '<script type="text/javascript"> WPEPShortCodeBuilder.Init( jQuery("#wpep-shortcode-builder" ) ); </script>';
  }

  private function _process_options( $options ) {
    if( !is_array( $options ) )
      return [];

    $response = $options;

    foreach( $options as $key => $option ) {
      if( !is_string( $option ) )
        continue;

      if( strpos( $option, 'wpep_admin_shortcode_builder_' ) !== 0 )
        continue;

      if( !is_callable( $option ) )
        continue;

      unset( $response[$key] );

      $func_call = call_user_func( $option, $options );

      if( !is_array( $func_call ) )
        continue;

      $response = $response + $func_call;
    }

    return $response;
  }

  public function get_value( $field_key, $field_default = '' ) {
    return $field_default;
  }
  
}