<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses;

use WPEP\Controller as WPEP_Controller;

class Dashboard {

  public function display( $filter_by = [] ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_table_data( $entries_per_page, $current_page, $filter_by );

    wpep_admin_display_table(
      __( "Courses List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  public function download( $filter_by = [] ) {
    $table_information = $this->_get_table_data( 1000000, 1, $filter_by );

    do_action(
      'wpep_admin_download_table',
      'courses-list-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $entries_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $courses_offset = ( $current_page - 1 ) * $entries_per_page;

    $entries_query_information = WPEP_Controller::instance()->courseActivity->get_completed_history(
      $entries_per_page,
      $courses_offset,
      true,
      $filter_by
    );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'name'                  => __( "Course Name", 'wpep' ),
      'course_identification' => __( "Course Identification", "wpep" ),
      'course_lessons'        => [
        'type'  => 'digits',
        'value' => __( "Course Lessons", 'wpep' )
      ],
      'completion_rate'   => [
        'type'  => 'digits',
        'value' => __( "Completion Rate", 'wpep' )
      ],
      'users_progressing' => [
        'type'  => 'digits',
        'value' => __( "Users progressing", 'wpep' )
      ],
      'users_completed'   => [
        'type'  => 'digits',
        'value' => __( "Users completed", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['course_identification'] );

    foreach( $entries_query_information['entries'] as $entry ) {

      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $entry['course_id'] ] ) . '">' . $entry['course_title']    . '</a>',
          'value_raw' => $entry['course_title']
        ],
        'course_lessons'        => [
          'type'      => 'digits',
          'value'     => $entry['course_lessons']
        ],
        'completion_rate' => [
          'type'      => 'digits',
          'value'     => wpep_get_percentage( $entry['course_users_completed'], $entry['course_users_started'] )
        ],
        'users_progressing'      => [
          'type'      => 'digits',
          'value'     => ( $entry['course_users_started'] - $entry['course_users_completed'] )
        ],
        'users_completed'         => [
          'type'      => 'digits',
          'value'     => $entry['course_users_completed']
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $entry['course_id'] ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ]
      ];

      if( wpep_has_sid_support() )
        $body['course_identification'] = get_post_meta( $entry['course_id'], 'wpep_course_identifier', true );

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}