<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses\Single;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as Entity_Course;

class Dashboard {

  public function get_alias() {
    return 'dashboard';
  }

  public function get_name() {
    return __( "Course Content", "wpep" );
  }

  public function get_file_alias( $course_id, $lesson_id = false, $filter_by = [] ) {
    return 'course-' . get_the_title( $course_id ) . '-' .
            ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) .
            wpep_format_date();
  }

  public function get_table_data( $course_id, $lesson_id = false, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $course_map = Entity_Course::instance()->get_with_lessons( $course_id );

    if( empty( $course_map ) )
      return $table_information;

    $course_started = wpep_utility_map_to_array_assoc(
      WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_STARTED,
        $course_id,
        false,
        false,
        false,
        false
      ), 'user_id', 'value' );

    $course_started = wpep_user_filter_map_entries( $course_started, $filter_by );

    $course_started_count = count( $course_started );

    global $wpdb;

    $table_information['head'] = [
      'section_name'            => __( "Section", 'wpep' ),
      'lesson_name'             => __( "Lesson", 'wpep' ),
      'median_previous'         => [
        'type'  => 'digits',
        'value' => __( "Lesson Time", 'wpep' )
      ],
      'median_total'            => [
        'type'  => 'digits',
        'value' => __( "Total Time", 'wpep' )
      ],
      'completion'              => [
        'type'  => 'digits',
        'value' => __( "Completed", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    $users_completed_previous = null;

    foreach( $course_map as $section_information ) {
      $table_information['body'][] = [
        'section_name'            => $section_information->title,
        'lesson_name'             => '',
        'median_previous'         => '',
        'median_total'            => '',
        'completion'              => '',
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => ''
        ],
      ];

      foreach( $section_information->lessons as $lesson_information ) {
        $users_completed = wpep_utility_map_to_array_assoc( $wpdb->get_results(
          'SELECT user_id, created_at FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed WHERE lesson_id = ' . $lesson_information->id
        ), 'user_id', 'created_at' );

        $users_completed = wpep_user_filter_map_entries( $users_completed, $filter_by );

        foreach( $users_completed as $k => $v )
          $users_completed[$k] = strtotime( $v ) - ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );

        $users_completed_count = count( $users_completed );

        $table_information['body'][] = [
          'section_name'            => '',
          'lesson_name'             => $lesson_information->title,
          'median_previous'         => [
            'type'  => 'digits',
            'value' => ( $users_completed_count == 0 ? '-' : $this->_get_median_previous( ( $users_completed_previous === null ? $course_started : $users_completed_previous ), $users_completed ) )
          ],
          'median_total'            => [
            'type'  => 'digits',
            'value' => ( $users_completed_count == 0 ? '-' : $this->_get_median_previous( $course_started, $users_completed ) )
          ],
          'completion'              => [
            'type'  => 'digits',
            'value' => $users_completed_count . ' ( ' . wpep_get_percentage( $users_completed_count, $course_started_count ) . ' ) '
          ],
          'view_submissions'        => [
            'type'  => 'link-item',
            'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id, 'lesson_id' => $lesson_information->id, 'view' => 'lesson_user_list' ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
          ],
        ];

        $users_completed_previous = $users_completed;
      }
    }

    return $table_information;
  }

  private function _get_median_previous( $course_started, $user_lesson_completed ) {
    foreach( $course_started as $key => $cs )
      if( !isset( $user_lesson_completed[ $key ] ) )
        unset( $course_started[ $key ] );

    // Retro-active lesson adds makes this required.
    foreach( $user_lesson_completed as $key => $cs )
      if( !isset( $course_started[ $key ] ) )
        unset( $user_lesson_completed[ $key ] );

    if( empty( $course_started ) || empty( $user_lesson_completed ) )
      return '-';

    $time = ( array_sum( $user_lesson_completed ) - array_sum( $course_started ) );

    $time /= count( $user_lesson_completed );

    if( $time < 0 ) {
      return __( 'previously completed', "wpep" );
    }

    if( $time < 60 )
      return sprintf( _n( '%s second', '%s seconds', $time, 'wpep' ), number_format_i18n( $time, 0 ) );

    $time /= 60;

    if( $time < 60 )
      return sprintf( _n( '%s minute', '%s minutes', $time, 'wpep' ), number_format_i18n( $time, 0 ) );

    $time /= 60;

    if( $time < 24 )
      return sprintf( _n( '%s hour', '%s hours', $time, 'wpep' ), number_format_i18n( $time, 2 ) );

    $time /= 24;

    return sprintf( _n( '%s day', '%s days', $time, 'wpep' ), number_format_i18n( $time, 2 ) );
  }

}