<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses\Single;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as Entity_Course;

class LessonUserList {

  public function is_public() {
    return false;
  }

  public function get_alias() {
    return 'lesson_user_list';
  }

  public function get_name( $course_id, $lesson_id ) {
    return Entity_Course::instance()->get_lesson( $lesson_id )->title;
  }

  public function get_file_alias( $course_id, $lesson_id = false, $filter_by = [] ) {
    return 'course-' . get_the_title( $course_id ) . '-' .
          Entity_Course::instance()->get_lesson( $lesson_id )->title . '-' .
          ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) .
          'user-list-' . wpep_format_date();
  }

  public function get_table_data( $course_id, $lesson_id = false, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $user_map = WPEP_Controller::instance()->courseActivity->get_course_started_users( $course_id );

    $user_map = wpep_user_filter_map_entries( $user_map, $filter_by );

    if( empty( $user_map ) || $lesson_id == false )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Name", 'wpep' ),
      'student_identification'  => __( "SID", 'wpep' ),
      'email_address'           => __( "Email Address", 'wpep' ),
      'lesson_completed'        => __( "Lesson Completed", 'wpep' ),
      'course_started'          => __( "Course Started", "wpep" )
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    global $wpdb;

    foreach( $user_map as $user_id => $started_at ) {
      $user_information = get_user_by( 'ID', $user_id );

      if( $user_information == NULL )
        continue;

      $lesson_completed = $wpdb->get_row(
        'SELECT created_at FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed WHERE user_id = ' . $user_id . ' AND lesson_id = ' . $lesson_id
      );

      if( $lesson_completed !== null ) {
        $lesson_completed = $lesson_completed->created_at;
        $lesson_completed = strtotime( $lesson_completed ) - ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
      }

      $body = [
        'name'                    => wpep_content_user_profile_format_name( $user_information ),
        'email_address'           => $user_information->user_email,
        'lesson_completed'        => ( $lesson_completed !== null ? wpep_format_date( $lesson_completed ) : '-' ),
        'course_started'          => wpep_format_date( $started_at )
      ];

      if( wpep_has_sid_support() )
        $body['student_identification'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      $table_information['body'][] = $body;
    }



    return $table_information;
  }

}