<?php

namespace WPEP\Admin\Coordinator\Statistics;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class Overview extends Skeleton {

  public $tabs = [
    'weekly'  => 'Weekly',
    'monthly' => 'Monthly',
    'yearly'  => 'Yearly'
  ];

  public function get_name() {
    return __( 'Overview', "wpep" );
  }

  public function get_alias() {
    return 'overview';
  }

  public function display( $filter_by = []  ) {
    $current_view_mode = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : 'weekly' );

    $this->display_tabs( $current_view_mode );

    if( isset( $filter_by['user_role'] ) && $filter_by['user_role'] != false ) {
      echo '<br/>';
      echo '<div class="wpep-notification wpep-notification-info">' . __( "This Statistics Page cannot be limited to your User Role Filter, you'll see all entries.", "wpep" ). '</div>';
    }

    $chart_labels = [];
    $chart_data   = [];

    $this->_setup_chart_data( $chart_labels, $chart_data, $current_view_mode );

    $chart_data_string = '{';

    foreach( $chart_data as $key => $data )
      $chart_data_string .= '"' . $key . '" : [' . implode( ", ", $data ) . '],';

    $chart_data_string .= '}';

    echo '<div id="wpep-statistics-primary-chart"></div>';
    echo '<script type="text/javascript" src="' . WPEP_BASE_URL_PATH . 'assets/admin/chart.min.js' . '"></script>';
    echo '<script type="text/javascript" src="' . WPEP_BASE_URL_PATH . 'assets/admin/chart-helper-min.js' . '"></script>';
    echo '<script type="text/javascript">
            jQuery( document ).ready( function() {
              WPEPStatisticsChartHandler.Init( "#wpep-statistics-primary-chart", ["' . implode( '","', $chart_labels ) . '"], ' . $chart_data_string . ' );
            } );
          </script>';
  }

  private function _setup_chart_data( array &$chart_labels, array &$chart_data, $chart_time_map = 'weekly', $date_format = 'l jS' ) {
    $active_users      = [];
    $courses_started   = [];
    $lessons_completed = [];
    $assessment_pending_review = [];
    $assessment_completed = [];
    $assessment_failed    = [];

    if( !is_array( $chart_time_map ) )
      $this->_setup_chart_data_map( $chart_time_map, $date_format );

    foreach( $chart_time_map as $time_from => $time_to ) {
      $active_users[ $time_from ]      = count( WPEP_Controller::instance()->courseActivity->get_period_active_user_ids( $time_from, $time_to ) );
      $courses_started[ $time_from ]   = count( WPEP_Controller::instance()->courseActivity->get_period_started_courses_list( $time_from, $time_to ) );
      $lessons_completed[ $time_from ] = WPEP_Controller::instance()->courseActivity->get_period_completed_lessons_count( $time_from, $time_to );

      $chart_labels[ $time_from ] = date( $date_format, $time_from );

      if( wpep_get_setting_flag( 'enable-course-assessments' ) ) {
        $assessment_pending_review[ $time_from ] = WPEP_Controller::instance()->assessmentActivity->get_period_status_count( $time_from, $time_to, WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW );
        $assessment_completed[ $time_from ]      = WPEP_Controller::instance()->assessmentActivity->get_period_status_count( $time_from, $time_to, WPEP_USER_ASSESSMENT_STATUS_COMPLETED );
        $assessment_failed[ $time_from ]         = WPEP_Controller::instance()->assessmentActivity->get_period_status_count( $time_from, $time_to, WPEP_USER_ASSESSMENT_STATUS_FAILED );
      }
    }

    $chart_data = [
      'Active Users'      => $active_users,
      'Courses Started'   => $courses_started,
      'Lessons Completed' => $lessons_completed,
    ];

    if( wpep_get_setting_flag( 'enable-course-assessments' ) ) {
      $chart_data['Assessments to Review'] = $assessment_pending_review;
      $chart_data['Assessments Completed'] = $assessment_completed;
      $chart_data['Assessments Failed'] = $assessment_failed;
    }

    $chart_data = apply_filters( 'wpep_administration_statistics_chart_data', $chart_data );
  }

  /**
   * @param $chart_time_map
   * @param $date_format
   */
  private function _setup_chart_data_map( &$chart_time_map, &$date_format ) {

    if( $chart_time_map == 'weekly' ) {
      $chart_time_map = [];

      for ( $iDay = 6; $iDay >= 0; $iDay-- ) {
        $day = strtotime("-" . $iDay . " day");
        $chart_time_map[$day] = $day;
      }

      $date_format       = 'l jS';

      return;
    }

    if( $chart_time_map == 'monthly' ) {
      $chart_time_map = [];

      for ( $iMonth = 11; $iMonth >= 0; $iMonth-- ) {
        $cmt = strtotime( "-" . $iMonth . " month" );

        $chart_time_map[ mktime (0, 0, 0, date( "m", $cmt ), 1, date( "Y", $cmt ) ) ] = mktime (0, 0, 0, date( "m", $cmt ), date( 't', $cmt ), date( "Y", $cmt ) );
      }

      $date_format       = 'F Y';

      return;
    }

    if( $chart_time_map == 'yearly' ) {
      $chart_time_map = [];

      for ( $iYear = date("Y") - 3; $iYear <= date("Y"); $iYear++ )
        $chart_time_map[ strtotime( $iYear . '-01-01' ) ] = strtotime( $iYear . '-12-31' );

      $date_format       = 'Y';

      return;
    }

  }

}
