<?php
namespace WPEP\Coordinator;

class AddOns {

  private $_store_id_addons_alias_map = false;

  public $addOns         = [];
  public $addOnSettings  = [];

  public $addOns_missing_dependencies = [];

  /**
   * @param $alias
   * @return bool|\WPEP_AddOn_Integration
   */
  public function get_add_on($alias) {
    return isset( $this->addOns[$alias] ) ? $this->addOns[$alias] : false;
  }

  public function get_add_on_by_store_id( $store_id ) {
    if( $this->_store_id_addons_alias_map === false ) {
      $this->_store_id_addons_alias_map = [];

      foreach( $this->addOns as $addon ) {
        $addon_information = $addon->get_information();

        if( !isset( $addon_information[ 'official' ] ) || ! $addon_information[ 'official' ] )
          continue;

        if( !isset( $addon_information[ 'store_id' ] ) )
          continue;

        $this->_store_id_addons_alias_map[ $addon_information[ 'store_id' ] ] = $addon->get_alias();
      }
    }

    if( isset( $this->_store_id_addons_alias_map[ $store_id ] ) )
      return $this->get_add_on( $this->_store_id_addons_alias_map[ $store_id ] );

    return false;
  }

  public function get_add_on_list() {
    return $this->addOns;
  }

  /**
   * @param $alias
   * @return bool|\WPEP_AddOn_Settings_Integration
   */
  public function get_add_on_settings($alias) {
    return isset( $this->addOnSettings[$alias] ) ? $this->addOnSettings[$alias] : false;
  }

  public function get_add_on_settings_list() {
    return $this->addOnSettings;
  }

  public function register( \WPEP_AddOn_Integration $addOn ) {
    $this->addOns[$addOn->get_alias()] = $addOn;
  }

  public function register_settings_page( \WPEP_AddOn_Settings_Integration $addOnSettings ) {
    $this->addOnSettings[$addOnSettings->get_alias()] = $addOnSettings;
  }

  public function _init_add_ons() {
    foreach( $this->addOns as $addOn ) {
      if( $addOn->has_requirements_met() !== true ) {
        $this->addOns_missing_dependencies[] = $addOn->get_alias();

        continue;
      }

      $addOn->init();
    }
  }

}