<?php
namespace WPEP\Coordinator\Ajax;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class PostComments {

  public function __construct() {
    add_action( 'wp_ajax_wpep_post_comments_post', [ $this, 'post_comment' ] );
    add_action( 'wp_ajax_wpep_post_comments_list', [ $this, 'list_comments' ] );
  }

  public function post_comment() {
    if( !isset( $_POST[ 'namespace' ] )
        || !isset( $_POST[ 'post_id' ] )
        || !isset( $_POST[ 'comment_parent' ] )
        || !isset( $_POST[ 'content' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace      = $_POST[ 'namespace' ];
    $post_id        = intval( $_POST[ 'post_id' ] );
    $comment_parent = intval( $_POST[ 'comment_parent' ] );
    $content        = $_POST[ 'content' ];

    $integration = WPEP_Controller::instance()->postComments->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Post Comments namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_post( $post_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ] ) );

    $content = stripcslashes( $content );
    $comment_type = isset( $_POST['comment_type'] ) ? $_POST['comment_type'] : '';
    $comment_type = $integration->has_comment_type( $comment_type ) ? $comment_type : '';

    $comment_id = wp_insert_comment( [
      'user_id'         => get_current_user_id(),
      'comment_parent'  => $comment_parent,
      'comment_post_ID' => $post_id,
      'comment_content' => $content,
      'comment_approved'=> 1,
      'comment_type'    => $comment_type
    ] );

    if( $comment_id === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Comment could not be saved.", "wpep" )
      ] ) );

    update_user_meta( get_current_user_id(), 'wpep_post_comments_interaction_'. $post_id, time() );

    $comment = get_comment( $comment_id );

    $integration->_after_comment_inserted( $comment, $post_id, get_current_user_id() );

    exit ( json_encode( [
      'status'      =>  'ok',
      'content'     => wpep_post_comments_template_callback_content( $comment ) . '</li>',
      'parent_id'   => $comment_parent,
      'comment_id'  => $comment_id
    ] ) );
  }

  public function list_comments() {
    if( !isset( $_POST[ 'namespace' ] ) || !isset( $_POST[ 'post_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace      = $_POST[ 'namespace' ];
    $post_id        = intval( $_POST[ 'post_id' ] );

    $integration = WPEP_Controller::instance()->postComments->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Post Comments namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $post_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ] ) );

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => $integration->get_comments_template( $post_id, true )
    ] ) );
  }

}