<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WP_Query;

class AssessmentActivity {

  public function setup() {
    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return;

    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_question', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_question_answer', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_user_data', [ 'post_id' => $post_id ] );
  }

  /**
   * @param $date_from
   * @param $date_to
   * @param $status
   * @return array of user_ids
   */
  public function get_period_status_count( $date_from = false, $date_to = false, $status = false ) {
    global $wpdb;

    $where = 'aud.key = "status" ';

    if( ( $date_from != false && $date_to != false ) ) {
      $date_from = ( is_numeric( $date_from ) ? date ( "Y-m-d", $date_from ) : $date_from );
      $date_to   = ( is_numeric( $date_to )   ? date ( "Y-m-d", $date_to )   : $date_to   );

      $where .= ' AND ' . (
        $date_from == $date_to ?
          'DATE( aud.created_at ) = "' . $date_from . '"' :
          'DATE( aud.created_at ) >= "' . $date_from . '" AND DATE( aud.created_at ) <= "' . $date_to . '"'
        );
    }
    if( $status != false ) {
      $where .= ' AND aud.value = "' . $status . '"';
    }

    $sql = 'SELECT aud.value as status, COUNT( aud.value ) as entries
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE ' . $where . '
             GROUP BY aud.value';

    if( $status != false ) {
      $data = $wpdb->get_row( $sql );

      if( $data === null )
        return 0;

      return $data->entries;
    }

    $response = [];

    foreach( $wpdb->get_results( $sql ) as $data )
      $response[ $data->status ] = $data->entries;

    return $response;
  }

  public function get_assessment_status_count( $assessment_id = false, $user_id = false, $filter_by = [] ) {
    global $wpdb;

    $where = ' aud.key = "status" ';

    if( $assessment_id != false )
      $where .= ' AND aud.post_id = ' . intval( $assessment_id );

    if( $user_id != false )
      $where .= ' AND aud.user_id = ' . intval ( $user_id );
    
    $sql = 'SELECT aud.value as status,
                   aud.user_id as user_id
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE ' . $where . '
             GROUP BY aud.value';

    $response = [
      WPEP_USER_ASSESSMENT_STATUS_PENDING        => 0,
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => 0,
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => 0,
      WPEP_USER_ASSESSMENT_STATUS_FAILED         => 0
    ];

    foreach( $wpdb->get_results( $sql ) as $data ) {
      if( !wpep_user_filter_requirement_met( $data->user_id, $filter_by ) )
        continue;

      $response[ $data->status ]++;
    }

    return $response;
  }

  public function get_assessment_average_score( $assessment_id, $filter_by = []  ) {
    global $wpdb;

    $where = ' aud.post_id = ' . $assessment_id . ' AND aud.key = "score" ';

    $sql = 'SELECT aud.value,
                   aud.user_id
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE ' . $where;

    $entries = $wpdb->get_results( $sql );

    $entries = wpep_utility_map_to_array_assoc( $entries, 'user_id', 'value' );

    $entries = wpep_user_filter_map_entries( $entries, $filter_by );

    if( empty( $entries ) )
      return 0;

    return array_sum( $entries ) / count( $entries );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param array $filter_by
   * @return array
   */
  public function get_submission_history( $interaction_number = 30, $interaction_offset = 0, $pagination_information = false, $filter_by = [] ) {
    $assessment_list_query = new WP_Query( [
      'post_type'       => WPEP_POST_TYPE_ASSESSMENT,
      'posts_per_page'  => $interaction_number,
      'offset'          => $interaction_offset,
      'orderby'         => 'title',
      'order'           => 'ASC',
      'author'          => wpep_user_statistics_query_author_id()
    ] );

    $assessment_list = $assessment_list_query->get_posts();

    $response = [];

    foreach( $assessment_list as $assessment ) {
      $assessment_submissions = $this->get_assessment_status_count( $assessment->ID, false, $filter_by );

      $response[ $assessment->ID ] = [
        'assessment_id'     => $assessment->ID,
        'assessment_title'  => $assessment->post_title,
        'average_score'     => $this->get_assessment_average_score( $assessment->ID, $filter_by ),
        'submissions_total' => array_sum( $assessment_submissions )
      ];

      foreach( $assessment_submissions as $assessment_status => $assessment_status_entries )
        $response[$assessment->ID][ 'submissions_' . $assessment_status ] = $assessment_status_entries;
    }

    if( $pagination_information == false )
      return $response;

    return [
      'entries_total'   => $assessment_list_query->found_posts,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $response,
    ];
  }

}