<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings  as WPEP_Settings;

class Content {

  /**
   * @var Content\Primary
   */
  public $primaryCoordinator;
  /**
   * @var Content\Single
   */
  public $singleCoordinator;
  /**
   * @var Content\UserProfile
   */
  public $userProfileCoordinator;
  /**
   * @var Content\Assessment
   */
  public $assessmentCoordinator;

  /**
   * @var Content\EBook
   */
  public $ebookCoordinator;

  public $wpep_wrapper_content_depth = 0;

  public function setup() {
    add_action( 'wpep_before_main_content', [ $this, 'before_main_content' ] );
    add_action( 'wpep_after_main_content', [ $this, 'after_main_content' ] );

    $this->primaryCoordinator     = new Content\Primary();
    $this->singleCoordinator      = new Content\Single();
    $this->userProfileCoordinator = new Content\UserProfile();
    $this->assessmentCoordinator  = new Content\Assessment();
    $this->ebookCoordinator       = new Content\EBook();
  }

  public function before_main_content() {
    $this->wpep_wrapper_content_depth++;

    if( $this->wpep_wrapper_content_depth > 1 )
      return;

    echo '<div class="wpep-wrapper' . ( wpep_is_rtl() ? ' wpep-is-rtl' : '' ) . '" style="' . WPEP_Controller::instance()->templateStyle->get_wpep_wrapper_style() . '"><div class="wpep-container" style="clear:both;">';
  }

  public function after_main_content() {
    $this->wpep_wrapper_content_depth--;

    if( $this->wpep_wrapper_content_depth >= 1 )
      return;

    echo '</div></div>';
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function get_courses( $course_categories = [] ) {
    $post_list = get_posts( $this->query_args_courses( $course_categories ) );

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $post_list = wpep_post_list_ordering( $post_list );

    return $post_list;
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function query_args_courses( $course_categories = [] ) {
    $args = [
      'post_type'   => WPEP_POST_TYPE_COURSE,
      'post_status' => 'publish',
      'order'       => 'ASC',
      'orderby'     => 'meta_value',
      'meta_key'    => 'wpep_order',
      'meta_type'   => 'NUMERIC',
      'nopaging'    => true,
      'meta_query'  => [
        'wpep_show_on_index' => [
          'key'     => 'wpep_show_on_index',
          'value'   => '1',
          'compare' => '=',
        ]
      ]
    ];

    if ( !empty( $course_categories ) ) {
      $args['tax_query'] = [
        WPEP_TAXONOMY_CATEGORY => [
          'taxonomy' => WPEP_TAXONOMY_CATEGORY,
          'field'    => 'term_id',
          'terms'    => $course_categories
        ]
      ];
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $args['post_status'] = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ];

    return apply_filters('wpep_primary_content_course_list_args', $args );
  }

  public function get_videos( $course_categories = [] ) {
    $post_list = get_posts( $this->query_args_videos( $course_categories ) );

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $post_list = wpep_post_list_ordering( $post_list );

    return wpep_utilities_map_object( $post_list );
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function query_args_videos( $course_categories = [] ) {
    $args = [
      'post_type'   => WPEP_POST_TYPE_VIDEO,
      'post_status' => 'publish',
      'order'       => 'ASC',
      'orderby'     => 'meta_value',
      'meta_key'    => 'wpep_order',
      'meta_type'   => 'NUMERIC',
      'nopaging'    => true
    ];

    if ( !empty( $course_categories ) ) {
      $args['tax_query'] = [
        WPEP_TAXONOMY_CATEGORY => [
          'taxonomy' => WPEP_TAXONOMY_CATEGORY,
          'field'    => 'term_id',
          'terms'    => $course_categories
        ]
      ];
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $args['post_status'] = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ];

    return apply_filters('wpep_primary_content_video_list_args', $args );
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function get_ebooks( $course_categories = [] ) {
    $post_list = get_posts( $this->query_args_ebooks( $course_categories ) );

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $post_list = wpep_post_list_ordering( $post_list );

    return $post_list;
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function query_args_ebooks( $course_categories = [] ) {
    $args = [
      'post_type'   => WPEP_POST_TYPE_E_BOOK,
      'post_status' => 'publish',
      'order'       => 'ASC',
      'orderby'     => 'meta_value',
      'meta_key'    => 'wpep_order',
      'meta_type'   => 'NUMERIC',
      'nopaging'    => true
    ];

    if ( !empty( $course_categories ) ) {
      $args['tax_query'] = [
        WPEP_TAXONOMY_CATEGORY => [
          'taxonomy' => WPEP_TAXONOMY_CATEGORY,
          'field'    => 'term_id',
          'terms'    => $course_categories
        ]
      ];
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $args['post_status'] = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ];

    return apply_filters('wpep_primary_content_ebook_list_args', $args );
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function get_offers( $course_categories = [] ) {
    $post_list = get_posts( $this->query_args_offers( $course_categories ) );

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $post_list = wpep_post_list_ordering( $post_list );

    return $post_list;
  }

  /**
   * @param array|string $course_categories
   * @return array
   */
  public function query_args_offers( $course_categories ) {
    $args = [
      'post_type'   => WPEP_POST_TYPE_OFFER,
      'post_status' => 'publish',
      'order'       => 'ASC',
      'orderby'     => 'meta_value',
      'meta_key'    => 'wpep_order',
      'meta_type'   => 'NUMERIC',
      'nopaging'    => true
    ];

    if ( !empty( $course_categories ) ) {
      $args['tax_query'] = [
        WPEP_TAXONOMY_CATEGORY => [
          'taxonomy' => WPEP_TAXONOMY_CATEGORY,
          'field'    => 'term_id',
          'terms'    => $course_categories
        ]
      ];
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $args['post_status'] = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ];

    return apply_filters('wpep_primary_content_offer_list_args', $args );
  }

}
