<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as EntityAssessment;

class Assessment {

  public $assessment_id = 0;
  public $course_id     = 0;
  public $section_id    = 0;

  public function __construct() {
    if( is_admin() && !( defined( 'DOING_AJAX' ) && DOING_AJAX ) )
      return;

    add_action( 'init', [ $this, '_wp_init' ] );
    add_action( 'wpep_single_assessment_content', [ $this, 'display_content' ], 10 );
    add_action( 'wpep_current_user_assessment_information_box', [ $this, 'display_information_box' ] );
  }

  public function _wp_init() {
    if( wpep_get_current_post_type() != WPEP_POST_TYPE_ASSESSMENT )
      return;

    $this->assessment_id = wpep_get_current_post_id();

    if( $this->assessment_id === 0 )
      return;

    $this->course_id  = wpep_get_assessment_course_id( $this->assessment_id );
    $this->section_id = wpep_get_assessment_course_section_id( $this->assessment_id );

    $previous_assessments = wpep_user_assessment_requirements_for_section( $this->section_id );

    if( is_array( $previous_assessments ) && !empty( $previous_assessments ) ) {
      $previous_assessment = array_shift( $previous_assessments );

      wp_redirect( get_permalink( $previous_assessment ) );
      exit;
    }

    if( isset( $_GET['retake_assessment'] ) ) {
      if( wp_verify_nonce( $_GET['retake_assessment'], 'retake_assessment' ) != 0 ) {
        WPEP_Controller::instance()->userActivity->assessment->delete_user_data(
          false,
          $this->assessment_id,
          false,
          false,
          get_current_user_id()
        );

        wp_redirect( get_permalink( $this->assessment_id ) );
        exit;
      }
    }

    $course_theme    = get_post_meta( $this->course_id, 'course_theme', true );

    if( $course_theme !== '' && !WPEP_Controller::instance()->templateStyle->custom_style_delivered )
      WPEP_Controller::instance()->templateStyle->theme = $course_theme;

    if( !isset( $_POST['wpep_assessment_id'] ) )
      return;

    if( intval( $_POST['wpep_assessment_id'] ) != $this->assessment_id )
      return;

    $this->_handle_answers_save();

    $question_number = isset( $_GET['step'] ) ? $_GET['step'] : 0;

    if( isset( $_POST['wpep_prev_part'] ) ) {
      if( $question_number == 1 )
        wp_redirect( get_permalink( $this->assessment_id ) );
      else
        wp_redirect( add_query_arg( 'step', $question_number - 1, get_permalink( $this->assessment_id ) ) );

      exit;
    }

    if( isset( $_POST['wpep_next_part'] ) ) {
      wp_redirect( add_query_arg( 'step', $question_number + 1, get_permalink( $this->assessment_id ) ) );
      exit;
    }
  }

  private function _handle_answers_save() {
    if( !isset( $_POST['wpep_assessment_answer'] ) )
      return;

    $current_request_answers = $_POST['wpep_assessment_answer'];
    $user_status = wpep_user_assessment_status( $this->assessment_id );

    if( !wpep_allow_assessment_submission( $this->assessment_id, $user_status ) )
      return;

    $allowed_time = intval( get_post_meta( $this->assessment_id, 'wpep_available_minutes', true ) );

    if( $allowed_time != 0 ) {
      $start_time = WPEP_Controller::instance()->userActivity->assessment->get_user_data(
        WPEP_USER_ASSESSMENT_START_TIME, $this->assessment_id, 0, 0, get_current_user_id(), true
      );

      $allowed_time *= 60;

      if( $start_time !== null )
        if( intval( $start_time->value ) - time() + $allowed_time + 20 <= 0 )
          return;
    }

    $internal_question_list = wpep_utilities_map_object( EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, false, false ), 'id' );
    $internal_answer_list   = EntityAssessment::instance()->get_answers_by_assessment_id( $this->assessment_id );
    $question_type_data     = wpep_get_assessment_field_types();

    foreach( $current_request_answers as $current_request_question_id => $current_request_answer ) {
      if( !isset( $question_type_data[$internal_question_list[$current_request_question_id]->type] ) )
        continue;

      $current_question_type_data = $question_type_data[$internal_question_list[$current_request_question_id]->type];

      if( $current_question_type_data['multiple_answers'] ) {
        WPEP_Controller::instance()->userActivity->assessment->delete_user_data( 'answer', $this->assessment_id, $current_request_question_id, false, get_current_user_id() );

        foreach( $current_request_answer as $current_question_answer ) {
          $answer_id  = $internal_answer_list[$current_question_answer]->id;

          WPEP_Controller::instance()->userActivity->assessment->add_user_data(
            'answer', $current_question_answer, $this->assessment_id,
            $current_request_question_id, $answer_id, get_current_user_id()
          );
        }
      } else if( $current_question_type_data['has_answers'] ) {

        if( isset( $internal_answer_list[$current_request_answer] ) ) {
          $answer_id = $internal_answer_list[$current_request_answer]->id;
          $current_request_answer = $internal_answer_list[$answer_id]->title;
        } else {
          $answer_id = 0;

          foreach( $internal_answer_list as $answer_item ) {
            if( $current_request_question_id != $answer_item->question_id )
              continue;

            if( strtolower( $answer_item->title ) != strtolower( $current_request_answer ) )
              continue;

            $answer_id = $answer_item->id;
          }
        }

        WPEP_Controller::instance()->userActivity->assessment->delete_user_data( 'answer', $this->assessment_id, $current_request_question_id, false, get_current_user_id() );
        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          'answer', $current_request_answer, $this->assessment_id,
          $current_request_question_id, $answer_id, get_current_user_id()
        );
      } else {
        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          'answer', $current_request_answer, $this->assessment_id,
          $current_request_question_id, 0, get_current_user_id()
        );
      }

    }

    if( isset( $_POST['wpep_assessment_submit'] ) ) {
      WPEP_Controller::instance()->userActivity->assessment->set_user_data(
        WPEP_USER_ASSESSMENT_SUBMISSION_TIME, time(), $this->assessment_id, 0, 0, get_current_user_id()
      );

      WPEP_Controller::instance()->userActivity->assessment->verify_user_submission_answers( $this->assessment_id );

      wp_redirect( get_permalink( $this->assessment_id ) );
      exit;
    } else {
      WPEP_Controller::instance()->userActivity->assessment->set_user_data(
        WPEP_USER_ASSESSMENT_STATUS, WPEP_USER_ASSESSMENT_STATUS_PENDING, $this->assessment_id, 0, 0, get_current_user_id()
      );
    }
  }

  public function display_content() {
    if( $this->assessment_id === 0 )
      return;

    $user_status  = wpep_user_assessment_status( $this->assessment_id );

    if( $this->maybe_display_success_message( $user_status ) )
      return;

    if( $this->maybe_display_failed_message( $user_status ) )
      return;

    $this->display_form();
  }

  public function maybe_display_success_message( $user_status ) {
    if( $user_status != WPEP_USER_ASSESSMENT_STATUS_COMPLETED )
      return false;

    if( get_post_meta( $this->assessment_id, 'wpep_enable_view_questions', true ) )
      if( isset( $_GET['step'] ) || ( isset( $_GET['display-questions'] ) && $_GET['display-questions'] == 1 ) )
        return false;

    echo WPEP_Controller::instance()->template->get_template(
      'assessment/success.php', $this->_get_message_template_data( $user_status )
    );

    return true;
  }

  public function maybe_display_failed_message( $user_status ) {
    if( $user_status != WPEP_USER_ASSESSMENT_STATUS_FAILED )
      return false;

    if( get_post_meta( $this->assessment_id, 'wpep_enable_view_questions', true ) )
      if( isset( $_GET['step'] ) || ( isset( $_GET['display-questions'] ) && $_GET['display-questions'] == 1 ) )
        return false;

    echo WPEP_Controller::instance()->template->get_template(
      'assessment/failed.php', $this->_get_message_template_data( $user_status )
    );
    return true;
  }

  private function _get_message_template_data( $user_status ) {
    $assessment_questions = EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, false, false );

    $failed_message = get_post_meta( $this->assessment_id, 'wpep_failed_message', true );

    if( $failed_message == '' )
      $failed_message = wpep_get_setting_translated( 'assessment-status-failed' );

    $grading_option = intval( get_post_meta( $this->assessment_id, 'wpep_grading_option', true ) );

    if( $grading_option != 0 ) {
      $success_visual_indicators = !get_post_meta( $this->assessment_id, 'wpep_success_message_disable_indicators', true );
      $failed_visual_indicators  = !get_post_meta( $this->assessment_id, 'wpep_failed_message_disable_indicators', true );
    } else {
      $success_visual_indicators = false;
      $failed_visual_indicators  = false;
    }

    return [
      'course_id'            => $this->course_id,
      'course_title'         => get_the_title( $this->course_id ),
      'course_permalink'     => apply_filters( 'wpep_course_link', get_permalink( $this->course_id ), $this->course_id ),
      'assessment_id'               => $this->assessment_id,
      'assessment_title'            => get_the_title( $this->assessment_id ),
      'assessment_view_permalink'   => add_query_arg( 'display-questions', 1, get_permalink( $this->assessment_id ) ),
      'assessment_retake_permalink' => wp_nonce_url( get_permalink( $this->assessment_id ), 'retake_assessment', 'retake_assessment' ),
      'user_status'               => $user_status,
      'failed_message'            => $failed_message,
      'success_message'           => get_post_meta( $this->assessment_id, 'wpep_success_message', true ),
      'enable_view_questions'     => ( get_post_meta( $this->assessment_id, 'wpep_enable_view_questions', true ) ),
      'allow_submission'          => wpep_allow_assessment_submission( $this->assessment_id, WPEP_USER_ASSESSMENT_STATUS_FAILED ),
      'question_count'   => count( $assessment_questions ),
      'correct_answers'  => count( WPEP_Controller::instance()->userActivity->assessment->get_user_data(
        'is_correct',
        $this->assessment_id,
        false,
        false,
        get_current_user_id(),
        false,
        1
      ) ),
      'score'          => WPEP_Controller::instance()->userActivity->assessment->get_user_data_value(
        WPEP_USER_ASSESSMENT_SCORE,
        $this->assessment_id,
        false,
        false,
        get_current_user_id()
      ),
      'grading_option' => $grading_option,
      'success_visual_indicators' => $success_visual_indicators,
      'failed_visual_indicators'  => $failed_visual_indicators
    ];
  }

  public function display_form() {
    $section_id       = get_post_meta( $this->assessment_id, 'wpep_section_id', true );
    $user_status      = wpep_user_assessment_status( $this->assessment_id );
    $allowed_time     = intval( get_post_meta( $this->assessment_id, 'wpep_available_minutes', true ) );
    $allow_submission = (
      in_array( $user_status, [ WPEP_USER_ASSESSMENT_STATUS_FAILED, WPEP_USER_ASSESSMENT_STATUS_COMPLETED ] )
        ? false
        : wpep_allow_assessment_submission( $this->assessment_id, $user_status )
    );
    $assessment_questions = EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, true, true );

    if( $allowed_time != 0 )
      $allowed_time *= 60;

    $start_time = WPEP_Controller::instance()->userActivity->assessment->get_user_data_value(
      WPEP_USER_ASSESSMENT_START_TIME, $this->assessment_id, 0, 0, get_current_user_id()
    );

    if( $start_time === false ) {
      if( !isset( $_POST['wpep_start_assessment'] )
          && intval( get_post_meta( $this->assessment_id, 'wpep_enable_welcome_page', true ) ) == 1 ) {
        echo WPEP_Controller::instance()->template->get_template( 'assessment/welcome.php', [
          'assessment_id'        => $this->assessment_id,
          'assessment_title'     => get_the_title( $this->assessment_id ),
          'allowed_time'         => $allowed_time,
          'course_id'            => $this->course_id,
          'course_title'         => get_the_title( $this->course_id ),
          'course_permalink'     => get_the_permalink( $this->course_id ),
          'question_count'       => count( $assessment_questions ),
          'score_requirement'    => (
            in_array( get_post_meta( $this->assessment_id, 'wpep_grading_option', true ), [ 2,3 ] ) ?
              intval( get_post_meta( $this->assessment_id, 'wpep_grading_requirement', true ) ) :
              0
            )
        ]);

        return;
      }

      $start_time = time();

      WPEP_Controller::instance()->userActivity->assessment->set_user_data(
        WPEP_USER_ASSESSMENT_STATUS, WPEP_USER_ASSESSMENT_STATUS_PENDING, $this->assessment_id, 0, 0, get_current_user_id()
      );

      WPEP_Controller::instance()->userActivity->assessment->set_user_data(
        WPEP_USER_ASSESSMENT_START_TIME, $start_time, $this->assessment_id, 0, 0, get_current_user_id()
      );
    }

    if( $allowed_time != 0 && ( $start_time + $allowed_time < time() ) ) {
      if( get_post_meta( $this->assessment_id, 'wpep_submission_restraint', true ) != 2 ) {
        $allow_submission = false;
      } else {
        $start_time = time();

        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          WPEP_USER_ASSESSMENT_START_TIME, $start_time, $this->assessment_id, 0, 0, get_current_user_id()
        );
      }
    }

    $questions_per_page = intval( get_post_meta( $this->assessment_id, 'wpep_paginate_questions', true ) );
    $user_answers = WPEP_Controller::instance()->userActivity->assessment->get_user_data(
      'answer', $this->assessment_id, false, false, get_current_user_id(), false
    );

    $template = $questions_per_page != 0 ? 'assessment/form-paginated.php' : 'assessment/form.php';
    $template_data = [
      'assessment_id'        => $this->assessment_id,
      'assessment_title'     => get_the_title( $this->assessment_id ),
      'form_action'          => add_query_arg( 'display-questions', 1, get_the_permalink( $this->assessment_id ) ),
      'assessment_questions' => $assessment_questions,
      'allowed_time'         => $allowed_time,
      'submission_started'   => $start_time,
      'course_id'            => $this->course_id,
      'course_title'         => get_the_title( $this->course_id ),
      'course_permalink'     => get_the_permalink( $this->course_id ),
      'section_id'           => $section_id,
      'user_status'          => $user_status,
      'allow_submission'     => $allow_submission,
      'user_question_status' => wpep_utility_map_to_array_assoc(
        WPEP_Controller::instance()->userActivity->assessment->get_user_data(
          'is_correct', $this->assessment_id, false, false, get_current_user_id(), false
        ),
        'question_id',
        'value'
      ),
      'user_question_answer' => wpep_utility_map_to_array_assoc(
        $user_answers,
        'question_id',
        'value',
        true
      ),
      'user_question_answer_id' => wpep_utility_map_to_array_assoc(
        $user_answers,
        'question_id',
        'question_answer_id',
        true
      )
    ];


    if( $questions_per_page != 0 ) {
      $step = intval( isset( $_GET['step'] ) ? $_GET['step'] : 0 );
      $offset = ( $step == 0 ? 0 : $step * $questions_per_page );

      $template_data['assessment_questions'] = array_slice( $assessment_questions, $offset, $questions_per_page );

      $i = 1; foreach( $template_data['assessment_questions'] as $index => $question ) {
        $template_data['assessment_questions'][$index]->question_number = $offset + $i;
        $i++;
      }

      $template_data['form_action']         = add_query_arg( 'step', $step, $template_data['form_action'] );
      $template_data['pagination_step']     = $step;
      $template_data['pagination_previous'] = ( $offset > 0 );
      $template_data['pagination_next']     = ( $questions_per_page + $offset < count( $assessment_questions ) );
    }

    echo WPEP_Controller::instance()->template->get_template( $template, $template_data );
    return;
  }

  public function display_information_box( $information ) {
    echo $this->get_information_box(
      $information[0],
      get_current_user_id(),
      ( isset( $information[1] ) ? $information[1] : false )
    );
  }

  public function get_information_box( $assessment_id, $user_id = false, $container_class = false ) {
    $user_id         = ( $user_id == false ? get_current_user_id() : $user_id );
    $container_class = 'wpep-assessment-information-box ' . ( $container_class == false ? '' : $container_class );

    if( $user_id == 0 || $assessment_id == 0 )
      return '';

    $user_status = wpep_user_assessment_status( $assessment_id, $user_id );

    $container_class .= ' wpep-assessment-status-' . $user_status;

    return WPEP_Controller::instance()->template->get_template( 'assessment/user-status-box.php', [
      'container_class'   => $container_class,
      'assessment_id'     => $assessment_id,
      'assessment_title'  => get_the_title( $assessment_id ),
      'assessment_url'    => get_the_permalink( $assessment_id ),
      'user_status'       => $user_status
    ] );
  }

}
