<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class EBook {

  public $current_id = false;

  public function __construct() {
    if( is_admin() && !( defined( 'DOING_AJAX' ) && DOING_AJAX ) )
      return;

    add_filter( 'init', [ $this, 'init' ], 15 );

    add_filter( 'the_content', [ $this, 'the_content_filter' ], 1 );
  }

  public function get_current_id() {
    if( $this->current_id === false )
      $this->current_id = wpep_get_current_post_id();

    return $this->current_id;
  }

  public function init() {
    if( get_post_type( $this->get_current_id() ) != WPEP_POST_TYPE_E_BOOK )
      return;

    if( wpep_post_has_member_magnet_option( $this->get_current_id(), get_current_user_id() ) ) {
      if( !isset( $_GET['wpep-login'] ) ) {
        wp_redirect( wpep_post_member_magnet_link( $this->get_current_id() ) );
        exit;
      }

      wpep_current_request_setup_member_magnet();

      return;
    }

    if( ( is_user_logged_in() && isset( $_GET['wpep-login'] ) ) || wpep_get_setting_flag( 'ebook-redirect-display' ) == false ) {
      wp_redirect( get_post_meta( $this->get_current_id(), '_ebook_link', true ) );
      exit;
    }
  }

  public function the_content_filter( $content ) {
    if( post_password_required() )
      return $content;

    if( get_post_type() !== WPEP_POST_TYPE_E_BOOK )
      return $content;

    $link  = get_post_meta( get_the_ID(), '_ebook_link', true );

    return '<div class="wpep-ebook-redirect-container">' .
              wpautop( wpep_get_setting_translated( 'ebook-redirect-message' ) ) .
              '<p class="wpep-link-container"><a href="' . $link . '">' . wpep_get_setting_translated( 'ebook-redirect-button' ) . '</a></p>' .
              '<script type="text/javascript">setTimeout( function() { window.location.href = "' . $link . '"; }, 3000 );</script>' .
            '</div>';
  }

}