<?php

namespace WPEP\Coordinator\Content;

class Primary {

  /**
   * @var Primary\CategoryList
   */
  public $categoryList;
  /**
   * @var Primary\GridList
   */
  public $gridList;
  /**
   * @var Primary\GridItem
   */
  public $gridItem;

  public function __construct() {
    if( is_admin() && !( defined( 'DOING_AJAX' ) && DOING_AJAX ) )
      return;

    $this->categoryList = new Primary\CategoryList();
    $this->gridList     = new Primary\GridList();
    $this->gridItem     = new Primary\GridItem();
  }

  public function has_categories( $categories = false ) {
    return apply_filters( "wpep_content_primary_has_categories",
      ( is_numeric( $categories ) ? $categories : count( ( !is_array( $categories ) ? $this->get_current_categories() : $categories ) ) ) >= 2
    );
  }

  public function has_categories_navigation( $categories = false, $category_requirement = true ) {
    $items_per_row = apply_filters(
      'wpep_template_category_list_items_per_row',
      wpep_get_setting( 'index-page-categories-per-row', 0 )
    );

    return apply_filters( 'wpep_primary_content_category_navigation_enabled', ( $items_per_row != 1 && ( !$category_requirement || $this->has_categories( $categories ) ) ) );
  }

  public function has_all_categories_navigation() {
    return apply_filters( 'wpep_primary_content_all_categories_navigation_enabled', wpep_get_setting_flag( 'index-page-enable-filter-all-categories' ) );
  }

  /**
   * @return array|\WP_Term
   */
  public function get_current_categories() {
    $category_list_args = apply_filters(
      'wpep_primary_content_category_list_args',
      [
        'taxonomy'   => 'course_category',
        'hide_empty' => false,
        'parent'     => 0,
        'orderby'    => 'meta_key',
        'meta_key'   => 'wpep_order'
      ]
    );

    $category_list = get_terms( $category_list_args );

    $category_list = apply_filters( 'wpep_primary_content_grid_categories', $category_list );

    if( count( $category_list ) == 1
      && apply_filters( 'wpep_primary_content_grid_categories_secondary_as_primary_view', wpep_get_setting_flag( 'index-page-single-category-primary-view' ) )
      && $this->has_categories_navigation( $category_list, false ) ) {
      $primary_category_id     = $category_list[ key( $category_list ) ]->term_id;
      $current_sub_categories  = get_term_children( $primary_category_id, 'course_category' );

      if( count( $current_sub_categories ) >= 2 )
        $category_list = get_terms( [ 'taxonomy' => 'course_category', 'include' => $current_sub_categories ] );

      foreach( $category_list as $category_item_key => $category_item )
        if($category_item->parent != $primary_category_id )
          unset( $category_list[ $category_item_key ] );
    }

    return $category_list;
  }

}
