<?php

namespace WPEP\Coordinator\Content\Primary;

use WPEP\Controller as WPEP_Controller;

class CategoryList {

  public function __construct() {
    add_filter( 'wpep_template_category_list_items_per_row', [ $this, 'category_list_items_per_row' ], 5 );
    add_action( 'wpep_primary_content_category_list', [ $this, 'output_category_list' ], 1 );
    add_action( 'wpep_primary_content_sub_category_list', [ $this, 'output_sub_category_list' ], 10, 1 );
  }

  public function category_list_items_per_row( $current_items_per_row ) {
    $items_per_row = wpep_get_setting( 'index-page-categories-per-row', 0 );

    return ( $items_per_row == 0 ? $current_items_per_row : $items_per_row );
  }

  public function output_category_list() {
    if( !WPEP_Controller::instance()->content->primaryCoordinator->has_categories_navigation() )
      return;

    echo $this->category_list();
  }

  public function category_list() {
    $category_list = WPEP_Controller::instance()->content->primaryCoordinator->get_current_categories();

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $category_list = wpep_term_list_ordering( $category_list );

    $resp = WPEP_Controller::instance()->template->get_template( 'primary/category-list.php', [
      'category_list'       => $category_list,
      'category_list_count' => count( $category_list )
    ] );

    return apply_filters( 'wpep_primary_content_category_list_html', $resp );
  }

  public function output_sub_category_list( $course_category ) {
    echo $this->sub_category_list( $course_category );
  }

  public function sub_category_list( $course_category ) {
    if( !isset( $course_category->term_id ) )
      return '';

    $category_children = get_terms( [ 'taxonomy' => 'course_category', 'hide_empty' => false, 'parent' => $course_category->term_id ] );

    if( empty( $category_children ) )
      return '';

    $resp = '';

    if( wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $category_children = wpep_term_list_ordering( $category_children );

    $resp .= '<div class="course-grid-sub-category-list" data-category-id="' . $course_category->term_id . '">';
    $resp .=    '<span>' . wpep_get_setting_translated( 'opt-course-secondary-filter' ) . '</span>';
    $resp .=    '<ul>';
    $resp .=      '<li data-display-category-id="0" class="cancel-filter wpep-hidden">' . __( "Cancel", "wpep" ) . '</li>';

    foreach( $category_children as $category_child ) {
      $sub_category_link = get_term_meta( $category_child->term_id, 'url', true );

      if( $sub_category_link !== '' ) {
        $resp .= '<li>';
        $resp .=  '<a href="' . $sub_category_link . '">' . $category_child->name . '</a>';
        $resp .= '</li>';
      } else {
        $resp .= '<li data-display-category-id="' . $category_child->term_id . '" class="trigger-filter">';
        $resp .=  $category_child->name;
        $resp .= '</li>';
      }
    }

    $resp .=    '</ul>';
    $resp .= '</div>';

    return $resp;
  }

}