<?php

namespace WPEP\Coordinator\Content\Primary;

use WPEP\Controller as WPEP_Controller;

class GridList {

  public function __construct() {
    add_action( 'wpep_primary_content_grid_list', [ $this, 'output_grid_list' ], 1 );
    add_action( 'wpep_primary_content_grid_courses_list', [ $this, 'output_grid_courses_list' ], 1 );
  }

  public function output_grid_list() {
    do_action( "wpep_primary_content_before_grid" );

    $course_categories = WPEP_Controller::instance()->content->primaryCoordinator->get_current_categories();

    $course_categories = is_array( $course_categories ) ? wpep_utility_map_to_array_assoc( $course_categories, 'term_id', 'term_id' ) : $course_categories;

    $this->output_grid_content_post_videos_html( $course_categories );
    $this->output_grid_content_post_ebooks_html( $course_categories );
    $this->output_grid_content_post_courses_html( $course_categories );
    $this->output_grid_content_post_offers_html( $course_categories );

    echo '<div class="clearfix"></div>';

    do_action( "wpep_primary_content_after_grid" );
  }

  private function output_grid_content_post_videos_html( $course_categories = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_videos_per_row', 1 );

    if( $items_per_row == 0 )
      return;

    $post_list = WPEP_Controller::instance()->content->get_videos( $course_categories );

    if( empty( $post_list ) )
      return;

    $front_post = ( get_post_type() == WPEP_POST_TYPE_VIDEO && isset( $post_list[get_the_ID()] ) ? $post_list[get_the_ID()] : false );

    echo '<div class="wpep-video-grid-items-container" data-items-per-row="' . $items_per_row . '">';

    echo wpep_content_primary_video_post_type_content( $front_post );

    $css_state_class = ( count( $post_list ) == 1 ? 'wpep-hidden' : 'wpep-visible' );

    echo '<h2 class="' . $css_state_class . ' video-archive-title">' . wpep_get_setting_translated( 'video-archive-title' ) . '</h2>';
    echo '<div class="' . $css_state_class . ' video-archive-container">';

    foreach ($post_list as $key => $value) {
      $link = apply_filters( 'wpep_video_link', get_permalink($value), $value->ID );

      echo WPEP_Controller::instance()->template->get_template( 'primary/item-video.php', [
        'item'                  => $value,
        'link'                  => $link,
        'container_class'       => 'video-grid-item video-grid-item-' . $value->ID . ' ' . ( ( isset( $front_post->ID ) && $value->ID == $front_post->ID ? 'video-grid-item-open' : '' ) )
      ] );
    }

    echo '</div>';
    echo '<div class="clearfix"></div>';

    echo '</div>';
  }

  private function output_grid_content_post_ebooks_html( $course_categories = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_ebooks_per_row', wpep_get_setting( 'index-page-ebooks-per-row' ) );

    if( $items_per_row == 0 )
      return;

    $post_list = WPEP_Controller::instance()->content->get_ebooks( $course_categories );

    if( empty( $post_list ) )
      return;

    echo '<div class="wpep-ebook-grid-items-container" data-items-per-row="' . $items_per_row . '">';

    foreach( $post_list as $key => $value)
      echo WPEP_Controller::instance()->template->get_template( 'primary/item-ebook.php', [
        'item'          => $value
      ] );

    echo '<div class="clearfix"></div>';
    echo '</div>';
  }

  private function output_grid_content_post_courses_html( $course_categories = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_courses_per_row', wpep_get_setting( 'index-page-courses-per-row' ) );

    if( $items_per_row == 0 )
      return;

    $post_list = WPEP_Controller::instance()->content->get_courses( $course_categories );

    if( empty( $post_list ) )
      return;

    $user_id = apply_filters( 'wpep_primary_content_grid_viewed_user_id', get_current_user_id() );

    $data = ' data-items-per-row="' . $items_per_row . '"';

    if( $items_per_row == WPEP_ITEMS_PER_ROW_DYNAMIC ) {
      $data = ' data-items-per-row="3" data-items-per-row-dynamic="' . WPEP_COURSE_PRESENTATION_CARD_MIN_WIDTH . '"';
    }

    echo '<div class="wpep-course-grid-items-container"' . $data . '>';

    foreach( $post_list as $post_item )
      echo    $this->_get_grid_content_post_courses_item_html( $post_item, false, $user_id );

    echo    '<div class="clearfix"></div>';
    echo '</div>';
  }

  private function _get_grid_content_post_courses_item_html( $course_object, $short_format = true, $user_id = false ) {
    return WPEP_Controller::instance()->template->get_template( 'primary/item-course.php', [
      'item'                  => $course_object,
      'short_description'     => wpep_course_excerpt( $course_object->ID, $short_format ),
      'course_link'           => apply_filters( 'wpep_course_link', get_permalink( $course_object->ID ), $course_object->ID ),
      'thumbnail_img'         => wpep_course_thumbnail_url( $course_object->ID ),
      'thumbnail_img_hover'   => ( wpep_get_setting_flag( 'index-page-enable-hover-image' )
        ? wpep_course_thumbnail_hover_url( $course_object->ID )
        : ''
      ),
      'thumbnail_ribbon_text' => get_post_meta( $course_object->ID, 'wpep_course_ribbon_text', true ),
      'user_id'               => ( $user_id == false ? get_current_user_id() : $user_id ),
    ] );
  }

  private function output_grid_content_post_offers_html( $course_categories = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_offers_per_row', wpep_get_setting( 'index-page-offers-per-row' ) );

    if( $items_per_row == 0 )
      return;

    $post_list = WPEP_Controller::instance()->content->get_offers( $course_categories );

    if( empty( $post_list ) )
      return;

    echo '<div class="wpep-offer-grid-items-container" data-items-per-row="' . $items_per_row . '">';

    foreach( $post_list as $key => $current_offer ) {
      $thumbnail_id = get_post_thumbnail_id( $current_offer->ID );

      $link_text = get_post_meta( $current_offer->ID, 'offer_link_text', true );

      if( $link_text == '' )
        $link_text = __( 'Redeem Offer', 'wpep' );

      echo WPEP_Controller::instance()->template->get_template( 'primary/item-offer.php', [
        'offer_id'        => $current_offer->ID,
        'image'           => ( $thumbnail_id != '' ? get_post( $thumbnail_id, 'full' )->guid : '' ),
        'title'           => get_the_title( $current_offer ),
        'sub_title'       => get_post_meta( $current_offer->ID, 'offer_subtitle', true ),
        'description'     => get_post_meta( $current_offer->ID, 'offer_description', true ),
        'link'            => apply_filters( 'wpep_offer_link', get_post_meta( $current_offer->ID, 'offer_link', true ), $current_offer->ID ),
        'link_text'       => apply_filters( 'wpep_text_view_offer', $link_text, $current_offer->ID ),
        'information_map' => get_post_meta( $current_offer->ID, '_offer_information_map', true )
      ] );
    }
    echo '<div class="clearfix"></div>';
    echo '</div>';
  }

  public function output_grid_courses_list() {
    $post_list = WPEP_Controller::instance()->content->get_courses();

    if( empty( $post_list ) )
      return;

    add_filter( 'wpep_primary_content_grid_courses_progress_bars', [ $this, '_grid_course_list_has_progress_bars' ] );

    echo '<div class="wpep-3d-carousel-container-wrapper">';
    echo    '<div class="' . apply_filters( 'wpep_primary_content_course_grid_container_class', 'wpep-3d-carousel-container' ) . '">';
    echo      '<div class="wpep-3d-carousel-perspective-container">';
    foreach( $post_list as $post_item ) {
      echo    '<div class="wpep-3d-carousel-item">';
      echo        $this->_get_grid_content_post_courses_item_html( $post_item );
      echo    '</div>';
    }
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    remove_filter( 'wpep_primary_content_grid_courses_progress_bars', [ $this, '_grid_course_list_has_progress_bars' ] );
  }

  public function _grid_course_list_has_progress_bars() {
    return false;
  }

}