<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;

class UserProfile {

  /**
   * @var UserProfile\PublicProfile
   */
  public $publicProfile;

  public $user;
  public $pagename_identifier = 'wpep-user-profile';
  public $user_identifier     = 'wpep-user-profile';

  /**
   * UserProfile constructor.
   */
  public function __construct() {
    add_action( 'init', [ $this, '_init' ] );

    if( wpep_get_setting_flag( 'user-profile-custom-tabs-enabled' ) )
      add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_custom_profile_tabs' ], 150, 2 );

    if( !wpep_public_profiles_enabled() )
      return;

    $user_slug = wpep_get_setting( 'user-profile-public-slug' );

    if( $user_slug != "" )
      $this->user_identifier = $user_slug;

    $this->publicProfile = new UserProfile\PublicProfile();
  }

  public function _custom_profile_tabs( $tabs, $user_id ) {
    $custom_tabs = wpep_get_setting( 'user-profile-custom-tabs' );

    if( !is_array( $custom_tabs ) )
      return $tabs;

    if( empty( $custom_tabs ) )
      return $tabs;

    $current_tabs = [];

    foreach( $custom_tabs as $tab_title => $tab_content ) {
      $current_tabs[ 'custom_' . sanitize_title( $tab_title ) ] = [
        'title'         => $tab_title,
        'content'       => wpep_format_content( $tab_content )
      ];
    }

    return (
      wpep_get_setting( 'user-profile-custom-tabs-position' ) === 'left' ?
        $current_tabs + $tabs
      : $tabs + $current_tabs
    );
  }

  /**
   * @param bool $default_current_user
   * @return \WP_User|false
   */
  public function get_display_user( $default_current_user = false ) {
    if( $this->user != null )
      return $this->user;

    $this->user = get_user_by( 'slug', $this->get_profile_username() );

    if( $this->user == false
      && $this->get_profile_username() == ''
      && get_query_var( 'pagename' ) == $this->pagename_identifier )
      $this->user = $this->get_current_user();

    if( $default_current_user && ( $this->user == false || $this->user == null ) )
      $this->user = $this->get_current_user();

    return $this->user;
  }

  /**
   * @return \WP_User
   */
  public function get_current_user() {
    return get_user_by( 'id', get_current_user_id() );
  }

  public function get_profile_username() {
    return get_query_var( $this->user_identifier );
  }

  public function _init() {
    if( !is_admin() ) {
      $current_post_id = wpep_get_current_post_id();

      if( $current_post_id != 0 ) {
        $post            = get_post($current_post_id);

        if( strpos( $post->post_content, '[wpep_profile' ) !== false && get_current_user_id() != 0 )
          WPEP_Controller::instance()->assets->has_enqueue_media_support = true;
      }
    }
  }



}
