<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WP_Query;

class CourseActivity {

  public function setup() {
    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_COURSE )
      return;

    global $wpdb;

    $lesson_ids = $wpdb->get_col(
      'SELECT id FROM ' . $wpdb->prefix . 'wpep_section_lesson WHERE post_id = ' . $post_id,
      0
    );

    if( !empty ( $lesson_ids ) ) {
      $wpdb->query( "DELETE FROM " . $wpdb->prefix . 'wpep_section_lesson WHERE id IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
      $wpdb->query( "DELETE FROM " . $wpdb->prefix . 'wpep_section_lesson_completed WHERE lesson_id IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
    };

    $wpdb->delete( $wpdb->prefix . 'wpep_section', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_course_user_data', [ 'course_id' => $post_id ] );
  }

  public function get_course_started_users( $course_id ) {
    $course_completed_map = wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_STARTED,
      $course_id,
      0,
      0,
      false,
      false
    ), 'user_id', "value" );

    return $course_completed_map;
  }
  
  /**
   * @param $date_from
   * @param $date_to
   * @param $min_lessons - the minimum lessons the user had to complete
   * @return array of user_ids
   */
  public function get_period_active_user_ids( $date_from = false, $date_to = false, $min_lessons = 0 ) {
    global $wpdb;

    $where = ' 1 = 1 ';

    if( ( $date_from != false && $date_to != false ) ||$min_lessons != 0 ) {
      $date_from = ( is_numeric( $date_from ) ? date ( "Y-m-d", $date_from ) : $date_from );
      $date_to   = ( is_numeric( $date_to )   ? date ( "Y-m-d", $date_to )   : $date_to   );

      $where .= ' AND ' . (
        $date_from == $date_to ?
          'DATE( lc.created_at ) = "' . $date_from . '"' :
          'DATE( lc.created_at ) >= "' . $date_from . '" AND DATE( lc.created_at ) <= "' . $date_to . '"'
        );
    }

    $sql = 'SELECT lc.user_id
              FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
             WHERE ' . $where . '
             GROUP BY lc.user_id';

    if( $min_lessons != 0 )
      $sql .= ' HAVING COUNT( lc.id ) >= ' . intval( $min_lessons ) . ' ';

    return $wpdb->get_col( $sql );
  }

  public function get_period_started_courses_list( $date_from, $date_to ) {
    $date_from = ( is_numeric( $date_from ) ? date ( "Y-m-d", $date_from ) : $date_from );
    $date_to   = ( is_numeric( $date_to )   ? date ( "Y-m-d", $date_to )   : $date_to   );

    global $wpdb;

    $where = (
      $date_from == $date_to ?
      'DATE( pt.started_at ) = "' . $date_from . '"' :
      'DATE( pt.started_at ) >= "' . $date_from . '" AND DATE( pt.started_at ) <= "' . $date_to . '"'
    );

    $sql = 'SELECT pt.* FROM ( SELECT l.post_id as course_id, lc.user_id, MIN( lc.created_at ) as started_at
                               FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                               LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON l.id = lc.lesson_id 
                               GROUP BY CONCAT( l.post_id, lc.user_id )
                              ) pt
                         WHERE ' . $where;

    return $wpdb->get_results( $sql );
  }

  public function get_period_completed_courses_list( $time_from, $time_to ) {
    $time_from = ( !is_numeric( $time_from ) ? strtotime( $time_from ) : $time_from );
    $time_to   = ( !is_numeric( $time_to )   ? strtotime ( $time_to )   : $time_to   );

    global $wpdb;

    $sql = 'SELECT user_data.user_id,
                   user_data.course_id,
                   user_data.value as completed_at
                   FROM ' . $wpdb->prefix . 'wpep_course_user_data user_data
                   WHERE user_data.key = "' . WPEP_USER_COURSE_COMPLETED_AT . '"
                     AND user_data.value >= ' . $time_from . '
                     AND user_data.value < ' . $time_to;

    return $wpdb->get_results( $sql );
  }

  public function get_period_completed_lessons_count( $date_from, $date_to ) {
    $date_from = ( is_numeric( $date_from ) ? date ( "Y-m-d", $date_from ) : $date_from );
    $date_to   = ( is_numeric( $date_to )   ? date ( "Y-m-d", $date_to )   : $date_to   );

    global $wpdb;

    $where = (
    $date_from == $date_to ?
      'DATE( lc.created_at ) = "' . $date_from . '"' :
      'DATE( lc.created_at ) >= "' . $date_from . '" AND DATE( lc.created_at ) <= "' . $date_to . '"'
    );


    $sql = 'SELECT COUNT( lc.id ) as completed_lessons
              FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
             WHERE ' . $where;

    return intval( $wpdb->get_var( $sql ) );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @return array
   */
  public function get_latest_interactions( $interaction_number = 30, $interaction_offset = 0, $pagination_information = false ) {
    global $wpdb;

    $sql = 'SELECT p.post_title as course_title, 
                   p.ID as course_id, 
                   s.title as section_title,
                   l.title as lesson_title,
                   u.user_nicename as user_nicename,
                   u.ID as user_id,
                   lc.created_at as completion_date
            FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
            LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON l.id = lc.lesson_id 
            LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id 
            LEFT JOIN ' . $wpdb->prefix . 'posts p ON p.id = s.post_id
            LEFT JOIN ' . $wpdb->prefix . 'users u ON u.id = lc.user_id
            ORDER BY lc.created_at DESC
            LIMIT ' . intval( $interaction_number );

    if( $interaction_offset != 0 )
      $sql .= " OFFSET " . intval( $interaction_offset );
    
    $interactions = $wpdb->get_results( $sql, ARRAY_A );

    if( $pagination_information == false )
      return $interactions;

    return [
      'entries_total'   => $wpdb->get_var( 'SELECT COUNT(id) FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed'),
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $interactions,
    ];
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param bool|string $filter_by
   * @return array
   */
  public function get_completed_history(
    $interaction_number     = 30,
    $interaction_offset     = 0,
    $pagination_information = false,
    $filter_by              = false
  ) {
    $course_list_query = new WP_Query( [
      'post_type'       => WPEP_POST_TYPE_COURSE,
      'posts_per_page'  => $interaction_number,
      'offset'          => $interaction_offset,
      'orderby'         => 'title',
      'order'           => 'ASC',
      'author'          => wpep_user_statistics_query_author_id()
    ] );

    $course_list = $course_list_query->get_posts();

    $response = [];

    foreach( $course_list as $course ) {
      $started_count   = 0;
      $completed_count = 0;

      foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_COMPLETED, $course->ID, 0, 0, false, false
      ) as $user_data_course_completed ) {
        if( isset( $filter_by['user_role'] ) && !wpep_user_has_role( $filter_by['user_role'], $user_data_course_completed->user_id ) )
          continue;

        $started_count++;

        if( $user_data_course_completed->value == 1 )
          $completed_count++;
      }

      $response[] = [
        'course_id'               => $course->ID,
        'course_title'            => $course->post_title,
        'course_lessons'          => $this->get_course_lesson_count( $course->ID ),
        'course_users_started'    => $started_count,
        'course_users_completed'  => $completed_count
      ];
    }

    if( $pagination_information == false )
      return $response;

    return [
      'entries_total'   => $course_list_query->found_posts,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $response,
    ];
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @return array
   */
  public function get_time_action_notify_map( $interaction_number = 30, $interaction_offset = 0, $pagination_information = false ) {
    global $wpdb;

    $sql = 'SELECT * FROM (
              SELECT `type`,
                     id,
                     post_id,
                     title,
                     time_action_notify,
                     user_started_at,
                     user_id
              FROM (       
                  SELECT "section" as `type`, 
                                       s.id,
                                       s.post_id,
                                       s.title,
                                       s.time_action_notify,
                                       cud.value as user_started_at,
                                       cud.user_id as user_id,
                                       COUNT( progress_l.id ) as lesson_count,
                                       COUNT( progress_lc.id ) as completed_lessons
                  FROM ' . $wpdb->prefix . 'wpep_section s
                  INNER JOIN ' . $wpdb->prefix . 'wpep_course_user_data cud
                          ON cud.course_id = s.post_id AND cud.key = "course_started"
                  LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson progress_l
                        ON s.id = progress_l.section_id
                  LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed progress_lc
                         ON progress_lc.lesson_id = progress_l.id
                        AND progress_lc.user_id   = cud.user_id
                  WHERE s.`time_action_notify` != 0
                    AND CAST(cud.value AS UNSIGNED) + s.`time_action_notify` < ' . time() . '
              ) inner_section
              WHERE inner_section.lesson_count > inner_section.completed_lessons
                                   
              UNION
              
              SELECT "lesson" as `type`, 
                          l.id,
                          l.post_id,
                          l.title,
                          l.time_action_notify,
                          cud.value as user_started_at,
                          cud.user_id as user_id
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              INNER JOIN ' . $wpdb->prefix . 'wpep_course_user_data cud
                      ON cud.course_id = l.post_id AND cud.key = "course_started"
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON lc.lesson_id = l.id AND lc.user_id = cud.user_id
              WHERE `time_action_notify` != 0
                AND CAST(cud.value AS UNSIGNED) + l.`time_action_notify` < ' . time() . '
                AND lc.id IS NULL
          ) result ORDER BY result.user_started_at - result.time_action_notify DESC';

    global $wpdb;

    $time_action_notify_map = $wpdb->get_results( $sql );

    if( $pagination_information == false )
      return $time_action_notify_map;

    return [
      'entries_total'   => count( $time_action_notify_map ),
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => array_slice( $time_action_notify_map, $interaction_offset, $interaction_number ),
    ];
  }

  public function get_course_lesson_count( $course_id ) {
    global $wpdb;

    $sql = 'SELECT COUNT( l.id ) 
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
              WHERE s.post_id = ' . intval( $course_id );

    return intval( $wpdb->get_var( $sql ) );
  }

  /**
   * Get the number of users that have progressed a course until a certain lesson count
   * @param $course_id
   * @return mixed
   */
  public function get_course_progress_map( $course_id) {
    global $wpdb;

    $sql = 'SELECT lc.user_id, 
             COUNT( lc.id ) as completed_count 
             FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON l.id = lc.lesson_id
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
             WHERE s.post_id = ' . intval( $course_id ) . '
             GROUP BY lc.user_id';

    $result = $wpdb->get_results( $sql );
    $response = [];

    foreach( $result as $r )
      $response[ $r->user_id ] = $r->completed_count;

    return $response;
  }

}