<?php
namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Posts;

class Hooks {

  private $_post_locked_by_prerequisite = [

  ];

  public function setup() {
    add_action( 'init', [ $this, 'register_post_types' ], WPEP_POST_TYPE_REGISTRATION_PRIORITY );
    add_action( 'widgets_init', [ $this, 'register_widget_area' ], 10 );

    // Early Events
    add_filter( 'wpep_text_view_course', [ $this, 'wpep_post_text_early' ], 5, 2 );
    add_filter( 'wpep_text_download_ebook', [ $this, 'wpep_post_text_early' ], 5, 2 );
    add_filter( 'wpep_text_view_video', [ $this, 'wpep_post_text_early' ], 5, 2 );
    // we have this done differently : add_filter( 'wpep_text_view_offer', [ $this, 'wpep_post_text_early' ], 5, 2 );

    add_filter( 'wpep_course_link', [ $this, 'wpep_post_link_early' ], 5, 2 );
    add_filter( 'wpep_course_link_early', [ $this, 'wpep_post_link_early' ], 5, 2 );
    add_filter( 'wpep_grid_item_button_style_args', [ $this, 'wpep_grid_item_button_style_args'], 5, 2 );

    // Mid Events
    add_filter( 'wpep_widget_item_class', [ $this, 'wpep_general_item_class' ], 50, 3 );
    add_filter( 'wpep_grid_item_class', [ $this, 'wpep_general_item_class' ], 50, 3 );

    // Late Events
    add_filter( 'wpep_text_view_course', [ $this, 'wpep_post_text' ], 1000, 2 );
    add_filter( 'wpep_course_link', [ $this, 'wpep_post_link' ], 1000, 2 );
    add_filter( 'wpep_ebook_link',  [ $this, 'wpep_post_link' ], 1000, 2 );
    add_filter( 'wpep_offer_link',  [ $this, 'wpep_post_link' ], 1000, 2 );

    add_filter( 'pre_get_avatar', [ $this, 'pre_get_avatar' ], 20, 3 );

    // WPEP Uploads Integration
    if( isset( $_COOKIE[ 'wpep_upload_post_id' ] ) && ( $_COOKIE[ 'wpep_upload_post_id' ] != '0'  ) ) {
      add_filter( 'ajax_query_attachments_args', [ $this, 'handle_frontend_attachments_query' ] );
      add_filter( 'user_has_cap', [ $this, 'user_has_cap'], 10, 4 );
      add_action( 'add_attachment', [ $this, 'add_attachment' ] );
    } else if( is_admin() ) {
      add_filter( 'ajax_query_attachments_args', [ $this, 'handle_backend_attachments_query' ] );
    }
  }

  public function register_post_types() {
    Posts\Courses::config();
    Posts\Video::config();
    Posts\EBook::config();
    Posts\Offer::config();
    Posts\Assessment::config();
  }

  public function wpep_grid_item_button_style_args( $button_style_args, $post_id ) {
    $text_color   = get_post_meta( $post_id, 'wpep_post_button_text_color', true );
    $button_color = get_post_meta( $post_id, 'wpep_post_button_color', true );

    if( $text_color != '' )
      $button_style_args['color'] = $text_color . ' !important';

    if( $button_color != '' )
      $button_style_args['background'] = $button_color . ' !important';

    return $button_style_args;
  }

  public function wpep_post_text_early( $default, $post_id ) {
    $text = trim( get_post_meta( $post_id, 'wpep_post_button_text', true ) );

    if( $text != '' )
      return $text;

    return $default;
  }

  public function wpep_post_text( $default, $post_id ) {
    if( wpep_post_coming_soon( $post_id ) )
      return wpep_get_setting_translated( 'opt-course-coming-soon' );

    if( $this->_post_is_locked_by_pre_requisites( $post_id ) )
      return wpep_get_setting_translated( 'prerequisite-course-button-lock-text' );

    return $default;
  }

  public function wpep_post_link_early( $default, $post_id ) {
    $link = get_post_meta( $post_id, 'wpep_post_alternative_link', true );

    if( is_string($link) && strpos( $link, 'http' ) === 0 )
      return $link;

    return $default;
  }

  public function wpep_post_link( $default, $post_id ) {
    if( wpep_post_coming_soon( $post_id ) )
      return '#';

    if( wpep_post_has_member_magnet_option( $post_id, get_current_user_id() ) ) {
      if( get_post_type( $post_id ) !== WPEP_POST_TYPE_COURSE
          || !in_array( wpep_course_type_from_id( $post_id ), wpep_course_type_member_magnet_integrated() ) )
        return wpep_post_member_magnet_link( $post_id );
    }

    if( $this->_post_is_locked_by_pre_requisites( $post_id ) )
      return '#';

    return $default;
  }

  public function wpep_general_item_class( $class, $post_id, $post_type = "none" ) {
    if( wpep_post_coming_soon( $post_id ) )
      return $class . ' wpep-coming-soon-item';

    if( $this->_post_is_locked_by_pre_requisites( $post_id, $post_type ) )
      return $class . ' wpep-locked-prerequisite-item';

    return $class;
  }

  private function _post_is_locked_by_pre_requisites( $post_id, $post_type = false ) {
    if( !wpep_get_setting_flag( 'prerequisite-course-button-lock' ) )
      return false;

    if( isset( $this->_post_locked_by_prerequisite[$post_id] ) )
      return $this->_post_locked_by_prerequisite[$post_id];

    if( $post_type === false )
      $post_type = get_post_type( $post_id );

    if( $post_type !== WPEP_POST_TYPE_COURSE ) {
      $this->_post_locked_by_prerequisite[$post_id] = false;

      return false;
    }

    $prerequisites      = apply_filters( "wpep_single_course_prerequisites_list", get_post_meta( $post_id, 'wpep_prerequisite' ) );
    $prerequisite_mode  = wpep_get_prerequisite_mode( get_post_meta( $post_id, WPEP_PREREQUISITE_MODE_POST_META, true ) );

    if( empty( $prerequisites )
        || $prerequisite_mode[WPEP_PREREQUISITE_FIELD_ID] === WPEP_PREREQUISITE_MODE_NOTIFY ) {
      $this->_post_locked_by_prerequisite[$post_id] = false;

      return false;
    }

    $completed_prerequisites = [];

    foreach( $prerequisites as $prerequisite ) {
      if( !WPEP_Controller::instance()->userActivity->has_completed_course( $prerequisite, get_current_user_id(), true ) )
        continue;

      $completed_prerequisites[] = $prerequisite;
    }

    $requirement = ( $prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] === 'all' ? count( $prerequisites ) : $prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] );

    $this->_post_locked_by_prerequisite[$post_id] = ( $requirement != 0 && $requirement > count( $completed_prerequisites )  );

    return $this->_post_locked_by_prerequisite[$post_id];
  }

  public function register_widget_area() {
    register_sidebar( [
      'name'          => __('WPEP Sidebar', "wpep" ),
      'id'            => 'wpep-sidebar',
      'before_widget' => '<div id="%1$s" class="wpep-widget %2$s">',
      'after_widget'  => '</div>',
      'before_title'  => '<h1 class="wpep-widget-title">',
      'after_title'   => '</h1>',
    ] );

    register_widget('WPEP\Widgets\Courses');
    register_widget('WPEP\Widgets\Search');
    register_widget('WPEP\Widgets\Image');
  }

  public function add_attachment( $post_ID ) {
    if( ! is_user_logged_in() )
      return;

    if( !isset( $_COOKIE[ 'wpep_upload_post_id' ] ) )
      return;

    $post = ( !is_numeric( $_COOKIE[ 'wpep_upload_post_id' ] ) ? false : get_post( intval( $_COOKIE[ 'wpep_upload_post_id' ] ) ) );

    if( $post !== null ) {
      wp_update_post( [
        'ID'          => $post_ID,
        'post_parent' => intval( $_COOKIE[ 'wpep_upload_post_id' ] )
      ] );
    }
    
    if( isset( $post->ID ) )
      update_post_meta( $post_ID, 'wpep_uploaded_file_parent_id', $post->ID );
    else
      update_post_meta( $post_ID, 'wpep_uploaded_file_parent_id', sanitize_text_field( $_COOKIE[ 'wpep_upload_post_id' ] ) );

    update_post_meta( $post_ID, 'wpep_uploaded_file_user_id', get_current_user_id() );
  }

  public function user_has_cap( $capabilities, $caps, $args, $user_object ) {
    if( ! is_user_logged_in() )
      return $capabilities;

    $capabilities['upload_files'] = true;

    return $capabilities;
  }

  public function handle_frontend_attachments_query( $query ) {
    if( ! is_user_logged_in() )
      return $query;

    if( current_user_can( 'manage_options' ) )
      return $this->handle_backend_attachments_query( $query );

    $user_id = get_current_user_id();

    $query['author'] = $user_id;

    return $query;
  }

  public function handle_backend_attachments_query( $query ) {
    if( ! is_user_logged_in() )
      return $query;
    
    if( !isset( $query['meta_query'] ) )
      $query['meta_query'] = [];

    $query['meta_query'][] = [
      'relation' => 'OR',
      [
        'key'     => 'wpep_uploaded_file_user_id',
        'value'   => get_current_user_id(),
        'compare' => '=',
      ],
      [
        'key'     => 'wpep_uploaded_file_user_id',
        'value'   => '',
        'compare' => 'NOT EXISTS',
      ],
    ];

    return $query;
  }

  public function pre_get_avatar( $default_avatar, $id_or_email, $args ) {
    if( !is_numeric( $id_or_email ) )
      return $default_avatar;

    $profile_picture = get_user_meta( $id_or_email, WPEP_USER_META_PROFILE_PICTURE, true );

    if( $profile_picture == '' )
      return $default_avatar;

    return sprintf(
      "<img alt='%s' src='%s' class='avatar' height='%d' width='%d' %s/>",
      esc_attr( $args['alt'] ),
      esc_url( $profile_picture ),
      (int) $args['height'],
      (int) $args['width'],
      $args['extra_attr']
    );
  }

}