<?php

namespace WPEP\Coordinator;

class Request {

  private $_current_post_id   = NULL;
  private $_current_post_type = NULL;
  private $_url_to_post_id    = [];

  /**
   * @return int
   */
  public function get_current_post_id() {
    if( $this->_current_post_id === NULL )
      $this->_current_post_id = $this->get_post_id_by_url( $this->get_current_url() );

    return $this->_current_post_id;
  }

  /**
   * @return int
   */
  public function get_current_post_type() {
    if( $this->_current_post_type === NULL )
      $this->_current_post_type = get_post_type( $this->get_current_post_id() );

    return $this->_current_post_type;
  }

  /**
   * @param $current_url
   * @return int
   */
  public function get_post_id_by_url( $current_url ) {
    if( isset( $this->_url_to_post_id[ $current_url ] ) )
      return $this->_url_to_post_id[ $current_url ];

    $post_id = url_to_postid( $current_url );

    if( $post_id == 0 ) {
      $current_url = strtok( $current_url, '?' );

      $post_id = url_to_postid( $current_url );
    }

    $this->_url_to_post_id[ $current_url ] = apply_filters( 'wpep_single_content_post_id', $post_id );

    return $this->_url_to_post_id[ $current_url ];
  }

  public function get_current_url() {
    if( !isset( $_SERVER['HTTP_HOST'] ) || !isset( $_SERVER['REQUEST_URI'] ) )
      return false;

    return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
  }

}