<?php
namespace WPEP\Coordinator\ShortCode;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\FilterProxy as FilterProxy;

class Course {
  
  use FilterProxy;

  public function __construct() {
    add_shortcode( 'wpep_index', [ $this, 'primary' ] );
    add_shortcode( 'wpep_course', [ $this, 'single' ] );
    add_shortcode( 'wpep_course_preview', [ $this, 'single_preview' ] );
    add_shortcode( 'wpep_course_grid', [ $this, 'grid' ] );
    add_shortcode( 'wpep_the_course', [ $this, 'the_single' ] );

    add_shortcode( 'wpep_course_is_completed', [ $this, 'course_is_completed' ] );
    add_shortcode( 'wpep_course_not_completed', [ $this, 'course_not_completed' ] );
  }

  public function primary($attributes, $content = null) {
    if( !is_user_logged_in() && in_array( 'index-page-shortcode', wpep_get_auth_status_page_restrictions() ) )
      return WPEP_Controller::instance()->shortCode->profile->authentication( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $post_id             = isset($attributes['post_id'])         ? intval( $attributes['post_id'] )        : false;
    $include_header      = isset($attributes['include_header'])  ? intval( $attributes['include_header'] ) : false;
    $include_footer      = isset($attributes['include_footer'])  ? intval( $attributes['include_footer'] ) : false;

    $this->_filter_proxy_setup( $this->_primary_filters( $attributes ) );

    $response = WPEP_Controller::instance()->template->get_template(
      'primary.php', [
        'post_id'        => $post_id,
        'include_header' => $include_header,
        'include_footer' => $include_footer
      ]
    );

    $this->_filter_proxy_remove();

    return $response;
  }

  private function _primary_filters( $attributes ) {
    $filter_proxy_list = [];

    $category_ids         = isset($attributes['categories'])           ? $attributes['categories']  : '';
    $categories_per_row   = isset($attributes['categories_per_row'])
      ? intval( $attributes['categories_per_row'] )  : ( isset($attributes['category_navigation'] ) && intval( $attributes['category_navigation'] )  ? 1 : '' );
    $all_categories_filter= isset($attributes['all_categories_filter']) ? $attributes['all_categories_filter']  : '';
    $single_category_view = isset($attributes['single_category_view']) ? $attributes['single_category_view']  : '';
    $course_progress_bars = isset($attributes['course_progress_bars']) ? intval( $attributes['course_progress_bars'] )  : '';
    $posts_per_page       = isset($attributes['posts_per_page'])       ? $attributes['posts_per_page']   : '';
    $courses_per_row      = isset($attributes['courses_per_row'])      ? $attributes['courses_per_row']  : '';
    $videos_per_row       = isset($attributes['videos_per_row'])       ? intval( $attributes['videos_per_row'] )  : '';
    $ebooks_per_row       = isset($attributes['ebooks_per_row'])       ? intval( $attributes['ebooks_per_row'] )  : '';
    $offers_per_row       = isset($attributes['offers_per_row'])       ? intval( $attributes['offers_per_row'] )  : '';
    $display_header       = isset($attributes['display_header'])       ? $attributes['display_header']  : null;
    $course_list_ids      = isset($attributes['courses'])              ? $attributes['courses'] : '';
    $ebook_list_ids       = isset($attributes['ebooks'])               ? $attributes['ebooks']  : '';
    $offer_list_ids       = isset($attributes['offers'])               ? $attributes['offers']  : '';
    $user_id              = isset($attributes['user_id'])              ? $attributes['user_id'] : '';
    $order                = isset($attributes['order'])                ? $attributes['order']   : '';
    $lock_by_prerequisites= isset($attributes['lock_by_prerequisites'])? $attributes['lock_by_prerequisites']   : '';
    $order_by             = isset($attributes['order_by'])             ? (
      in_array( $attributes['order_by'], array_keys( wpep_content_primary_order_by_options() ) )
        ? $attributes['order_by'] : ''
    ) : '';

    if( $category_ids !== '' )
      $filter_proxy_list['wpep_primary_content_category_list_args'] = function( $args ) use ( $category_ids ) {
        $args['include'] = $category_ids;

        return $args;
      };

    if( $all_categories_filter !== '' )
      $filter_proxy_list['wpep_primary_content_all_categories_navigation_enabled'] = intval( $all_categories_filter );

    if( $categories_per_row !== '' )
      $filter_proxy_list['wpep_template_category_list_items_per_row'] = function ($current_per_row) use ($categories_per_row) {
        return ($categories_per_row == 0 ? $current_per_row : $categories_per_row);
      };

    if( $single_category_view !== '' )
      $filter_proxy_list['wpep_primary_content_grid_categories_secondary_as_primary_view'] = function ($current_single_category_view) use ($single_category_view) {
        return ( $single_category_view == true ? 1 : 0 );
      };

    if( $course_progress_bars !== '' )
      $filter_proxy_list['wpep_primary_content_grid_courses_progress_bars'] = function( $current_per_row ) use ( $course_progress_bars ) {
        return $course_progress_bars;
      };

    if( $courses_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_courses_per_row'] = function( $current_per_row ) use ( $courses_per_row ) {
        return ( $courses_per_row === WPEP_ITEMS_PER_ROW_DYNAMIC ? WPEP_ITEMS_PER_ROW_DYNAMIC : intval( $courses_per_row ) );
      };

    if( $course_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_course_list_args'] = function ($args) use ($course_list_ids) {
        $args['include'] = $course_list_ids;

        if( isset( $args['meta_query'] ) && isset( $args['meta_query']['wpep_show_on_index'] ) )
          unset( $args['meta_query']['wpep_show_on_index'] );

        return $args;
      };
    }

    if( $ebook_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_ebook_list_args'] = function ($args) use ($ebook_list_ids) {
        $args['include'] = $ebook_list_ids;

        return $args;
      };
    }

    if( $offer_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_offer_list_args'] = function ($args) use ($offer_list_ids) {
        $args['include'] = $offer_list_ids;

        return $args;
      };
    }

    if( $videos_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_videos_per_row'] = function( $current_per_row ) use ( $videos_per_row ) {
        return $videos_per_row;
      };

    if( $ebooks_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_ebooks_per_row'] = function( $current_per_row ) use ( $ebooks_per_row ) {
        return $ebooks_per_row;
      };

    if( $offers_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_offers_per_row'] = function( $current_per_row ) use ( $offers_per_row ) {
        return $offers_per_row;
      };

    if( $display_header !== null )
      $filter_proxy_list['wpep_primary_content_display_header'] = function( $internal_display_header ) use ( $display_header ) {
        return $display_header;
      };

    if( $user_id !== '' )
      $filter_proxy_list['wpep_primary_content_grid_viewed_user_id'] = function( $current_user_id ) use ( $user_id ) {
        return $user_id;
      };

    if( $order !== '' || $order_by !== '' || $posts_per_page !== '' ) {
      $wpep_ordering_enforced = (
        $order !== '' || $order_by !== ''
        ? wpep_get_setting_flag( 'index-page-enforce-ordering' )
        : false
      );

      $filter_proxy_list['pre_get_posts'] = function( $wp_query ) use ( $order, $order_by, $posts_per_page ) {
        if( $order !== '' )
          $wp_query->set( 'order', ( $order === 'DESC' ? 'DESC' : 'ASC' ) );

        if( $order_by !== '' ) // Sanitized Earlier
          $wp_query->set( 'orderby', $order_by );

        if( $posts_per_page !== '' ) {
          $wp_query->set( 'posts_per_page', intval( $posts_per_page ) );
          $wp_query->set( 'numberposts', intval( $posts_per_page ) );
          $wp_query->set( 'nopaging', false );
        }

        return $wp_query;
      };

      // Lets make it light-weight, without the need of hooking trough multiple times.
      if( $wpep_ordering_enforced ) {
        WPEP_Settings::instance()->options[ 'index-page-enforce-ordering' ] = 0;

        // Actions are actually a sub-set of filters, smart architecture, we're going to use this to boost performance
        $filter_proxy_list['wpep_after_main_content'] = function() {
          WPEP_Settings::instance()->options[ 'index-page-enforce-ordering' ] = 1;
        };
      }
    } else if( $lock_by_prerequisites !== '' ) {
      $locked_by_prerequisites = wpep_get_setting_flag( 'prerequisite-course-button-lock' );

      if( $locked_by_prerequisites != $lock_by_prerequisites ) {
        WPEP_Settings::instance()->options[ 'prerequisite-course-button-lock' ] = $lock_by_prerequisites;

        // Actions are actually a sub-set of filters, smart architecture, we're going to use this to boost performance
        $filter_proxy_list['wpep_after_main_content'] = function() use ( $locked_by_prerequisites ) {
          WPEP_Settings::instance()->options[ 'prerequisite-course-button-lock' ] = $locked_by_prerequisites;
        };
      }
    }

    return $filter_proxy_list;
  }

  public function single( $attributes ) {
    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );
    
    $post_id     = intval( isset($attributes['id']) ? $attributes['id'] : get_the_ID() );

    if( $post_id == false )
      return __( "You need to offer the id param for this shortcode. ", "wpep" );

    global $post;
    $previous_post = $post;
    $post = get_post( $post_id );

    if( $post == null || get_post_type( $post ) != WPEP_POST_TYPE_COURSE ) {
      $post = $previous_post;

      return sprintf( __( "This Course has gone missing, please verify the following course id %s ", "wpep" ), $post_id );
    }

    if( !is_user_logged_in() && in_array( 'course-shortcode', wpep_get_auth_status_page_restrictions() ) )
      return WPEP_Controller::instance()->shortCode->profile->authentication( $attributes );

    $this->_filter_proxy_setup( $this->_single_filters( $attributes ) );

    setup_postdata( $post );

    WPEP_Controller::instance()->content->singleCoordinator->init( $post_id );

    if( isset($attributes['read_only']) && $attributes['read_only'] != '' )
      WPEP_Controller::instance()->content->singleCoordinator->getCurrentDeliveryMethod()->read_only = intval( $attributes['read_only'] );

    $content = WPEP_Controller::instance()->template->get_template(
      'content-course-single-minimal.php'
    );

    $post = $previous_post;
    setup_postdata( $post );

    $this->_filter_proxy_remove();

    return $content;
  }

  private function _single_filters( $attributes ) {
    $show_header              = intval( isset($attributes['show_header'])         ? $attributes['show_header'] : true );
    $disable_assessments      = intval( isset($attributes['disable_assessments']) ? $attributes['disable_assessments'] : false );
    $course_type              = ( isset( $attributes['course_type'] )                  ? $attributes['course_type'] : 'default' );
    $course_available_lessons = ( isset( $attributes['course_lessons'] )               ? explode( ",", $attributes['course_lessons'] ) : [] );

    $filter_proxy_list = [
      'wpep_single_content_course_ignore_delivery' => false,
      'wpep_single_course_prerequisites_list'      => [],
      'wpep_single_course_has_next_suggestions'    => false
    ];

    $filter_proxy_list['wpep_course_single_content_include_header'] = $show_header;

    if( $course_type != 'default' )
      $filter_proxy_list['wpep_single_content_course_type'] = function( $course_type_internal ) use ( $course_type ) {
        return $course_type;
      };

    if( $disable_assessments || !empty( $course_available_lessons ) ) {
      $filter_proxy_list['wpep_course_single_content_form_args'] = function( $args ) use ( $disable_assessments, $course_available_lessons ) {
        foreach( $args['course_sections'] as $course_section_id => $course_section ) {
          if( $disable_assessments )
            $args[ 'course_sections' ][$course_section_id]['assessment_id'] = 0;

          if( empty( $course_available_lessons ) )
            continue;

          foreach( $args[ 'course_sections' ][$course_section_id]['lessons'] as $course_lesson_id => $course_lesson ) {
            if( in_array( $course_lesson_id, $course_available_lessons ) )
              continue;

            $args[ 'course_sections' ][$course_section_id]['lessons'][$course_lesson_id]['content'] = '';
            $args[ 'course_sections' ][$course_section_id]['lessons'][$course_lesson_id]['is_disabled'] = true;
          }

        }

        if( $disable_assessments )
          $args[ 'final_assessment_id' ] = 0;

        return $args;
      };
    }


    return $filter_proxy_list;
  }

  public function single_preview( $attributes ) {
    $attributes['course_type']          = ( isset( $attributes['course_type'] ) && $attributes['course_type'] != '' ? $attributes['course_type'] : WPEP_COURSE_TYPE_EXECUTION_PLAN );
    $attributes['disable_assessments']  = true;
    $attributes['read_only']            = true;
    $attributes['course_lessons']       = ( !isset( $attributes['course_lessons'] ) ? '' : $attributes['course_lessons'] );

    add_filter( 'wp_redirect', '__return_false', 100 );
    add_filter( 'wpep_single_content_course_before_html', '__return_empty_string', 100 );
    add_filter( 'wpep_single_content_course_after_html', '__return_empty_string', 100 );

    $response = $this->single( $attributes );

    remove_filter( 'wp_redirect', '__return_false', 100 );
    remove_filter( 'wpep_single_content_course_before_html', '__return_empty_string', 100 );
    remove_filter( 'wpep_single_content_course_after_html', '__return_empty_string', 100 );

    return $response;
  }

  public function grid( $attributes ) {
    if( !is_user_logged_in() && in_array( 'course-grid-shortcode', wpep_get_auth_status_page_restrictions() ) )
      return WPEP_Controller::instance()->shortCode->profile->authentication( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $this->_filter_proxy_setup( $this->_grid_filters( $attributes ) );

    $response = WPEP_Controller::instance()->template->get_template(
      'primary-grid-courses.php',
      [
        'include_header' => false,
        'include_footer' => false
      ]
    );

    $this->_filter_proxy_remove();

    return $response;
  }

  private function _grid_filters( $attributes ) {
    $filter_proxy_list = [];

    $course_list_ids      = isset( $attributes['courses'] ) ? $attributes['courses'] : '';
    $container_class      = isset( $attributes['container_class'] ) ? $attributes['container_class'] : '';

    if( $course_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_course_list_args'] = function ($args) use ($course_list_ids) {
        $args['include'] = $course_list_ids;

        return $args;
      };
    }

    if( $container_class !== '' ) {
      $filter_proxy_list['wpep_primary_content_course_grid_container_class'] = function ( $class ) use ( $container_class ) {
        return $class . ' ' . $container_class;
      };
    }

    return $filter_proxy_list;
  }

  public function the_single() {
    $post_id     = get_the_ID();

    if( $post_id == false )
      return __( "You need to offer the id param for this shortcode. ", "wpep" );

    global $post;

    if( get_post_type( $post ) != WPEP_POST_TYPE_COURSE ) {
      return sprintf( __( "This Course has gone missing, please verify the following course id %s ", "wpep" ), $post_id );
    }

    $hide_header = get_post_meta( $post_id, 'wpep_course_layout_hide_header', true );
    $hide_header = ( $hide_header === '' ? false : intval( $hide_header ) );

    $this->_filter_proxy_setup( [
      'wpep_single_content_course_ignore_delivery'  => false,
      'wpep_course_single_content_include_header'   => !$hide_header
    ] );

    WPEP_Controller::instance()->content->singleCoordinator->init( $post_id );

    $content = WPEP_Controller::instance()->template->get_template(
      'content-course-single.php',
      [
        'include_header' => false,
        'include_footer' => false
      ]
    );

    $this->_filter_proxy_remove();

    return $content;
  }

  public function _return_false() {
    return false;
  }

  public function course_is_completed( $attributes, $content ) {
    if( !isset( $attributes['id'] ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_course_is_completed - Missing id parameter", "wpep" ) . '</p>';

    if( !is_user_logged_in() )
      return '';

    $course_ids = explode( ",", $attributes['id'] );

    foreach( $course_ids as $course_id ) {
      if( !WPEP_Controller::instance()->userActivity->has_completed_course( $course_id ) )
        return '';
    }

    return wpep_format_content( $content );
  }

  public function course_not_completed( $attributes, $content ) {
    if( !isset( $attributes['id'] ) )
      return '<p class="wpep-notification wpep-notification-danger">' . __( "wpep_course_is_completed - Missing id parameter", "wpep" ) . '</p>';

    if( !is_user_logged_in() )
      return $content;

    $course_ids = explode( ",", $attributes['id'] );

    foreach( $course_ids as $course_id ) {
      if( WPEP_Controller::instance()->userActivity->has_completed_course( $course_id ) )
        return '';
    }

    return wpep_format_content( $content );
  }

}