<?php
namespace WPEP\Coordinator\ShortCode;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;

class Profile {

  use FilterProxy;

  public function __construct() {
    add_shortcode( 'wpep_profile', [ $this, 'primary' ] );
    add_shortcode( 'wpep_profile_authentication', [ $this, 'authentication' ] );
    add_shortcode( 'wpep_user_is_logged_in', [ $this, 'user_is_logged_in' ] );
    add_shortcode( 'wpep_user_not_logged_in', [ $this, 'user_not_logged_in' ] );
    add_shortcode( 'wpep_public_profile_link', [ $this, 'public_profile_link'] );

    add_action( 'wpep_user_profile_authentication_before', [ $this, 'authentication_action_before'] );
    add_action( 'wpep_user_profile_authentication_after', [ $this, 'authentication_action_after'] );
    add_filter( 'wpep_user_profile_authentication_form_view', [ $this, 'authentication_form_view' ] );
  }

  public function primary($attributes, $content = null) {
    $user_information = WPEP_Controller::instance()->content->userProfileCoordinator->get_display_user( true );

    if( $user_information == false )
      return $this->authentication( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    return WPEP_Controller::instance()->template->get_template(
      'user-profile.php', [
        'user_information' => $user_information,
        'include_header'   => false,
        'include_footer'   => false
      ]
    );
  }

  public function authentication($attributes, $content = null) {
    if( get_current_user_id() != 0 )
      return $this->primary( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    return WPEP_Controller::instance()->template->get_template( 'authentication.php',
      [
        'include_header'   => false,
        'include_footer'   => false
      ]
    );
  }

  public function authentication_action_before() {
    if( !is_single() )
      return;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return;

    echo wpautop( get_post_meta( get_the_ID(), 'wpep_member_magnet_option_text_before', true ) );
  }

  public function authentication_form_view( $form_view ) {
    if( !is_single() )
      return $form_view;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return $form_view;

    $post_form_view = get_post_meta( get_the_ID(), 'wpep_member_magnet_option_form_view', true );

    return ( $post_form_view != '' ? $post_form_view : $form_view );
  }

  public function authentication_action_after() {
    if( !is_single() )
      return;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return;

    echo wpautop( get_post_meta( get_the_ID(), 'wpep_member_magnet_option_text_after', true ) );
  }

  public function user_is_logged_in( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function user_not_logged_in( $attributes, $content = '' ) {
    if( is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function public_profile_link( $attributes, $content = '' ) {
    $content = ( is_string( $content ) ? $content : '' );
    $link    = wpep_user_profile_public_link( get_current_user_id() );

    if( $link === false )
      return '';

    if( $content != '' )
      return '<a href="' . $link . '">' . $content . '</a>';

    return $link;
  }

}