<?php

namespace WPEP\Coordinator\UserActivity;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as EntityAssessment;

class Assessment {

  public function get_user_data_value(
    $key = false,
    $post_id = false,
    $question_id = false,
    $question_answer_id = false,
    $user_id = false
  ) {
    $user_data = $this->get_user_data( $key, $post_id, $question_id, $question_answer_id, $user_id );

    return ( $user_data == null ? false : $user_data->value );
  }

  public function get_user_data( 
    $key = false, 
    $post_id = false, 
    $question_id = false, 
    $question_answer_id = false, 
    $user_id = false, 
    $single_row = true,
    $value = false
  ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    global $wpdb;

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND aud.user_id = ' . intval( $user_id );

    if( $value !== false )
      $sql .= ' AND aud.value = "' . htmlentities( $value ) . '"';

    if( $key !== false )
      $sql .= ' AND aud.key = "' . htmlentities( $key ) . '"';

    if( $post_id !== false )
      $sql .= ' AND aud.post_id  = ' . intval( $post_id );

    if( $question_id !== false )
      $sql .= ' AND aud.question_id = ' . intval( $question_id );

    if( $question_answer_id !== false )
      $sql .= ' AND aud.question_answer_id  = ' . intval( $question_answer_id );

    return ( $single_row ? $wpdb->get_row( $sql ) : $wpdb->get_results( $sql ) );
  }

  public function set_user_data( 
    $key, 
    $value, 
    $post_id, 
    $question_id, 
    $question_answer_id, 
    $user_id = false 
  ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $post_id  = intval( $post_id );
    $question_id = intval( $question_id );
    $question_answer_id  = intval( $question_answer_id );

    $value_updated = true;

    $existent_entries = $this->get_user_data( $key, $post_id, $question_id, $question_answer_id, $user_id, false );

    if( empty( $existent_entries ) ) {
      $this->add_user_data( $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
    } else if( count( $existent_entries ) >= 2 ) {
      $this->delete_user_data( $key, $post_id, $question_id, $question_answer_id, $user_id );

      $this->add_user_data( $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
    } else {
      $existent_entry = array_shift( $existent_entries );

      if( $existent_entry->value != $value ) {
        global $wpdb;

        $wpdb->update(
          $wpdb->prefix . 'wpep_assessment_user_data',
          [ 'value' => $value ],
          [ 'id'    => $existent_entry->id ]
        );
      } else {
        $value_updated = false;
      }
    }

    if( !$value_updated )
      return;

    do_action( "wpep_user_set_assessment_data", $key, $value, $post_id, $question_id, $question_answer_id, $user_id, true );
    do_action( "wpep_user_set_assessment_data_" . $key, $value, $post_id, $question_id, $question_answer_id, $user_id, true );
  }

  public function add_user_data( 
    $key, 
    $value, 
    $post_id, 
    $question_id = 0, 
    $question_answer_id = 0, 
    $user_id = false 
  ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $post_id  = intval( $post_id );
    $question_id = intval( $question_id );
    $question_answer_id  = intval( $question_answer_id );

    global $wpdb;

    $wpdb->insert( $wpdb->prefix . 'wpep_assessment_user_data', [
      'user_id'             => $user_id,
      'post_id'             => $post_id,
      'question_id'         => $question_id,
      'question_answer_id'  => $question_answer_id,
      'key'                 => $key,
      'value'               => $value,
      'created_at'          => date( 'Y-m-d H:i:s' )
    ] );

    do_action( "wpep_user_add_assessment_data", $key, $value, $post_id, $question_id, $question_answer_id, $user_id );
  }

  public function delete_user_data(
    $key = false,
    $post_id = false,
    $question_id = false,
    $question_answer_id = false,
    $user_id = false
  ) {
    if ($user_id === false)
      $user_id = get_current_user_id();

    if ($user_id == 0)
      return null;

    global $wpdb;

    $where = [
      'user_id' => intval( $user_id )
    ];

    if( $key !== false )
      $where['key'] = htmlentities( $key );

    if( $post_id !== false )
      $where['post_id'] = intval( $post_id );

    if( $question_id !== false )
      $where['question_id'] = intval( $question_id );

    if( $question_answer_id !== false )
      $where['question_answer_id'] = intval( $question_answer_id );

    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_user_data', $where );

    do_action( "wpep_user_delete_assessment_data", $key, $post_id, $question_id, $question_answer_id, $user_id );
  }
  
  public function get_list_statistics_by_assessment_id( $assessment_id, $filter_by = [] ) {
    $start_time_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'start_time', $assessment_id, false, false, false, false ), 'user_id', 'value' );
    $submission_time_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'submission_time', $assessment_id, false, false, false, false ), 'user_id', 'value' );
    $status_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'status', $assessment_id, false, false, false, false ), 'user_id', 'value' );
    $score_list  = wpep_utility_map_to_array_assoc( $this->get_user_data( 'score', $assessment_id, false, false, false, false ), 'user_id', 'value' );
    $correct_answers = wpep_utility_map_to_array_assoc( $this->get_user_data( 'is_correct', $assessment_id, false, false, false, false, '1' ), 'user_id', 'value', true );
    $incorrect_answers = wpep_utility_map_to_array_assoc( $this->get_user_data( 'is_correct', $assessment_id, false, false, false, false, '0' ), 'user_id', 'value', true );

    $response = [];

    foreach( $start_time_list as $user_id => $start_time ) {
      if( !wpep_user_filter_requirement_met( $user_id, $filter_by ) )
        continue;

      $user_correct_answers = isset( $correct_answers[$user_id] ) ?
        ( is_array( $correct_answers[$user_id] ) ? count( $correct_answers[$user_id] ) : 1 )
        : 0;
      $user_incorrect_answers = isset( $incorrect_answers[$user_id] ) ?
        ( is_array( $incorrect_answers[$user_id] ) ? count( $incorrect_answers[$user_id] ) : 1 )
        : 0;

      $response[ $user_id ] = [
        'user_id'           => $user_id,
        'start_time'        => $start_time,
        'submission_time'   => ( isset( $submission_time_list[$user_id] ) ? $submission_time_list[$user_id] : 0 ),
        'status'            => ( isset( $status_list[ $user_id ] ) ? $status_list[ $user_id ] : WPEP_USER_ASSESSMENT_STATUS_PENDING ),
        'score'             => ( isset( $score_list[$user_id] ) ? $score_list[$user_id] : 0 ),
        'total_answers'     => ( $user_correct_answers + $user_incorrect_answers ),
        'correct_answers'   => $user_correct_answers,
        'incorrect_answers' => $user_incorrect_answers
      ];
    }

    return $response;
  }

  public function get_list_statistics( $user_id ) {
    $user_data_start_time_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'start_time', false, false, false, $user_id, false ), 'post_id', 'value' );
    $user_data_submission_time_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'submission_time', false, false, false, $user_id, false ), 'post_id', 'value' );
    $user_data_status_list = wpep_utility_map_to_array_assoc( $this->get_user_data( 'status', false, false, false, $user_id, false ), 'post_id', 'value' );
    $user_data_score_list  = wpep_utility_map_to_array_assoc( $this->get_user_data( 'score', false, false, false, $user_id, false ), 'post_id', 'value' );
    $user_data_correct_answers = wpep_utility_map_to_array_assoc( $this->get_user_data( 'is_correct', false, false, false, $user_id, false, '1' ), 'post_id', 'value', true );
    $user_data_incorrect_answers = wpep_utility_map_to_array_assoc( $this->get_user_data( 'is_correct', false, false, false, $user_id, false, '0' ), 'post_id', 'value', true );

    $response = [];

    foreach( $user_data_start_time_list as $assessment_id => $user_data_start_time ) {
      $correct_answers = isset( $user_data_correct_answers[$assessment_id] ) ?
        ( is_array( $user_data_correct_answers[$assessment_id] ) ? count( $user_data_correct_answers[$assessment_id] ) : 1 )
        : 0;
      $incorrect_answers = isset( $user_data_incorrect_answers[$assessment_id] ) ?
        ( is_array( $user_data_incorrect_answers[$assessment_id] ) ? count( $user_data_incorrect_answers[$assessment_id] ) : 1 )
        : 0;

      $response[ $assessment_id ] = [
        'assessment_id'     => $assessment_id,
        'assessment_title'  => get_the_title( $assessment_id ),
        'start_time'        => $user_data_start_time,
        'submission_time'   => ( isset( $user_data_submission_time_list[$assessment_id] ) ? $user_data_submission_time_list[$assessment_id] : 0 ),
        'status'            => ( isset( $user_data_status_list[ $assessment_id ] ) ? $user_data_status_list[ $assessment_id ] : WPEP_USER_ASSESSMENT_STATUS_PENDING ),
        'score'             => ( isset( $user_data_score_list[$assessment_id] ) ? $user_data_score_list[$assessment_id] : 0 ),
        'total_answers'     => ( $correct_answers + $incorrect_answers ),
        'correct_answers'   => $correct_answers,
        'incorrect_answers' => $incorrect_answers
      ];
    }

    return $response;
  }

  public function verify_user_submission_answers( $assessment_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $answer_list = wpep_utilities_map_object(
      $this->get_user_data( 'answer', $assessment_id, false, false, $user_id, false ),
      'question_id',
      true
    );

    $grading_status    = get_post_meta( $assessment_id, 'wpep_grading_option', true );

    $internal_question_list = wpep_utilities_map_object( EntityAssessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false ), 'id' );
    $internal_answer_list   = EntityAssessment::instance()->get_answers_by_assessment_id( $assessment_id );
    $question_type_data     = wpep_get_assessment_field_types();

    foreach( $answer_list as $question_id => $answer_user_data ) {
      if( !isset( $internal_question_list[$question_id] ) )
        continue;

      $current_question_type_data = $question_type_data[ $internal_question_list[$question_id]->type ];

      $is_correct = ( in_array( $grading_status, [ 1,2 ] ) ? -1 : 1 );

      if( $current_question_type_data['multiple_answers'] && in_array( $grading_status, [ 2,3 ] ) ) {
        $question_correct_answers = wpep_utilities_filter_object_list_by_params( $internal_answer_list, [
          'question_id' => $question_id,
          'is_correct'  => '1'
        ], 'id' );

        if( count( $question_correct_answers ) != count( $answer_user_data ) ) {
          $is_correct = 0;
        } else {
          foreach( $answer_user_data as $current_user_answer ) {
            if( !isset( $question_correct_answers[ $current_user_answer->question_answer_id ] ) ) {
              $is_correct = 0;
              break;
            }
          }
        }

        foreach( $answer_user_data as $answer_user_data_single ) {
          if( !isset( $internal_answer_list[$answer_user_data_single->question_answer_id] ) )
            continue;

          if( $is_correct != 0 && 1 == 1 )
            $is_correct = $internal_answer_list[$answer_user_data_single->question_answer_id]->is_correct;
        }

        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      } else if( $current_question_type_data['has_answers'] && in_array( $grading_status, [ 2,3 ] ) ) {
        $answer_user_data = array_shift( $answer_user_data );

        if( in_array( $grading_status, [ 2,3 ] ) && isset( $answer_user_data->question_answer_id ) )
          $is_correct = ( isset( $internal_answer_list[ $answer_user_data->question_answer_id ] )
                            ? $internal_answer_list[ $answer_user_data->question_answer_id ]->is_correct
                            : 0
                        );

        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      } else {
        WPEP_Controller::instance()->userActivity->assessment->set_user_data(
          'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id
        );
      }

    }

    $this->handle_user_submission_grading( $assessment_id, $user_id );
  }

  public function handle_user_submission_grading( $assessment_id, $user_id ) {
    do_action( "wpep_user_assessment_before_grading", $assessment_id, $user_id );

    $internal_question_list = wpep_utilities_map_object( EntityAssessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false ), 'id' );
    $grading_status    = get_post_meta( $assessment_id, 'wpep_grading_option', true );

    $question_answer_status_list = wpep_utility_map_to_array_assoc(
      $this->get_user_data( 'is_correct', $assessment_id, false, false, $user_id, false ),
      'question_id',
      'value'
    );

    $answers_correct   = 0;
    $answers_graded    = 0;
    $answers_score_max = 0;
    $answers_score     = 0;

    foreach( $question_answer_status_list as $question_id => $is_correct ) {
      if( $is_correct ) {
        $answers_score += $internal_question_list[$question_id]->weight;

        $answers_correct++;
        $answers_graded++;
      } else if( $is_correct != -1 ) {
        $answers_graded++;
      }

      $answers_score_max += $internal_question_list[$question_id]->weight;
    }

    $grading_score = ( $answers_score == 0 ? 0 : ( $answers_score / $answers_score_max ) * 100 );

    $status = WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW;

    if( $grading_status == 0 ) {
      $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;
    } else if( $answers_graded >= count( $internal_question_list ) ) {
      if( $grading_score >= intval( get_post_meta( $assessment_id, 'wpep_grading_requirement', true ) ) )
        $status = WPEP_USER_ASSESSMENT_STATUS_COMPLETED;
      else
        $status = WPEP_USER_ASSESSMENT_STATUS_FAILED;
    }

    WPEP_Controller::instance()->userActivity->assessment->set_user_data(
      WPEP_USER_ASSESSMENT_STATUS, $status, $assessment_id, 0, 0, $user_id
    );

    WPEP_Controller::instance()->userActivity->assessment->set_user_data(
      WPEP_USER_ASSESSMENT_SCORE, $grading_score, $assessment_id, 0, 0, $user_id
    );

    do_action( "wpep_user_assessment_after_grading", $assessment_id, $user_id );
  }

}