<?php

namespace WPEP\Coordinator\UserActivity;

use WPEP\Controller as WPEP_Controller;

class Statistics {

  public function get_started_courses_count( $user_id = false ) {
    return count( $this->get_started_course_list( $user_id ) );
  }

  public function get_completed_course_count( $user_id = false ) {
    return count( $this->get_completed_course_list( $user_id ) );
  }

  public function get_completed_lesson_count( $user_id = false, $completed_from = false, $completed_to = false ) {
    return count( $this->get_completed_lesson_list( $user_id, $completed_from, $completed_to ) );
  }

  /**
   * @param bool|int $user_id
   * @param bool|int $started_at
   * @return array
   */
  public function get_started_course_list( $user_id = false, $started_at = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_STARTED, false, false, false, $user_id, false, $started_at
    ) as $user_data_course_started ) {
      $response[$user_data_course_started->course_id] = $user_data_course_started->value;
    }

    return $response;
  }

  /**
   * @param bool $user_id
   * @return array
   */
  public function get_progressing_course_list( $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_COMPLETED, false, false, false, $user_id, false
    ) as $user_data_course_completed ) {
      if( $user_data_course_completed->value == 0 )
        $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  /**
   * @param bool|int $user_id
   * @param bool|int $completed_at
   * @return array
   */
  public function get_completed_course_list( $user_id = false, $completed_at = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_COMPLETED_AT, false, false, false, $user_id, false, $completed_at
    ) as $user_data_course_completed ) {
      $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  /**
   * @param bool $user_id
   * @param int $is_bookmarked
   * @return array
   */
  public function get_bookmarked_course_list( $user_id = false, $is_bookmarked = 1 ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    $response = [];

    foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_IS_BOOKMARKED, false, false, false, $user_id, false, $is_bookmarked
    ) as $user_data_course_completed ) {
      $response[] = $user_data_course_completed->course_id;
    }

    return $response;
  }

  public function get_course_progress_map( $user_id ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    global $wpdb;

    $sql = 'SELECT p.ID         as course_id,
                   COUNT(l.id)  as lessons_total,
                   COUNT(lc.id) as lessons_completed
             FROM ' . $wpdb->posts . ' p 
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.post_id = p.ID
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON s.id = l.section_id
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc ON l.id = lc.lesson_id AND lc.user_id = ' . intval( $user_id ) . '
             WHERE p.post_type = "'. WPEP_POST_TYPE_COURSE . '"
             GROUP BY p.ID
             HAVING COUNT(l.id) >= 1
             ';

    return $wpdb->get_results( $sql );
  }

  public function get_completed_lesson_list( $user_id = false, $completed_from = false, $completed_to = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return [];

    global $wpdb;

    $sql = 'SELECT lc.lesson_id
             FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc 
             WHERE lc.user_id = ' . intval( $user_id );

    if( $completed_from !== false )
      $sql .= ' AND lc.created_at >= "' . date( "Y-m-d H:i:s", $completed_from) . '"';

    if( $completed_to !== false )
      $sql .= ' AND lc.created_at <= "' . date( "Y-m-d H:i:s", $completed_to) . '"';

    return $wpdb->get_col( $sql );
  }

}