<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class UserEngagement {

  /**
   * @var UserEngagement\WeeklyRecap
   */
  public $weeklyRecap;

  /**
   * @var int|\WP_User
   */
  public $loop_user = 0;
  public $loop_courses_started = [];
  public $loop_courses_completed = [];
  public $loop_lessons_completed_count = 0;

  private $_temporary_shortcodes_registered = [];

  public function __construct() {
    $this->weeklyRecap = new UserEngagement\WeeklyRecap();
  }

  public function get_user_list() {
    return wpep_utility_map_to_array_assoc(
      WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_STARTED, false, false, false, false, false
      ),
      'user_id',
      'user_id'
    );

  }

  private function _register_temporary_shortcodes() {
    if( !empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $this->_temporary_shortcodes_registered = $shortcode_tags;
    remove_all_shortcodes();

    add_shortcode( "user_name", function() {
      return wpep_content_user_profile_format_name( get_user_by( 'id', $this->loop_user->ID ) );
    } );
    add_shortcode( "site_url", function() {
      return get_site_url();
    } );
    add_shortcode( "date", function() {
      return wpep_format_date( time(), 'm/d/y');
    } );
    add_shortcode( "number_lessons_completed", function() {
      return $this->loop_lessons_completed_count;
    } );
    add_shortcode( "list_courses_started", function() {
      return $this->formatted_course_list( $this->loop_courses_started, WPEP_USER_COURSE_STARTED );
    } );
    add_shortcode( "list_courses_completed", function() {
      return $this->formatted_course_list( $this->loop_courses_completed, WPEP_USER_COURSE_COMPLETED_AT );
    } );

    // Conditional ShortCodes

    add_shortcode( "condition_has_started_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_completed_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_completed ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_activity", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) || !empty( $this->loop_courses_completed ) || $this->loop_lessons_completed_count != 0 ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_no_activity", function( $atts, $content ) {
      return empty( $this->loop_courses_started ) && empty( $this->loop_courses_completed ) && $this->loop_lessons_completed_count == 0 ? do_shortcode( $content ) : '';
    } );
  }

  private function _remove_temporary_shortcodes() {
    if( empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $shortcode_tags = $this->_temporary_shortcodes_registered;

    $this->_temporary_shortcodes_registered = [];
  }

  public function send_email( $user_ids, $email_title, $email_content ) {
    if( $user_ids == 0 )
      return false;

    if( !is_array( $user_ids ) )
      $user_ids = [ $user_ids ];

    $this->_register_temporary_shortcodes();

    $user_list = get_users( [
      'include' =>  $user_ids
    ] );

    foreach( $user_list as $user ) {
      $this->loop_user = $user;
      $this->loop_courses_started   = WPEP_Controller::instance()->userActivity->statistics->get_started_course_list( $user->ID, [ strtotime( "-1 week" ), time() ] );
      $this->loop_courses_completed = WPEP_Controller::instance()->userActivity->statistics->get_completed_course_list( $user->ID, [ strtotime( "-1 week" ), time() ] );
      $this->loop_lessons_completed_count = WPEP_Controller::instance()->userActivity->statistics->get_completed_lesson_count( $this->loop_user->ID, strtotime( "-1 week" ), time() );

      $current_email_title   = do_shortcode( $email_title );
      $current_email_content = do_shortcode( $email_content );

      WPEP_Controller::instance()->email->send( $user->user_email, $current_email_title, $current_email_content );
    }

    $this->_remove_temporary_shortcodes();

    return true;
  }

  public function formatted_course_list( $course_ids, $key_field ) {
    if( empty( $course_ids ) )
      return '';

    $response = '<ul>';

    foreach( $course_ids as $course_id ) {
      $response .= '<li>' . wpep_format_date( WPEP_Controller::instance()->userActivity->get_course_user_data_value( $key_field, $course_id, false, false, $this->loop_user->ID ) );
      $response .=  ' - <a href="' . get_the_permalink( $course_id ) . '">' . get_the_title( $course_id ) . '</a>';
      $response .= '</li>';
    }

    $response .= '</ul>';

    return $response;
  }

}