<?php
namespace WPEP\Entity;

class Assessment {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance == null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var \wpdb;
   */
  public $wpdb;
  public $table_question = 'wpep_assessment_question';
  public $table_question_answer = 'wpep_assessment_question_answer';

  public function __construct() {
    $this->_setup();
  }

  private function _setup() {
    global $wpdb;

    $this->wpdb = $wpdb;

    $this->table_question = $this->wpdb->prefix . $this->table_question;
    $this->table_question_answer = $this->wpdb->prefix . $this->table_question_answer;
  }

  public function get_question_by_id( $question_id ) {
    $sql = 'SELECT * FROM ' . $this->table_question . ' WHERE id = ' . $question_id;

    $section = $this->wpdb->get_row($sql);

    return $section;
  }

  public function get_answers_by_assessment_id( $post_id = false ) {
    if( $post_id == false )
      $post_id = get_the_ID();

    $sql = 'SELECT * FROM ' . $this->table_question_answer . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC';

    $questions = $this->wpdb->get_results($sql);

    return wpep_utilities_map_object( $questions, 'id' );
  }

  public function get_questions_by_assessment_id( $post_id = false, $with_answers = true, $handle_shuffle_settings = false ) {
    if( $post_id == false )
      $post_id = get_the_ID();

    $sql = 'SELECT * FROM ' . $this->table_question . ' WHERE post_id = ' . $post_id . ' ORDER BY `order` ASC';

    $questions = $this->wpdb->get_results($sql);

    foreach( $questions as $k => $question ) {

      if( $with_answers )
          $questions[$k]->answers = $this->get_question_answers_by_question_id( $question->id );

      if( $handle_shuffle_settings && get_post_meta( $post_id, 'wpep_shuffle_answers', true ) == 1 && !empty( $questions[$k]->answers ) )
         shuffle( $questions[$k]->answers );
    }

    if( $handle_shuffle_settings && get_post_meta( $post_id, 'wpep_shuffle_questions', true ) == 1 && !empty( $questions ) )
      shuffle( $questions );

    return $questions;
  }

  public function get_question_answers_by_question_id( $question_id ) {
    if( $question_id === '' )
      return [];

    $sql = 'SELECT * FROM ' . $this->table_question_answer . ' WHERE `question_id` = ' . intval( $question_id ) . ' ORDER BY `order` ASC';

    return $this->wpdb->get_results($sql);
  }

  public function insert_question( $data ) {
    $this->wpdb->insert( $this->table_question, $data );

    return $this->wpdb->insert_id;
  }

  public function insert_question_answer( $data ) {
    $this->wpdb->insert( $this->table_question_answer, $data );

    return $this->wpdb->insert_id;
  }

  public function update_question( $data, $where = [] ) {
    $this->wpdb->update( $this->table_question, $data, $where );
  }

  public function update_question_answer( $data, $where = [] ) {
    $this->wpdb->update( $this->table_question_answer, $data, $where );
  }

  public function delete_question_by_id( $question_id ) {
    $this->wpdb->delete( $this->table_question, [ 'id' => $question_id ] );
  }

  public function delete_answers_by_question_id( $question_id ) {
    $this->wpdb->delete( $this->table_question_answer, [ 'question_id' => $question_id ] );
  }

  public function delete_answer_by_id( $question_answer_id ) {
    $this->wpdb->delete( $this->table_question_answer, [ 'id' => $question_answer_id ] );
  }
  
}