<?php

namespace WPEP\Posts;

use WPEP\Controller as WPEP_Controller;

/**
 * declare Courses class.
 *
 * @since        1.0
 */
class Courses extends Skeleton {


  /**
   * config custom post type.
   *
   * @access    public
   * @static
   * @return      void
   * @since       1.0
   */
  public static function config() {
    $supports = [ 'title' ];

    if( wpep_get_setting_flag( 'course-layout-content-editor' ) )
      $supports[] = 'editor';

    if( wpep_get_setting_flag( 'enable-course-comments' ) )
      $supports[] = 'comments';

    $post_config = [
      'supports'            => $supports,
      'taxonomies'          => [ WPEP_TAXONOMY_CATEGORY ],
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => true,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => intval( wpep_get_setting( 'menu-position' ) ),
      'menu_icon'           => '',
      'can_export'          => true,
      'has_archive'         => wpep_get_setting_flag( 'enable-course-archive' ),
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capability_type'     => WPEP_POST_CAP_COURSE,
      'map_meta_cap'        => true,
      'rewrite'             => [
        'slug'  => wpep_get_setting( 'rewrite-slug-courses', WPEP_POST_TYPE_COURSE )
      ]
    ];

    self::$post_config['singular_name'] = 'Course';
    self::$post_config['plural_name']   = 'Courses';
    self::$post_config['menu_name']     = 'WPEP';
    self::$post_config['config']        = $post_config;
    self::$post_config['post_type']     = WPEP_POST_TYPE_COURSE;

    self::register();

    self::_registerCategoryTaxonomy();
  } /* config() */

  private static function _registerCategoryTaxonomy() {
    $labels = [
      'name'                       => __('WPEP Categories', 'wpep'),
      'singular_name'              => __('Plan Category', 'wpep'),
      'menu_name'                  => __('Categories', 'wpep'),
      'all_items'                  => __('All Items', 'wpep'),
      'parent_item'                => __('Parent Item', 'wpep'),
      'parent_item_colon'          => __('Parent Item:', 'wpep'),
      'new_item_name'              => __('New Category', 'wpep'),
      'add_new_item'               => __('Add New Category', 'wpep'),
      'edit_item'                  => __('Edit Category', 'wpep'),
      'update_item'                => __('Update Category', 'wpep'),
      'view_item'                  => __('View Category', 'wpep'),
      'separate_items_with_commas' => __('Separate items with commas', 'wpep'),
      'add_or_remove_items'        => __('Add or remove items', 'wpep'),
      'choose_from_most_used'      => __('Choose from the most used', 'wpep'),
      'popular_items'              => __('Popular Categories', 'wpep'),
      'search_items'               => __('Search Categories', 'wpep'),
      'not_found'                  => __('Not Found', 'wpep'),
    ];
    $args   = [
      'labels'            => $labels,
      'hierarchical'      => true,
      'public'            => true,
      'show_ui'           => true,
      'show_admin_column' => true,
      'show_in_nav_menus' => true,
      'show_tagcloud'     => false,
      'rewrite'             => [
        'slug'  => wpep_get_setting( 'rewrite-slug-categories', 'course_category' )
      ],
      'capabilities'          => array(
        'manage_terms' => 'manage_' . WPEP_POST_CAP_COURSE . '_terms',
        'edit_terms'   => 'edit_'   . WPEP_POST_CAP_COURSE . '_terms',
        'delete_terms' => 'delete_' . WPEP_POST_CAP_COURSE . '_terms',
        'assign_terms' => 'assign_' . WPEP_POST_CAP_COURSE . '_terms',
      ),
    ];

    if (!class_exists('Tax_Meta_Class'))
      require_once( WPEP_BASE_PATH . '/lib/Tax-meta-class/tax-meta-class.php');

    $config = [
        'id'             => 'course_category_custom_fields',
        'pages'          => [ 'course_category' ],
        'context'        => 'normal',
        'fields'         => [],
        'local_images'   => false,
        'use_with_theme' => false
    ];

    $customTaxonomyFields = new \Tax_Meta_Class($config);
    $customTaxonomyFields->addImage(
      'icon',
      [
        'name' => 'Category Image ',
        'desc' => 'Recommended size is : 100x100.'
      ]
    );
    $customTaxonomyFields->addText(
      'url',
      [
        'name' => 'Link',
        'desc' => 'If you want this category to redirect the user to a specific page, add a link here.'
      ]
    );
    $customTaxonomyFields->Finish();

    register_taxonomy('course_category', [ WPEP_POST_TYPE_COURSE ], $args);
  }

}
