<?php

namespace WPEP\Widgets;

use WPEP\Controller as WPEP_Controller;

/**
 * declare WPEPWidget class.
 *
 * @since        1.0
 */
class Courses extends \WP_Widget {


  public function __construct() {

    parent::__construct(
        'wpepc_widget', /* id of widget */
        __('WPEP Courses', "wpep"), /* widget name */
        [
          'description' => __('Display Course Presentation Tabs in your Sidebar', "wpep")
        ]
    );

  } /* __construct() */


  public function widget($args, $instance) {
    echo $args['before_widget'];

    WPEP_Controller::instance()->templateStyle->before_main_content();

    do_action( "wpep_widget_content_before_grid" );

    $course_list = $this->_get_widget_course_list( $instance['noc'], true );

    if( count( $course_list ) < $instance['noc'] )
      $course_list = $this->_get_widget_course_list( $instance['noc'] - count( $course_list ), false ) + $course_list;

    if( !empty( $course_list ) ) {

      echo WPEP_Controller::instance()->template->get_template( 'widgets/courses.php', [
        'course_list' => $course_list
      ] );

    }

    do_action( "wpep_widget_content_after_grid" );

    echo $args['after_widget'];

  }

  private function _get_widget_course_list( $posts_per_page, $after ) {
    global $post;

    $course_list_args = [
      'post_type'      => WPEP_POST_TYPE_COURSE,
      'post_status'    => 'publish',
      'orderby'        => 'meta_value',
      'meta_key'       => 'wpep_order',
      'meta_type'      => 'NUMERIC',
      'posts_per_page' => $posts_per_page,
      'paged'          => 1,
      'post__not_in'   => ( isset( $post->ID ) ? [ $post->ID ] : [] ),
      'meta_query'  => [
        [
          'key'     => 'wpep_show_on_index',
          'value'   => '1',
          'compare' => '=',
        ]
      ]
    ];

    $order = ( isset( $post->ID ) ? intval( get_post_meta( $post->ID, 'wpep_order', true ) ) : 0 );

    if( $after ) {
      $course_list_args['order'] = 'ASC';
      $course_list_args['meta_query'][] = [
        'key'     => 'wpep_order',
        'compare' => '>=',
        'value'   => $order,
        'type'    => 'NUMERIC'
      ];
    } else {
      $course_list_args['order'] = 'DESC';
      $course_list_args['meta_query'][] = [
        'key'     => 'wpep_order',
        'compare' => '<=',
        'value'   => $order,
        'type'    => 'NUMERIC'
      ];
    }

    $course_list = get_posts( $course_list_args );

    if( $after == false )
      $course_list = array_reverse( $course_list );

    $course_list = wpep_utilities_map_object( $course_list );

    return $course_list;
  }

  public function form($instance) {
    $noc = isset( $instance['noc'] ) ? intval( $instance['noc'] ) : 3;
    ?>
    <p>
      <label for="<?php echo $this->get_field_id('noc'); ?>"><?php _e('No of Courses:', "wpep"); ?></label>
      <input class="widefat" id="<?php echo $this->get_field_id('noc'); ?>"
             name="<?php echo $this->get_field_name('noc'); ?>" type="text" value="<?php echo esc_attr($noc); ?>"/>
    </p>
  <?php

  }

  public function update($new_instance, $old_instance) {

    $instance        = [];
    $instance['noc'] = (!empty($new_instance['noc'])) ? strip_tags($new_instance['noc']) : '';

    return $instance;

  }

}