<?php

namespace WPEP\Widgets;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Configuration;

/**
 * declare WPEPWidget class.
 *
 * @since        1.0
 */
class Image extends \WP_Widget {

  use WPEP_Configuration;

  public function __construct() {

    parent::__construct(
        'wpep_image', /* id of widget */
        __('WPEP Image', "wpep"), /* widget name */
        [
          'description' => __('Display an image and opt. incl. a redirect URL.', "wpep")
        ]
    );

  } /* __construct() */


  public function widget($args, $instance) {
    echo $args['before_widget'];

    if ( ! empty( $instance['title'] ) ) {
      echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
    }

    if( ! empty($instance['image']) ) {
      if( !empty( $instance[ 'link' ] ) )
        echo '<a href="' . $instance[ 'link' ] . '" ' . ( intval( $instance['link_new_tab' ] ) == 0 ? '' : ' target="_blank"' ) . '>';

      echo '<img src="' . $instance['image'] . '" style="display:block;width:100%;"/>';

      if( !empty( $instance[ 'link' ] ) )
        echo '</a>';
    }


    echo $args['after_widget'];
  }


  public function form($instance) {
    $title        = ! empty( $instance['title'] ) ? $instance['title'] : '';
    $image        = ! empty( $instance['image'] ) ? $instance['image'] : '';
    $link         = ! empty( $instance['link'] )  ? $instance['link'] : '';
    $link_new_tab = ! empty( $instance['link_new_tab'] )  ? $instance['link_new_tab'] : 0;

    echo '<div class="wpep-image-widget-settings">';
    echo '<p class="title">' . __( "Widget Title", "wpep" ) . '</p>';
    $this->display_text_field ( 'title', $this->get_field_name( 'title' ), $title, false, __( "Leave this empty to not display a title.", "wpep" ) );

    echo '<p class="title">' . __( "Image", "wpep" ) . '</p>';
    $this->display_image_field( 'image', "Image", $this->get_field_name( 'image' ), $image );

    echo '<p class="title">' . __( "Link", "wpep" ) . '</p>';
    $this->display_text_field ( 'link', $this->get_field_name( 'link' ), $link, false );
    echo '<p class="description">' . __( "Attach a link to the image.", "wpep" ) . '</p>';

    echo '<p class="widget-checkbox-container">' . __( "Open Link in New Tab", "wpep" );
    $this->display_checkbox_field('link_new_tab', $link_new_tab, $this->get_field_name( 'link_new_tab' ), 0 );
    echo '</p>';
    echo '</div>';
  }

  public function update( $new_instance, $old_instance ) {
    $instance = [];
    $instance['title']        = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
    $instance['image']        = ( ! empty( $new_instance['image'] ) ) ? sanitize_text_field( $new_instance['image'] ) : '';
    $instance['link']         = ( ! empty( $new_instance['link'] ) ) ? strip_tags( $new_instance['link'] ) : '';
    $instance['link_new_tab'] = ( ! empty( $new_instance['link_new_tab'] ) ) ? strip_tags( $new_instance['link_new_tab'] ) : '';

    return $instance;
  }

}