<?php

namespace WPEP\Widgets;

use WPEP\Controller as WPEP_Controller;
use WPEP\Admin\Container\Configuration as WPEP_Configuration;

/**
 * declare WPEPWidget class.
 *
 * @since        1.0
 */
class Search extends \WP_Widget {

  use WPEP_Configuration;

  public function __construct() {

    parent::__construct(
        'wpep_search', /* id of widget */
        __('WPEP Search', "wpep"), /* widget name */
        [
          'description' => __( 'Deploy a Searchbar for each Individual WPEP Content Type or one that searches them all.', "wpep")
        ]
    );

  } /* __construct() */


  public function widget($args, $instance) {
    echo $args['before_widget'];

    if ( ! empty( $instance['title'] ) ) {
      echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
    }

    $post_type_list = ! empty( $instance['post_types'] ) ? $instance['post_types'] : '';

    if( is_array( $post_type_list ) ) {
      foreach( $post_type_list as $post_type => $post_type_status ) {
        if( $post_type_status == 0 )
          unset( $post_type_list[ $post_type ] );
      }

      $post_type_list = implode( ",", array_keys( $post_type_list ) );
    }

    WPEP_Controller::instance()->template->load_template( "search/form.php", [
      'post_type_list' => $post_type_list
    ] );

    echo $args['after_widget'];
  }


  public function form($instance) {
    $title        = ! empty( $instance['title'] )      ? $instance['title'] : '';
    $post_types   = ! empty( $instance['post_types'] ) ? $instance['post_types'] : [];

    echo '<div class="wpep-search-widget-settings">';
    echo '<p class="title">' . __( "Widget Title", "wpep" ) . '</p>';
      $this->display_text_field ( 'title', $this->get_field_name( 'title' ), $title, false, __( "Leave this empty to not display a title.", "wpep" ) );

    echo '<p class="title">' . __( "Search Trough", "wpep" ) . '</p>';

    foreach( [
      WPEP_POST_TYPE_COURSE => __( "Courses", "wpep" ),
      WPEP_POST_TYPE_E_BOOK => __( "E-Books", "wpep" ),
      WPEP_POST_TYPE_VIDEO  => __( "Videos", "wpep" ),
      WPEP_POST_TYPE_OFFER  => __( "Offers", "wpep" ),
             ] as $post_type => $post_type_label ) {
      echo '<p>';
      echo  '<label>';
              $this->display_checkbox_field( 'post_types', ( isset( $post_types[ $post_type ] ) ? $post_types[ $post_type ] : 1 ), $this->get_field_name( 'post_types[' . $post_type . ']' ) );
      echo    $post_type_label;
      echo  '</label>';
      echo '</p>';
    }

    echo '</div>';
  }

  public function update( $new_instance, $old_instance ) {
    $instance = [];
    $instance['title']        = ( ! empty( $new_instance['title'] ) )      ? sanitize_text_field( $new_instance['title'] ) : '';
    $instance['post_types']   = ( ! empty( $new_instance['post_types'] ) ) ? $new_instance['post_types'] : [];

    return $instance;
  }

}