<?php

/**
 * @return array|mixed|string
 */
function wpep_assessments_enabled() {
  return wpep_get_setting_flag( 'enable-course-assessments' );
}

function wpep_get_assessment_course_id( $assessment_id ) {
  return intval( get_post_meta( $assessment_id, 'wpep_course_id', true ) );
}

function wpep_get_assessment_course_section_id( $assessment_id ) {
  return intval( get_post_meta( $assessment_id, 'wpep_section_id', true ) );
}

/**
 * @return string
 */
function wpep_get_assessment_field_type_default() {
  return 'input_text';
}

/**
 * @return array
 */
function wpep_get_assessment_field_types() {
  $assessment_field_types = [
    'input_text'  => [
      'label'             => __( "Free Text", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'correct_answers'   => 0,
    ],
    'textarea'  => [
      'label'             => __( "Essay", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'correct_answers'   => 0,
    ],
    'radio'  => [
      'label'             => __( "Single Answer", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'correct_answers'   => 0,
    ],
    'checkbox'  => [
      'label'             => __( "Multiple Answer", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 1,
      'correct_answers'   => 0,
    ],
    'select'  => [
      'label'             => __( "Single Select", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'correct_answers'   => 0,
    ],
    'input_match' => [
      'label'             => __( "Text Match", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'correct_answers'   => 1,
    ],
    'file' => [
      'label'             => __( "File Upload", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'correct_answers'   => 0,
    ]
  ];

  return $assessment_field_types;
}

/**
 * @param $course_id
 * @param array $assessment_status
 * @return array
 */
function wpep_get_course_assessment_ids(
  $course_id,
  $assessment_status = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ]
) {
  $args = [
    'post_type'   => WPEP_POST_TYPE_ASSESSMENT,
    'fields'      => 'ids',
    'order'       => 'DESC',
    'orderby'     => 'DATE',
    'meta_query'  => [
      [
        'key'   => 'wpep_course_id',
        'value' => $course_id
      ]
    ],
    'post_status' => $assessment_status
  ];

  $course_assessment_ids = get_posts( $args );

  foreach( $course_assessment_ids as $key => $course_assessment_id ) {
    if( get_post_meta( $course_assessment_id, 'wpep_course_id', true ) == $course_id )
      continue;

    unset( $course_assessment_ids[ $key ] );
  }

  return $course_assessment_ids;
}

/**
 * @param $course_id
 * @param int $section_id
 * @param array $assessment_status
 * @return int|mixed
 */
function wpep_get_course_assessment_id(
  $course_id,
  $section_id = 0,
  $assessment_status = [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit' ]
) {
  $args = [
    'post_type'   => WPEP_POST_TYPE_ASSESSMENT,
    'fields'      => 'ids',
    'order'       => 'DESC',
    'orderby'     => 'DATE',
    'meta_query'  => [
      [
        'key'   => 'wpep_course_id',
        'value' => $course_id
      ],
      [
        'key'   => 'wpep_section_id',
        'value' => $section_id
      ]
    ],
    'post_status' => $assessment_status
  ];

  $course_assessments = get_posts( $args );

  if( empty( $course_assessments ) )
    return 0;

  foreach( $course_assessments as $course_assessment ) {
    if( get_post_meta( $course_assessment, 'wpep_course_id', true ) != $course_id )
      continue;

    if( get_post_meta( $course_assessment, 'wpep_section_id', true ) != $section_id )
      continue;

    return $course_assessment;
  }

  return 0;
}

/**
 * @param $assessment_id
 * @param bool $user_status
 * @return bool
 */
function wpep_allow_assessment_submission( $assessment_id, $user_status = false ) {
  if( $user_status == false )
    wpep_user_assessment_status( $assessment_id );

  if( $user_status == WPEP_USER_ASSESSMENT_STATUS_PENDING )
    return true;

  if( get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) == 0 )
    return false;

  if( in_array( $user_status, [ WPEP_USER_ASSESSMENT_STATUS_FAILED, WPEP_USER_ASSESSMENT_STATUS_COMPLETED ] )
    && get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) == 1 )
    return false;

  return true;
}