<?php

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

/**
 * @return int|bool
 */
function wpep_content_primary_sidebar_active() {
  $response = is_active_sidebar('wpep-sidebar');

  $response = apply_filters( 'wpep_single_content_sidebar_active', $response );

  return $response;
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_categories() {
  return WPEP_Controller::instance()->content->primaryCoordinator->has_categories();
}

if( !function_exists( 'wpep_content_primary_available_post_types' ) ) {
  function wpep_content_primary_available_post_types() {
    return [
      WPEP_POST_TYPE_COURSE,
      WPEP_POST_TYPE_VIDEO,
      WPEP_POST_TYPE_E_BOOK,
      WPEP_POST_TYPE_OFFER
    ];
  }
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_categories_navigation() {
  return WPEP_Controller::instance()->content->primaryCoordinator->has_categories_navigation();
}

/**
 * @return int|bool
 */
function wpep_content_primary_has_all_categories_navigation() {
  return WPEP_Controller::instance()->content->primaryCoordinator->has_all_categories_navigation();
}

/**
 * @return int|bool
 */
function wpep_content_primary_course_grid_has_progress_bar() {
  return WPEP_Controller::instance()->content->primaryCoordinator->gridItem->course_grid_has_progress_bar();
}

function wpep_content_primary_order_options() {
  $order_options = [
    'ASC'   => __( 'Default - Ascending', 'wpep' ),
    'DESC'  => __( 'Descending', 'wpep' )
  ];

  return apply_filters( 'wpep_index_page_post_order_options', $order_options );
}

function wpep_content_primary_order_by_options() {
  $order_options = [
    'meta_value' => __( 'Default - Manual Order', 'wpep' ),
    'title'      => __( 'Title', 'wpep' ),
    'date'       => __( 'Publish Date', 'wpep' ),
    'modified'   => __( 'Last Updated Date', 'wpep' ),
    'post__in'   => __( "Advanced : Order in which you've selected specific items.", "wpep" )
  ];

  return apply_filters( 'wpep_index_page_post_order_by_options', $order_options );
}

/**
 * @param null $post_type
 * @return array
 */
function wpep_content_primary_row_options( $post_type = null ) {
  $index_page_row_options = [
    1 => sprintf( __( '%s Per Row', 'wpep' ), '1' ),
    2 => sprintf( __( '%s Per Row', 'wpep' ), '2' ),
    3 => sprintf( __( '%s Per Row', 'wpep' ), '3' ),
    4 => sprintf( __( '%s Per Row', 'wpep' ), '4' ),
  ];

  if( $post_type === WPEP_POST_TYPE_COURSE )
    $index_page_row_options = [ WPEP_ITEMS_PER_ROW_DYNAMIC => __( "Dynamic Course Tab Placement", "wpep" ) ] + $index_page_row_options;

  return apply_filters( 'wpep_index_page_items_per_row', $index_page_row_options );
}

/**
 * @return array
 */
function wpep_content_categories_row_options() {
  $courses_per_row = [
    0 => __( "Autodetect", 'wpep' ),
    1 => __( "Disable Category Navigation", "wpep" ),
    2 => sprintf( __( '%s Per Row', 'wpep' ), '2' ),
    3 => sprintf( __( '%s Per Row', 'wpep' ), '3' ),
    4 => sprintf( __( '%s Per Row', 'wpep' ), '4' ),
    5 => sprintf( __( '%s Per Row', 'wpep' ), '5' ),
    6 => sprintf( __( '%s Per Row', 'wpep' ), '6' ),
    7 => sprintf( __( '%s Per Row', 'wpep' ), '7' ),
    8 => sprintf( __( '%s Per Row', 'wpep' ), '8' ),
  ];

  return apply_filters( 'wpep_index_page_categories_per_row', $courses_per_row );
}

/**
 * @param $video_post
 * @return string
 */
function wpep_content_primary_video_post_type_content( $video_post ) {
  if( $video_post === false ) {
    $resp = '';

    $resp .= '<div class="video-page-container wpep-hidden">';

    $resp .= '</div>';

    return $resp;
  }

  $video_post = ( is_object( $video_post ) ? $video_post : get_post( $video_post ) );

  global $post;

  $previous_post = $post;
  $post = $video_post;

  setup_postdata( $post );

  $resp = '';

  $resp .= '<div class="video-page-container">';
  $resp .=  '<h2 class="video-title">' . $video_post->post_title . '</h2>';
  $resp .=  '<h3 class="video-excerpt wpep-content-from-editor">' . $video_post->post_excerpt . '</h3>';
  $resp .=  '<div class="video-container">' . wpep_get_video_source_output( get_post_meta( $video_post->ID, '_video_link', true) ) . '</div>';
  $resp .=  '<div class="description wpep-content-from-editor">' . wpautop( $video_post->post_content ). '</div>';
  $resp .=  '<div class="clearfix"></div>';
  $resp .= '</div>';

  $resp = apply_filters( 'the_content', $resp );

  $post = $previous_post;
  setup_postdata( $post );

  return $resp;
}

/**
 * @param string|array $course_list_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_course_list_by_ids( $course_list_ids, $options = [] ) {
  if( empty( $course_list_ids ) )
    return '';

  return do_shortcode( wpep_content_primary_course_list_shortcode_by_ids( $course_list_ids, $options ) );
}

/**
 * @param $course_list_ids
 * @param array $options
 * @return string
 */
function wpep_content_primary_course_list_shortcode_by_ids( $course_list_ids, $options = [] ) {
  if( empty( $course_list_ids ) )
    return '';

  $course_list_ids = ( is_array( $course_list_ids ) ? implode( ",", $course_list_ids ) : $course_list_ids );

  $options = array_merge( [
    'category_navigation' => 0,
    'display_header'      => 0,
    'courses_per_row'     => 3,
    'videos_per_row'      => 0,
    'ebooks_per_row'      => 0,
    'offers_per_row'      => 0
  ], $options );

  $response = '[wpep_index courses="' . $course_list_ids . '"';

  foreach( $options as $option_key => $option_value )
    $response .= ' ' . $option_key . '="' . $option_value . '"';

  $response .= ']';

  return $response;
}