<?php

function wpep_public_profiles_enabled() {
  return apply_filters( 'wpep_public_profiles_enabled', wpep_get_setting_flag( 'user-profile-public-active' ) );
}

/**
 * @param bool $user_id
 * @return array
 */
function wpep_content_user_profile_tabs( $user_id = false ) {
  $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

  if( $user_id == 0 )
    return [];


  $progressing_course_list = wpep_controller()->userActivity->statistics->get_progressing_course_list( $user_id );
  $completed_course_list   = wpep_controller()->userActivity->statistics->get_completed_course_list( $user_id );

  $tab_list = [];

  if( !empty( $progressing_course_list ) ) {
    $tab_list['progressing_course_list'] = [
      'title'         => wpep_get_setting_translated( 'user-profile-tab-progressing-courses' ),
      'entries_count' => count( $progressing_course_list ),
      'content'       => wpep_content_primary_course_list_shortcode_by_ids( $progressing_course_list,
        [
          'course_progress_bars'  => (
            wpep_get_setting_flag( 'user-profile-course-public-progress-bars' )
             ? true
             : ( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) || $user_id == get_current_user_id() )
          ),
          'courses_per_row'       => wpep_get_setting( 'user-profile-courses-per-row' ),
          'user_id'               => $user_id
        ]
      )
    ];
  }

  if( !empty( $completed_course_list ) ) {
    $tab_list['completed_course_list'] = [
      'title'         => wpep_get_setting_translated( 'user-profile-tab-completed-courses' ),
      'entries_count' => count( $completed_course_list ),
      'content'       => wpep_content_primary_course_list_shortcode_by_ids( $completed_course_list, [ 'course_progress_bars' => 0, 'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' ) ] )
    ];
  }

  if( wpep_course_has_bookmarks() ) {
    $bookmarked_course_list   = wpep_controller()->userActivity->statistics->get_bookmarked_course_list( $user_id, 1 );

    if( !empty( $bookmarked_course_list ) ) {
      $tab_list['bookmarked_course_list'] = [
        'title'         => wpep_get_setting_translated( 'user-profile-tab-bookmarked-courses' ),
        'entries_count' => count( $bookmarked_course_list ),
        'content'       => wpep_content_primary_course_list_shortcode_by_ids( $bookmarked_course_list, [ 'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' ) ] )
      ];
    }
  }

  return apply_filters( 'wpep_content_user_profile_tab_list', $tab_list, $user_id );
}

/**
 * @param \WP_User|int $user
 * @return mixed
 */
function wpep_content_user_profile_format_name( $user ) {
  $user = ( is_numeric( $user ) ? get_user_by( 'ID', intval( $user ) ) : $user );

  $name = trim ( $user->first_name . ' ' . $user->last_name );

  return ( $name == '' ? $user->display_name : $name );
}

/**
 * @return array
 */
function wpep_user_profile_social_network_platform_list_active() {
  $active_social_networks = wpep_get_setting( 'user-profile-enabled-social-networks' );
  $response = wpep_user_profile_social_network_platform_list();

  foreach( $response as $key => $r )
    if( !in_array( $key, $active_social_networks ) )
      unset( $response[ $key ] );


  return $response;
}

function wpep_user_profile_public_link( $id = false ) {
  $user = ( is_object( $id ) ? $id : get_user_by( 'id', intval( $id === false ? get_current_user_id() : $id ) ) );

  if( $user === false )
    return false;

  return get_site_url() . '/' . wpep_controller()->content->userProfileCoordinator->user_identifier . '/' . $user->user_login . '/';
}

function wpep_user_profile_picture( $user_id ) {
  return get_avatar( $user_id, 256, wpep_get_setting( 'user-profile-default-profile-picture' ) );
}

function wpep_user_profile_picture_url( $user_id ) {
  $profile_picture = get_user_meta( $user_id, WPEP_USER_META_PROFILE_PICTURE, true );

  if( $profile_picture != '' )
    return esc_url( $profile_picture );

  return get_avatar_url( $user_id, [
    'size'    => 256,
    'default' => wpep_get_setting( 'user-profile-default-profile-picture' )
  ] );
}

function wpep_user_profile_description_excerpt( $user_id ) {
  $excerpt = get_the_author_meta('description', $user_id );
  $excerpt = strip_shortcodes( $excerpt );
  $excerpt = strip_tags( $excerpt );
  $excerpt = wp_trim_words( $excerpt, 15 );

  return htmlentities( $excerpt, ENT_QUOTES, "UTF-8" );
}

/**
 * @return array
 */
function wpep_user_profile_social_network_platform_list() {
  return [
    'facebook'  => [
      'name'   => "Facebook",
      'icon'   => "wpep-social-media-icon-facebook",
      'type'   => 'link'
    ],
    'twitter'   => [
      'name'   => "Twitter",
      'icon'   => "wpep-social-media-icon-twitter",
      'type'   => 'link'
    ],
    'instagram'   => [
      'name'   => "Instagram",
      'icon'   => "wpep-social-media-icon-instagram",
      'type'   => 'link'
    ],
    'linkedin'    => [
      'name'   => "Linkedin",
      'icon'   => "wpep-social-media-icon-linkedin",
      'type'   => 'link'
    ],
    'youtube'     => [
      'name'   => "Youtube",
      'icon'   => "wpep-social-media-icon-youtube",
      'type'   => 'link'
    ],
    'pinterest'   => [
      'name'   => "Pinterest",
      'icon'   => "wpep-social-media-icon-pinterest",
      'type'   => 'link'
    ],
    'google_plus' => [
      'name'   => "Google+",
      'icon'   => "wpep-social-media-icon-gplus",
      'type'   => 'link'
    ],
    'skype' => [
      'name'   => "Skype",
      'icon'   => "wpep-social-media-icon-skype"
    ],
    'snapchat' => [
      'name'   => "Snapchat",
      'icon'   => "wpep-social-media-icon-snapchat"
    ],
    'vk' => [
      'name'   => "VKontakte",
      'icon'   => "wpep-social-media-icon-vk",
      'type'   => 'link'
    ],
    'reddit' => [
      'name'   => "Reddit",
      'icon'   => "wpep-social-media-icon-reddit",
      'type'   => 'link'
    ],
    'deviant_art' => [
      'name'   => "Deviant Art",
      'icon'   => "wpep-social-media-icon-deviant-art",
      'type'   => 'link'
    ],
    'behance' => [
      'name'   => "Behance",
      'icon'   => "wpep-social-media-icon-behance",
      'type'   => 'link'
    ],
    'xing' => [
      'name'   => "Xing",
      'icon'   => "wpep-social-media-icon-xing",
      'type'   => 'link'
    ],
  ];
}