<?php

if( !function_exists( 'wpep_format_content' ) ) {

  /**
   * @param $content
   * @return string
   */
  function wpep_format_content( $content ) {
    return do_shortcode( wpautop( $content ) );
  }

}

if( !function_exists( 'wpep_get_video_source_output' ) ) {

  /**
   * @param $video_embed_source
   * @param $enable_js_api
   * @return string
   */
  function wpep_get_video_source_output( $video_embed_source, $enable_js_api = false ) {
    global $post;

    $is_null = ( isset( $post->ID ) ? 0 : 1 );

    if( $is_null ) {
      $post = new \stdClass();
      $post->ID = 999999999999; // We cheat a bit, heh ?
    }

    $video_embed_source = trim( $video_embed_source );

    if( wpep_string_ends_with( $video_embed_source, ['.mp4', '.ogv', '.flv'] ) ) {
      $response = wp_video_shortcode( [ 'src' => $video_embed_source, 'width' => 640, 'height' => 480] );
    } else {
      global $wp_embed;

      $response = $wp_embed->run_shortcode( '[embed width=640 height=480]' . $video_embed_source . '[/embed]' );

      if( $enable_js_api )
        $response = wpep_get_video_source_output_with_js_api( $response );
    }

    if( $is_null )
      $post = null;

    return $response;
  }

}

if( !function_exists( 'wpep_get_video_source_output_with_js_api' ) ) {

  /**
   * @param $html
   * @return mixed
   */
  function wpep_get_video_source_output_with_js_api($html) {
    if (strstr($html, 'youtube.com/embed/')) {
      $replace = '?feature=oembed&enablejsapi=1';

      if( !strstr( $html, 'autoplay=' ) )
        $replace .= '&autoplay=0';

      $html = str_replace('?feature=oembed', $replace, $html);
    }

    return $html;
  }

}

if( !function_exists( 'wpep_get_indexed_number_list' ) ) {

  /**
   * @param $from
   * @param $to
   * @param string $format
   * @return array
   */
  function wpep_get_indexed_number_list( $from, $to, $format = '%value%' ) {
    $response = [];

    for( ;$from <= $to; $from++ )
      $response[$from] = str_replace( '%value%', $from, $format );

    return $response;
  }

}

if( !function_exists( 'wpep_implode_associative_array' ) ) {

  /**
   * @param $array
   * @param $inner_separator
   * @param $outer_separator
   * @return string
   */
  function wpep_implode_associative_array( $array, $inner_separator, $outer_separator ) {
    $response = '';

    foreach ($array as $k=>$v) {
      if( $response != '' )
        $response .= $outer_separator;

      $response .= $k . $inner_separator . $v;
    }

    return $response;
  }

}

if( !function_exists( 'wpep_alias_to_label' ) ) {

  /**
   * @param $alias
   * @return string
   */
  function wpep_alias_to_label( $alias ) {
    return ucwords( str_replace( [ '_', '-' ], ' ', $alias ) );
  }

}

if( !function_exists( 'wpep_label_to_alias' ) ) {

  /**
   * @param $label
   * @return string
   */
  function wpep_label_to_alias( $label ) {
    return strtolower( str_replace( ' ', '_', $label ) );
  }

}

if( !function_exists( 'wpep_format_report_data_to_line' ) ) {

  /**
   * @param $entry_key
   * @param $entry_data
   * @param string $format
   * @param string $class_status_prefix
   * @return string
   */
  function wpep_format_report_data_to_line( $entry_key, $entry_data, $format = "%label% : %value%", $class_status_prefix = 'wpep-row-highlight  wpep-row-td-highlight-' ) {
    $label  = '';
    $status = '';

    if( is_array( $entry_data ) ) {
      $value = $entry_data['value'];

      if( isset( $entry_data['status'] ) )
        $status = $entry_data['status'];

      if( isset( $entry_data['type'] ) ) {
        if( in_array( $entry_data['type'], [ 'table', 'user_role', 'plugin', 'php_var', 'storage_size', 'php_function' ] ) ) {
          $label = $entry_key;
        }

        if( in_array( $entry_data['type'], [ 'storage_size' ] ) )
          $value = size_format( $value );

      }
    } else {
      $value = $entry_data;
    }

    if( $label == '' ) {
      $label = wpep_alias_to_label( $entry_key );
      $label = str_ireplace( 'url', 'URL', $label );
      $label = str_ireplace( 'php', 'PHP', $label );
      $label = str_ireplace( 'wpep', 'WPEP', $label );
      $label = str_ireplace( 'mysql', 'MySQL', $label );
      $label = str_replace( 'Wp', 'WordPress', $label );
    }

    if( is_bool( $value ) ) {
      $value = ( $value ? __( "Yes", "wpep" ) : __( "No", "wpep" ) );
    }

    $format = str_replace( '%label%', $label, $format );
    $format = str_replace( '%value%', $value, $format );
    $format = str_replace( '%class_status%', ( $status != '' ? $class_status_prefix . $status : '' ), $format );

    return $format;
  }

}

if( !function_exists( 'wpep_string_ends_with' ) ) {

  /**
   * @param $haystack
   * @param $needle
   * @return bool
   */
  function wpep_string_ends_with($haystack, $needle) {
    if( is_array( $needle ) ) {
      foreach( $needle as $current_needle )
        if( wpep_string_ends_with( $haystack, $current_needle ) )
          return true;

      return false;
    }

    // search forward starting from end minus needle length characters
    return $needle === "" || (($temp = strlen($haystack) - strlen($needle)) >= 0 && strpos($haystack, $needle, $temp) !== false);
  }

}

if( !function_exists( 'wpep_utility_selected' ) ) {

  /**
   * @param $selected
   * @param bool $current
   * @param bool $echo
   * @return string
   */
  function wpep_utility_selected( $selected, $current = true, $echo = true ) {
    if( is_array( $current ) )
      return selected( 1, in_array( $selected, $current ), $echo );

    return selected( $selected, $current, $echo );
  }

}

if( !function_exists( 'wpep_utility_checked' ) ) {

  function wpep_utility_checked( $checked, $current = true, $echo = true ) {
    if( is_array( $current ) )
      return checked( 1, in_array( $checked, $current ), $echo );

    return checked( $checked, $current, $echo );
  }

}

if( !function_exists( 'wpep_utilities_sanitize_input' ) ) {

  function wpep_utilities_sanitize_input( $value ) {
    if( is_array( $value ) ) {
      foreach( $value as $k => $v )
        $value[$k] = wpep_utilities_sanitize_input( $v );

      return $value;
    }

    return is_numeric( $value ) ? $value : stripslashes( $value );
  }

}


if( !function_exists( 'wpep_utilities_map_object' ) ) {

  function wpep_utilities_map_object($array, $param = 'ID', $multiple = false ) {
    $ret = [];

    foreach ($array as $a) {
      if( !$multiple ) {
        $ret[ $a->$param ] = $a;
        continue;
      }

      if( !isset( $ret[ $a->$param ] ) )
        $ret[ $a->$param ] = [];

      $ret[ $a->$param ][] = $a;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utilities_filter_object_list_by_params' ) ) {

  function wpep_utilities_filter_object_list_by_params($array, $filter_rules = [], $map_by = false ) {
    $ret = [];

    foreach ($array as $a) {
      foreach( $filter_rules as $param => $value ) {
        if( $a->$param != $value )
          continue 2;
      }

      if( $map_by === false )
        $ret[] = $a;
      else
        $ret[ $a->$map_by ] = $a;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_map_to_array_assoc' ) ) {

  function wpep_utility_map_to_array_assoc( $array, $key_param = 'key', $value_param = 'value', $merge_existent_indexes = false ) {
    if( !is_array( $array ) )
      return [];

    $ret = [];

    foreach ( $array as $a ) {
      if( is_object( $a ) )
        $a = get_object_vars( $a );

      if( isset( $ret[ $a[$key_param] ] ) && $merge_existent_indexes ) {
        if( !is_array( $ret[ $a[$key_param] ] ) )
          $ret[ $a[$key_param] ] = [ $ret[ $a[$key_param] ] ];

        $ret[ $a[$key_param] ][] = $a[$value_param];

        continue;
      }

      $ret[ $a[$key_param] ] = $a[$value_param];
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_array_sum' ) ) {

  function wpep_utility_array_sum( $array ) {
    if( is_object( $array ) )
      $array = get_object_vars( $array );

    return array_sum( $array );
  }

}

if( !function_exists( 'wpep_utility_map_to_object_assoc' ) ) {

  function wpep_utility_map_to_object_assoc( $object, $key_param = 'key', $value_param = 'value' ) {
    if( !is_array( $object ) )
      return new stdClass();

    $ret = new stdClass();

    foreach ($object as $o) {
      $index = $o->$key_param;
      $ret->$index = $o->$value_param;
    }

    return $ret;
  }

}

if( !function_exists( 'wpep_utility_time_remaining' ) ) {

  function wpep_utility_time_remaining($seconds) {
    $t = round($seconds);
    return sprintf('%02d:%02d:%02d', ($t/3600),($t/60%60), $t%60);
  }

}

if( !function_exists( 'wpep_get_percentage' ) ) {

  /**
   * @param $percent / goal
   * @param int|float|bool $total
   * @param string $prefix
   * @param string $suffix
   * @param int $decimals
   * @return string
   */
  function wpep_get_percentage( $percent, $total = false, $prefix = '', $suffix = '%', $decimals = 2 ) {
    $percent = floatval( $percent );

    if( $total !== false )
      $percent = ( $total == 0 ? 0 : ( $percent / $total ) * 100 );

    $response = ( $percent != 0 ? number_format_i18n( $percent, $decimals ) : '0.00' );

    return $prefix . $response . $suffix;
  }

}

if( !function_exists( 'wpep_time_to_dhms' ) ) {

  /**
   * @param $time
   * @return array [
  'days'    => 0,
  'hours'   => 0,
  'minutes' => 0,
  'seconds' => 0
  ]
   */
  function wpep_time_to_dhms( $time ) {
    $time = intval( $time );

    $response = [
      'days'    => 0,
      'hours'   => 0,
      'minutes' => 0,
      'seconds' => 0
    ];

    if( $time == 0 )
      return $response;

    $response['days'] = floor( $time / ( 60 * 60 * 24 ) );
    $time %= ( 60 * 60 * 24 );
    $response['hours'] = floor( $time / ( 60 * 60 ) );
    $time %= ( 60 * 60 );
    $response['minutes'] = floor( $time / 60 );
    $time %= 60;
    $response['seconds'] = $time;

    return $response;
  }

}

if( !function_exists( 'wpep_format_date' ) ) {

  function wpep_format_date( $date = false, $date_format = false ) {
    $date = ( is_numeric( $date ) || $date === false ? $date : strtotime( $date ) );

    $date_format = ( $date_format == false ? get_option( 'date_format' ) : $date_format );

    return date_i18n( $date_format, $date );
  }

}

if( !function_exists( 'wpep_permalink_extend' ) ) {

  function wpep_permalink_extend($permalink, $param, $value ) {
    $permalink = remove_query_arg( $param, $permalink );

    $permalink = add_query_arg( $param, $value, $permalink );

    return $permalink;
  }

}

if( !function_exists( 'wpep_let_to_num' ) ) {

  function wpep_let_to_num( $size ) {
    $l   = substr( $size, -1 );
    $ret = substr( $size, 0, -1 );
    switch ( strtoupper( $l ) ) {
      case 'P':
        $ret *= 1024;
      case 'T':
        $ret *= 1024;
      case 'G':
        $ret *= 1024;
      case 'M':
        $ret *= 1024;
      case 'K':
        $ret *= 1024;
    }
    return $ret;
  }

}

if( !function_exists( 'wpep_link_is_404' ) ) {

  function wpep_link_is_404($url) {
    $handle = curl_init($url);
    curl_setopt($handle,  CURLOPT_RETURNTRANSFER, TRUE);

    /* Get the HTML or whatever is linked in $url. */
    $response = curl_exec($handle);

    /* Check for 404 (file not found). */
    $httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
    curl_close($handle);

    /* If the document has loaded successfully without any redirection or error */
    if ($httpCode >= 200 && $httpCode < 300) {
      return false;
    } else {
      return true;
    }
  }

}

if( !function_exists( 'wpep_get_admin_icon_selection_list' ) ) {

  function wpep_get_admin_icon_selection_list() {
    $response = get_transient( 'wpep_admin_icon_selection_list' );

    if( is_array( $response ) )
      return $response;

    $dashicon_list = wpep_get_dashicons_list();
    sort( $dashicon_list );

    $response = [];

    foreach( $dashicon_list as $dashicon )
      $response[ 'dashicons ' . $dashicon ] = wpep_alias_to_label( str_replace( 'dashicons-', '', $dashicon ) );

    set_transient( 'wpep_admin_icon_selection_list', $response, 3600 * 24 );

    return $response;
  }

}

if( !function_exists( 'wpep_get_month_list_within_year' ) ) {

  function wpep_get_month_list_within_year( $year ) {
    if( $year > date("Y" ) )
      return [];

    $months_to = ( $year == date( "Y" ) ? intval( date("m" ) ) : 12 );
    $response  = [];

    global $wp_locale;

    for( $i = 1; $i <= $months_to; $i++ )
      $response[ $i ] = $wp_locale->get_month( $i );

    return $response;
  }

}

function wpep_convert_date_format_for_javascript($php_format) {
  $symbol_map = [
    // Day
    'd' => 'dd',
    'D' => 'D',
    'j' => 'd',
    'l' => 'DD',
    'N' => '',
    'S' => '',
    'w' => '',
    'z' => 'o',
    // Week
    'W' => '',
    // Month
    'F' => 'MM',
    'm' => 'mm',
    'M' => 'M',
    'n' => 'm',
    't' => '',
    // Year
    'L' => '',
    'o' => '',
    'Y' => 'yy',
    'y' => 'y',
  ];
  
  $response = ""; $flag_escape = false;
  
  for($i = 0; $i < strlen($php_format); $i++) {
    $char = $php_format[$i];
    if($char === '\\') {
      $i++;

      $response .= ($flag_escape ? $php_format[$i] : '\'' . $php_format[$i] );

      $flag_escape = true;

      continue;
    }

    if( $flag_escape ) {
      $response .= "'";
      $flag_escape = false;
    }

    $response .= isset( $symbol_map[$char] ) ? $symbol_map[$char] : $char;
  }

  return $response;
}

if( !function_exists( 'wpep_get_dashicons_list' ) ) {
  function wpep_get_dashicons_list() {
    return [
      'dashicons-menu',
      'dashicons-dashboard',
      'dashicons-admin-site',
      'dashicons-admin-media',
      'dashicons-admin-page',
      'dashicons-admin-comments',
      'dashicons-admin-appearance',
      'dashicons-admin-plugins',
      'dashicons-admin-users',
      'dashicons-admin-tools',
      'dashicons-admin-settings',
      'dashicons-admin-network',
      'dashicons-admin-generic',
      'dashicons-admin-home',
      'dashicons-admin-collapse',
      'dashicons-admin-links',
      'dashicons-admin-post',
      'dashicons-format-standard',
      'dashicons-format-image',
      'dashicons-format-gallery',
      'dashicons-format-audio',
      'dashicons-format-video',
      'dashicons-format-links',
      'dashicons-format-chat',
      'dashicons-format-status',
      'dashicons-format-aside',
      'dashicons-format-quote',
      'dashicons-welcome-write-blog',
      'dashicons-welcome-edit-page',
      'dashicons-welcome-add-page',
      'dashicons-welcome-view-site',
      'dashicons-welcome-widgets-menus',
      'dashicons-welcome-comments',
      'dashicons-welcome-learn-more',
      'dashicons-image-crop',
      'dashicons-image-rotate-left',
      'dashicons-image-rotate-right',
      'dashicons-image-flip-vertical',
      'dashicons-image-flip-horizontal',
      'dashicons-undo',
      'dashicons-redo',
      'dashicons-editor-bold',
      'dashicons-editor-italic',
      'dashicons-editor-ul',
      'dashicons-editor-ol',
      'dashicons-editor-quote',
      'dashicons-editor-alignleft',
      'dashicons-editor-aligncenter',
      'dashicons-editor-alignright',
      'dashicons-editor-insertmore',
      'dashicons-editor-spellcheck',
      'dashicons-editor-distractionfree',
      'dashicons-editor-expand',
      'dashicons-editor-contract',
      'dashicons-editor-kitchensink',
      'dashicons-editor-underline',
      'dashicons-editor-justify',
      'dashicons-editor-textcolor',
      'dashicons-editor-paste-word',
      'dashicons-editor-paste-text',
      'dashicons-editor-removeformatting',
      'dashicons-editor-video',
      'dashicons-editor-customchar',
      'dashicons-editor-outdent',
      'dashicons-editor-indent',
      'dashicons-editor-help',
      'dashicons-editor-strikethrough',
      'dashicons-editor-unlink',
      'dashicons-editor-rtl',
      'dashicons-editor-break',
      'dashicons-editor-code',
      'dashicons-editor-paragraph',
      'dashicons-align-left',
      'dashicons-align-right',
      'dashicons-align-center',
      'dashicons-align-none',
      'dashicons-lock',
      'dashicons-calendar',
      'dashicons-visibility',
      'dashicons-post-status',
      'dashicons-edit',
      'dashicons-post-trash',
      'dashicons-trash',
      'dashicons-external',
      'dashicons-arrow-up',
      'dashicons-arrow-down',
      'dashicons-arrow-left',
      'dashicons-arrow-right',
      'dashicons-arrow-up-alt',
      'dashicons-arrow-down-alt',
      'dashicons-arrow-left-alt',
      'dashicons-arrow-right-alt',
      'dashicons-arrow-up-alt2',
      'dashicons-arrow-down-alt2',
      'dashicons-arrow-left-alt2',
      'dashicons-arrow-right-alt2',
      'dashicons-leftright',
      'dashicons-sort',
      'dashicons-randomize',
      'dashicons-list-view',
      'dashicons-exerpt-view',
      'dashicons-hammer',
      'dashicons-art',
      'dashicons-migrate',
      'dashicons-performance',
      'dashicons-universal-access',
      'dashicons-universal-access-alt',
      'dashicons-tickets',
      'dashicons-nametag',
      'dashicons-clipboard',
      'dashicons-heart',
      'dashicons-megaphone',
      'dashicons-schedule',
      'dashicons-wordpress',
      'dashicons-wordpress-alt',
      'dashicons-pressthis,',
      'dashicons-update,',
      'dashicons-screenoptions',
      'dashicons-info',
      'dashicons-cart',
      'dashicons-feedback',
      'dashicons-cloud',
      'dashicons-translation',
      'dashicons-tag',
      'dashicons-category',
      'dashicons-archive',
      'dashicons-tagcloud',
      'dashicons-text',
      'dashicons-media-archive',
      'dashicons-media-audio',
      'dashicons-media-code',
      'dashicons-media-default',
      'dashicons-media-document',
      'dashicons-media-interactive',
      'dashicons-media-spreadsheet',
      'dashicons-media-text',
      'dashicons-media-video',
      'dashicons-playlist-audio',
      'dashicons-playlist-video',
      'dashicons-yes',
      'dashicons-no',
      'dashicons-no-alt',
      'dashicons-plus',
      'dashicons-plus-alt',
      'dashicons-minus',
      'dashicons-dismiss',
      'dashicons-marker',
      'dashicons-star-filled',
      'dashicons-star-half',
      'dashicons-star-empty',
      'dashicons-flag',
      'dashicons-share',
      'dashicons-share1',
      'dashicons-share-alt',
      'dashicons-share-alt2',
      'dashicons-twitter',
      'dashicons-rss',
      'dashicons-email',
      'dashicons-email-alt',
      'dashicons-facebook',
      'dashicons-facebook-alt',
      'dashicons-networking',
      'dashicons-googleplus',
      'dashicons-location',
      'dashicons-location-alt',
      'dashicons-camera',
      'dashicons-images-alt',
      'dashicons-images-alt2',
      'dashicons-video-alt',
      'dashicons-video-alt2',
      'dashicons-video-alt3',
      'dashicons-vault',
      'dashicons-shield',
      'dashicons-shield-alt',
      'dashicons-sos',
      'dashicons-search',
      'dashicons-slides',
      'dashicons-analytics',
      'dashicons-chart-pie',
      'dashicons-chart-bar',
      'dashicons-chart-line',
      'dashicons-chart-area',
      'dashicons-groups',
      'dashicons-businessman',
      'dashicons-id',
      'dashicons-id-alt',
      'dashicons-products',
      'dashicons-awards',
      'dashicons-forms',
      'dashicons-testimonial',
      'dashicons-portfolio',
      'dashicons-book',
      'dashicons-book-alt',
      'dashicons-download',
      'dashicons-upload',
      'dashicons-backup',
      'dashicons-clock',
      'dashicons-lightbulb',
      'dashicons-microphone',
      'dashicons-desktop',
      'dashicons-tablet',
      'dashicons-smartphone',
      'dashicons-smiley'
    ];
  }
}

if( !function_exists( 'wpep_get_native_font_list' ) ) {
  function wpep_get_native_font_list() {
    return [
      "Arial",
      "Calibri",
      "Century Gothic",
      "Courier",
      "Geneva",
      "Georgia",
      "Gill Sans",
      "Helvetica",
      "Impact",
      "Lucida",
      "Myriad Pro",
      "Palatino",
      "Tahoma",
      "Times New Roman",
      "Trebuchet",
      "Verdana"
    ];
  }
}

if( !function_exists( 'wpep_get_google_font_list' ) ) {

  function wpep_get_google_font_list() {
    return [
      "ABeeZee", "Abel", "Abhaya Libre", "Abril Fatface", "Aclonica",
      "Acme", "Actor", "Adamina", "Advent Pro", "Aguafina Script",
      "Akronim", "Aladin", "Aldrich", "Alef", "Alegreya",
      "Alegreya SC", "Alegreya Sans", "Alegreya Sans SC", "Alex Brush", "Alfa Slab One",
      "Alice", "Alike", "Alike Angular", "Allan", "Allerta",
      "Allerta Stencil", "Allura", "Almendra", "Almendra Display", "Almendra SC",
      "Amarante", "Amaranth", "Amatic SC", "Amatica SC", "Amethysta",
      "Amiko", "Amiri", "Amita", "Anaheim", "Andada",
      "Andika", "Angkor", "Annie Use Your Telescope", "Anonymous Pro", "Antic",
      "Antic Didone", "Antic Slab", "Anton", "Arapey", "Arbutus",
      "Arbutus Slab", "Architects Daughter", "Archivo Black", "Archivo Narrow", "Aref Ruqaa",
      "Arima Madurai", "Arimo", "Arizonia", "Armata", "Artifika",
      "Arvo", "Arya", "Asap", "Asar", "Asset",
      "Assistant", "Astloch", "Asul", "Athiti", "Atma",
      "Atomic Age", "Aubrey", "Audiowide", "Autour One", "Average",
      "Average Sans", "Averia Gruesa Libre", "Averia Libre", "Averia Sans Libre", "Averia Serif Libre",
      "Bad Script", "Baloo", "Baloo Bhai", "Baloo Bhaina", "Baloo Chettan",
      "Baloo Da", "Baloo Paaji", "Baloo Tamma", "Baloo Thambi", "Balthazar",
      "Bangers", "Basic", "Battambang", "Baumans", "Bayon",
      "Belgrano", "Belleza", "BenchNine", "Bentham", "Berkshire Swash",
      "Bevan", "Bigelow Rules", "Bigshot One", "Bilbo", "Bilbo Swash Caps",
      "BioRhyme", "BioRhyme Expanded", "Biryani", "Bitter", "Black Ops One",
      "Bokor", "Bonbon", "Boogaloo", "Bowlby One", "Bowlby One SC",
      "Brawler", "Bree Serif", "Bubblegum Sans", "Bubbler One", "Buda",
      "Buenard", "Bungee", "Bungee Hairline", "Bungee Inline", "Bungee Outline",
      "Bungee Shade", "Butcherman", "Butterfly Kids", "Cabin", "Cabin Condensed",
      "Cabin Sketch", "Caesar Dressing", "Cagliostro", "Cairo", "Calligraffitti",
      "Cambay", "Cambo", "Candal", "Cantarell", "Cantata One",
      "Cantora One", "Capriola", "Cardo", "Carme", "Carrois Gothic",
      "Carrois Gothic SC", "Carter One", "Catamaran", "Caudex", "Caveat",
      "Caveat Brush", "Cedarville Cursive", "Ceviche One", "Changa", "Changa One",
      "Chango", "Chathura", "Chau Philomene One", "Chela One", "Chelsea Market",
      "Chenla", "Cherry Cream Soda", "Cherry Swash", "Chewy", "Chicle",
      "Chivo", "Chonburi", "Cinzel", "Cinzel Decorative", "Clicker Script",
      "Coda", "Coda Caption", "Codystar", "Coiny", "Combo",
      "Comfortaa", "Coming Soon", "Concert One", "Condiment", "Content",
      "Contrail One", "Convergence", "Cookie", "Copse", "Corben",
      "Cormorant", "Cormorant Garamond", "Cormorant Infant", "Cormorant SC", "Cormorant Unicase",
      "Cormorant Upright", "Courgette", "Cousine", "Coustard", "Covered By Your Grace",
      "Crafty Girls", "Creepster", "Crete Round", "Crimson Text", "Croissant One",
      "Crushed", "Cuprum", "Cutive", "Cutive Mono", "Damion",
      "Dancing Script", "Dangrek", "David Libre", "Dawning of a New Day", "Days One",
      "Dekko", "Delius", "Delius Swash Caps", "Delius Unicase", "Della Respira",
      "Denk One", "Devonshire", "Dhurjati", "Didact Gothic", "Diplomata",
      "Diplomata SC", "Domine", "Donegal One", "Doppio One", "Dorsa",
      "Dosis", "Dr Sugiyama", "Droid Sans", "Droid Sans Mono", "Droid Serif",
      "Duru Sans", "Dynalight", "EB Garamond", "Eagle Lake", "Eater",
      "Economica", "Eczar", "Ek Mukta", "El Messiri", "Electrolize",
      "Elsie", "Elsie Swash Caps", "Emblema One", "Emilys Candy", "Engagement",
      "Englebert", "Enriqueta", "Erica One", "Esteban", "Euphoria Script",
      "Ewert", "Exo", "Exo 2", "Expletus Sans", "Fanwood Text",
      "Farsan", "Fascinate", "Fascinate Inline", "Faster One", "Fasthand",
      "Fauna One", "Federant", "Federo", "Felipa", "Fenix",
      "Finger Paint", "Fira Mono", "Fira Sans", "Fjalla One", "Fjord One",
      "Flamenco", "Flavors", "Fondamento", "Fontdiner Swanky", "Forum",
      "Francois One", "Frank Ruhl Libre", "Freckle Face", "Fredericka the Great", "Fredoka One",
      "Freehand", "Fresca", "Frijole", "Fruktur", "Fugaz One",
      "GFS Didot", "GFS Neohellenic", "Gabriela", "Gafata", "Galada",
      "Galdeano", "Galindo", "Gentium Basic", "Gentium Book Basic", "Geo",
      "Geostar", "Geostar Fill", "Germania One", "Gidugu", "Gilda Display",
      "Give You Glory", "Glass Antiqua", "Glegoo", "Gloria Hallelujah", "Goblin One",
      "Gochi Hand", "Gorditas", "Goudy Bookletter 1911", "Graduate", "Grand Hotel",
      "Gravitas One", "Great Vibes", "Griffy", "Gruppo", "Gudea",
      "Gurajada", "Habibi", "Halant", "Hammersmith One", "Hanalei",
      "Hanalei Fill", "Handlee", "Hanuman", "Happy Monkey", "Harmattan",
      "Headland One", "Heebo", "Henny Penny", "Herr Von Muellerhoff", "Hind",
      "Hind Guntur", "Hind Madurai", "Hind Siliguri", "Hind Vadodara", "Holtwood One SC",
      "Homemade Apple", "Homenaje", "IM Fell DW Pica", "IM Fell DW Pica SC", "IM Fell Double Pica",
      "IM Fell Double Pica SC", "IM Fell English", "IM Fell English SC", "IM Fell French Canon", "IM Fell French Canon SC",
      "IM Fell Great Primer", "IM Fell Great Primer SC", "Iceberg", "Iceland", "Imprima",
      "Inconsolata", "Inder", "Indie Flower", "Inika", "Inknut Antiqua",
      "Irish Grover", "Istok Web", "Italiana", "Italianno", "Itim",
      "Jacques Francois", "Jacques Francois Shadow", "Jaldi", "Jim Nightshade", "Jockey One",
      "Jolly Lodger", "Jomhuria", "Josefin Sans", "Josefin Slab", "Joti One",
      "Judson", "Julee", "Julius Sans One", "Junge", "Jura",
      "Just Another Hand", "Just Me Again Down Here", "Kadwa", "Kalam", "Kameron",
      "Kanit", "Kantumruy", "Karla", "Karma", "Katibeh",
      "Kaushan Script", "Kavivanar", "Kavoon", "Kdam Thmor", "Keania One",
      "Kelly Slab", "Kenia", "Khand", "Khmer", "Khula",
      "Kite One", "Knewave", "Kotta One", "Koulen", "Kranky",
      "Kreon", "Kristi", "Krona One", "Kumar One", "Kumar One Outline",
      "Kurale", "La Belle Aurore", "Laila", "Lakki Reddy", "Lalezar",
      "Lancelot", "Lateef", "Lato", "League Script", "Leckerli One",
      "Ledger", "Lekton", "Lemon", "Lemonada", "Libre Baskerville",
      "Libre Franklin", "Life Savers", "Lilita One", "Lily Script One", "Limelight",
      "Linden Hill", "Lobster", "Lobster Two", "Londrina Outline", "Londrina Shadow",
      "Londrina Sketch", "Londrina Solid", "Lora", "Love Ya Like A Sister", "Loved by the King",
      "Lovers Quarrel", "Luckiest Guy", "Lusitana", "Lustria", "Macondo",
      "Macondo Swash Caps", "Mada", "Magra", "Maiden Orange", "Maitree",
      "Mako", "Mallanna", "Mandali", "Marcellus", "Marcellus SC",
      "Marck Script", "Margarine", "Marko One", "Marmelad", "Martel",
      "Martel Sans", "Marvel", "Mate", "Mate SC", "Maven Pro",
      "McLaren", "Meddon", "MedievalSharp", "Medula One", "Meera Inimai",
      "Megrim", "Meie Script", "Merienda", "Merienda One", "Merriweather",
      "Merriweather Sans", "Metal", "Metal Mania", "Metamorphous", "Metrophobic",
      "Michroma", "Milonga", "Miltonian", "Miltonian Tattoo", "Miniver",
      "Miriam Libre", "Mirza", "Miss Fajardose", "Mitr", "Modak",
      "Modern Antiqua", "Mogra", "Molengo", "Molle", "Monda",
      "Monofett", "Monoton", "Monsieur La Doulaise", "Montaga", "Montez",
      "Montserrat", "Montserrat Alternates", "Montserrat Subrayada", "Moul", "Moulpali",
      "Mountains of Christmas", "Mouse Memoirs", "Mr Bedfort", "Mr Dafoe", "Mr De Haviland",
      "Mrs Saint Delafield", "Mrs Sheppards", "Mukta Vaani", "Muli", "Mystery Quest",
      "NTR", "Neucha", "Neuton", "New Rocker", "News Cycle",
      "Niconne", "Nixie One", "Nobile", "Nokora", "Norican",
      "Nosifer", "Nothing You Could Do", "Noticia Text", "Noto Sans", "Noto Serif",
      "Nova Cut", "Nova Flat", "Nova Mono", "Nova Oval", "Nova Round",
      "Nova Script", "Nova Slim", "Nova Square", "Numans", "Nunito",
      "Nunito Sans", "Odor Mean Chey", "Offside", "Old Standard TT", "Oldenburg",
      "Oleo Script", "Oleo Script Swash Caps", "Open Sans", "Open Sans Condensed", "Oranienbaum",
      "Orbitron", "Oregano", "Orienta", "Original Surfer", "Oswald",
      "Over the Rainbow", "Overlock", "Overlock SC", "Ovo", "Oxygen",
      "Oxygen Mono", "PT Mono", "PT Sans", "PT Sans Caption", "PT Sans Narrow",
      "PT Serif", "PT Serif Caption", "Pacifico", "Palanquin", "Palanquin Dark",
      "Paprika", "Parisienne", "Passero One", "Passion One", "Pathway Gothic One",
      "Patrick Hand", "Patrick Hand SC", "Pattaya", "Patua One", "Pavanam",
      "Paytone One", "Peddana", "Peralta", "Permanent Marker", "Petit Formal Script",
      "Petrona", "Philosopher", "Piedra", "Pinyon Script", "Pirata One",
      "Plaster", "Play", "Playball", "Playfair Display", "Playfair Display SC",
      "Podkova", "Poiret One", "Poller One", "Poly", "Pompiere",
      "Pontano Sans", "Poppins", "Port Lligat Sans", "Port Lligat Slab", "Pragati Narrow",
      "Prata", "Preahvihear", "Press Start 2P", "Pridi", "Princess Sofia",
      "Prociono", "Prompt", "Prosto One", "Proza Libre", "Puritan",
      "Purple Purse", "Quando", "Quantico", "Quattrocento", "Quattrocento Sans",
      "Questrial", "Quicksand", "Quintessential", "Qwigley", "Racing Sans One",
      "Radley", "Rajdhani", "Rakkas", "Raleway", "Raleway Dots",
      "Ramabhadra", "Ramaraja", "Rambla", "Rammetto One", "Ranchers",
      "Rancho", "Ranga", "Rasa", "Rationale", "Ravi Prakash",
      "Redressed", "Reem Kufi", "Reenie Beanie", "Revalia", "Rhodium Libre",
      "Ribeye", "Ribeye Marrow", "Righteous", "Risque", "Roboto",
      "Roboto Condensed", "Roboto Mono", "Roboto Slab", "Rochester", "Rock Salt",
      "Rokkitt", "Romanesco", "Ropa Sans", "Rosario", "Rosarivo",
      "Rouge Script", "Rozha One", "Rubik", "Rubik Mono One", "Rubik One",
      "Ruda", "Rufina", "Ruge Boogie", "Ruluko", "Rum Raisin",
      "Ruslan Display", "Russo One", "Ruthie", "Rye", "Sacramento",
      "Sahitya", "Sail", "Salsa", "Sanchez", "Sancreek",
      "Sansita One", "Sarala", "Sarina", "Sarpanch", "Satisfy",
      "Scada", "Scheherazade", "Schoolbell", "Scope One", "Seaweed Script",
      "Secular One", "Sevillana", "Seymour One", "Shadows Into Light", "Shadows Into Light Two",
      "Shanti", "Share", "Share Tech", "Share Tech Mono", "Shojumaru",
      "Short Stack", "Shrikhand", "Siemreap", "Sigmar One", "Signika",
      "Signika Negative", "Simonetta", "Sintony", "Sirin Stencil", "Six Caps",
      "Skranji", "Slabo 13px", "Slabo 27px", "Slackey", "Smokum",
      "Smythe", "Sniglet", "Snippet", "Snowburst One", "Sofadi One",
      "Sofia", "Sonsie One", "Sorts Mill Goudy", "Source Code Pro", "Source Sans Pro",
      "Source Serif Pro", "Space Mono", "Special Elite", "Spicy Rice", "Spinnaker",
      "Spirax", "Squada One", "Sree Krushnadevaraya", "Sriracha", "Stalemate",
      "Stalinist One", "Stardos Stencil", "Stint Ultra Condensed", "Stint Ultra Expanded", "Stoke",
      "Strait", "Sue Ellen Francisco", "Suez One", "Sumana", "Sunshiney",
      "Supermercado One", "Sura", "Suranna", "Suravaram", "Suwannaphum",
      "Swanky and Moo Moo", "Syncopate", "Tangerine", "Taprom", "Tauri",
      "Taviraj", "Teko", "Telex", "Tenali Ramakrishna", "Tenor Sans",
      "Text Me One", "The Girl Next Door", "Tienne", "Tillana", "Timmana",
      "Tinos", "Titan One", "Titillium Web", "Trade Winds", "Trirong",
      "Trocchi", "Trochut", "Trykker", "Tulpen One", "Ubuntu",
      "Ubuntu Condensed", "Ubuntu Mono", "Ultra", "Uncial Antiqua", "Underdog",
      "Unica One", "UnifrakturCook", "UnifrakturMaguntia", "Unkempt", "Unlock",
      "Unna", "VT323", "Vampiro One", "Varela", "Varela Round",
      "Vast Shadow", "Vesper Libre", "Vibur", "Vidaloka", "Viga",
      "Voces", "Volkhov", "Vollkorn", "Voltaire", "Waiting for the Sunrise",
      "Wallpoet", "Walter Turncoat", "Warnes", "Wellfleet", "Wendy One",
      "Wire One", "Work Sans", "Yanone Kaffeesatz", "Yantramanav", "Yatra One",
      "Yellowtail", "Yeseva One", "Yesteryear", "Yrsa", "Zeyada"
    ];
  }
}