<?php

class WPEP_Integration_Email_Marketing_Coordinator_Frontend {

  /**
   * @var $controller WPEP_Integration_Email_Marketing
   */
  protected $_controller;

  /**
   * @var $controller WPEP_Integration_Email_Marketing
   */
  public function __construct( WPEP_Integration_Email_Marketing $controller ) {
    $this->_controller = $controller;
  }

  public function init() {
    add_action( 'wpep_user_activity_lesson_status', [ $this, 'user_lesson_activity' ], 10, 3 );
    add_action( 'wpep_user_set_course_data', [ $this, 'user_course_data'], 10, 7 );
    add_action( 'wpep_user_set_assessment_data', [ $this, 'user_assessment_data' ], 10, 7 );
  }

  public function user_lesson_activity($lesson_id, $is_completed, $user_id) {
    $option_name = $this->_controller->lesson_option_name;
    $lesson = WPEP\Entity\Course::instance()->get_lesson( $lesson_id );

    if( isset( $lesson->options->$option_name ) && $lesson->options->$option_name != '' ) {
      if( $is_completed )
        $this->_controller->add_tag( explode(",", $lesson->options->$option_name ), $user_id );
      else
        $this->_controller->remove_tag( explode(",", $lesson->options->$option_name ), $user_id );
    }

    $section = WPEP\Entity\Course::instance()->get_section( $lesson->section_id );

    $this->_user_lesson_activity_section( $section, $is_completed, $user_id );

    return true;
  }

  private function _user_lesson_activity_section( $section, $is_completed, $user_id ) {
    $option_name = $this->_controller->section_option_name;

    if( !isset( $section->options->$option_name ) || $section->options->$option_name == '' )
      return;

    $removeTags = true;

    if( $is_completed ) {
      $progress = WPEP\Controller::instance()->userActivity->course_progress_percentage( $section->post_id, $user_id, $section->id );

      $removeTags = ( $progress == 100 ? 0 : 1 );
    }

    if( !$removeTags )
      $this->_controller->add_tag( explode(",", $section->options->$option_name ), $user_id );
    else
      $this->_controller->remove_tag( explode(",", $section->options->$option_name ), $user_id );

  }

  public function user_course_data( $key, $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry ) {
    if( $key === WPEP_USER_COURSE_STARTED ) {
      $this->_user_course_data_started_at( $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry );
      return;
    }

    if( $key === WPEP_USER_COURSE_COMPLETED ) {
      $this->_user_course_data_completed( $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry );
      return;
    }
  }

  private function _user_course_data_started_at( $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry ) {
    if( $value === 0 )
      return;

    $tag = get_post_meta( $course_id, $this->_controller->post_meta_started, true );

    if( $tag == null || $tag == '' )
      return;

    $this->_controller->add_tag( explode(",", $tag ), $user_id );
  }

  private function _user_course_data_completed( $course_completed, $course_id, $section_id, $lesson_id, $user_id, $existent_entry ) {
    $tag = get_post_meta( $course_id, $this->_controller->post_meta_completed, true );

    if( $tag == null || $tag == '' )
      return;

    if( $course_completed )
      $this->_controller->add_tag( explode(",", $tag ), $user_id );
    else
      $this->_controller->remove_tag( explode(",", $tag ), $user_id );
  }

  public function user_assessment_data( $key, $value, $post_id, $question_id, $question_answer_id, $user_id, $updated ) {
    if( $key != WPEP_USER_ASSESSMENT_STATUS )
      return;

    $status_meta_map = [
      WPEP_USER_ASSESSMENT_STATUS_PENDING        => $this->_controller->post_meta_prefix . 'tag_status_pending',
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => $this->_controller->post_meta_prefix . 'tag_status_pending_review',
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => $this->_controller->post_meta_prefix . 'tag_status_completed',
      WPEP_USER_ASSESSMENT_STATUS_FAILED         => $this->_controller->post_meta_prefix . 'tag_status_failed',
    ];

    $remove_tags = [];

    foreach( $status_meta_map as $status => $meta_name ) {
      if( $status == $value )
        continue;

      $tag = get_post_meta( $post_id, $meta_name, true );

      if( $tag == null || $tag == '' )
        continue;

      $remove_tags = array_merge( $remove_tags, explode(",", $tag ) );
    }

    $remove_tags = array_unique( $remove_tags );

    if( !empty( $remove_tags ) )
      $this->_controller->remove_tag( $remove_tags, $user_id );

    if( !isset( $status_meta_map[ $value ] ) )
      return;

    $tag = get_post_meta( $post_id, $status_meta_map[ $value ], true );

    if( $tag == null || $tag == '' )
      return;

    $this->_controller->add_tag( explode(",", $tag ), $user_id );
  }

}