<?php

use WPEP\Admin\Coordinator\PlatformHealth\Skeleton as WPEP_PlatformHealth_Skeleton;

class WPEP_Integration_PlatformHealth_Coordinator_User_Course_Log extends WPEP_PlatformHealth_Skeleton {

  public $name;
  public $alias;
  public $log_option_name;

  public function __construct( $name, $alias, $log_option_name ) {
    $this->name   = $name;
    $this->alias  = $alias;
    $this->log_option_name = $log_option_name;
  }

  public function get_name() {
    return $this->name;
  }

  public function get_alias() {
    return $this->alias;
  }

  public function download() {
    $table_information = $this->_get_table_data( 1000000, 1 );

    do_action(
      'wpep_admin_download_table',
      sanitize_title( $this->get_name() ) . '-' . date_i18n( get_option( 'date_format' ) ),
      $table_information['head'],
      $table_information['body']
    );
  }

  public function display( $filter_by = [] ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_table_data( $entries_per_page, $current_page );

    wpep_admin_display_table(
      __( $this->get_name(), "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  private function _get_table_data( $entries_per_page, $current_page ) {
    $current_page = ( $current_page <= 0 ? 1 : $current_page );

    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $entries = get_option( $this->log_option_name, [] );

    if( empty( $entries ) )
      return $table_information;

    $entries_total = count( $entries );
    $entries = array_slice( $entries, $entries_per_page * ( $current_page - 1 ), $entries_per_page );

    $table_information['head'] = [
      'created_at'   => "",
      'type'         => __( "Type" ),
      'message'      => __( "Notification", 'wpep' ),
      'user_name'    => __( "Student Name", 'wpep' ),
    ];

    foreach( $entries as $entry ) {
      $user_information = ( isset( $entry['user_id'] ) ? get_user_by( 'ID', $entry['user_id'] ) : NULL );

      $table_information['body'][] = [
        'created_at' => date_i18n( get_option( 'date_format' ), $entry['time'] ),
        'type'       => wpep_alias_to_label( $entry['type'] ),
        'message'    => $entry['message'],
        'user_name'  => ( $user_information == NULL ? '-' : wpep_content_user_profile_format_name( $user_information ) ),
      ];
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_total / $entries_per_page );

    return $table_information;
  }


}