<?php

namespace WPEP;

global $wpdb;

$section_table_name          = $wpdb->prefix . 'wpep_section';
$lesson_table_name           = $wpdb->prefix . 'wpep_section_lesson';
$lesson_completed_table_name = $wpdb->prefix . 'wpep_section_lesson_completed';

$existing_courses = get_posts( [ 'post_status' => 'any', 'post_type' => WPEP_POST_TYPE_COURSE, 'numberposts' => -1 ] );

$position = 1; foreach( $existing_courses as $existing_course ) {

  $description = get_post_meta( $existing_course->ID, '_cic_cb-course-desc', true );

  if( $description != null && $description != false && $description != '' )
    update_post_meta($existing_course->ID, 'wpep_course_description', $description );

  $section_count = get_post_meta($existing_course->ID, 'section_count', true);

  if( $section_count == false )
    continue;

  update_post_meta( $existing_course->ID, 'wpep_order', $position );

  $user_ids = get_users( [
      'meta_key' => 'course_progress_' . $existing_course->ID,
      'number'   => -1,
      'fields'   => 'ID'
  ] );

  $user_progress_list = [];

  foreach( $user_ids as $user_id )
    $user_progress_list[$user_id] = get_user_meta( $user_id, 'course_progress_' . $existing_course->ID, true );

  for( $section_iterator = 0; $section_iterator < $section_count; $section_iterator++ ) {
    $section_information = get_post_meta($existing_course->ID, 'section_meta_' . $section_iterator, true);

    if( $section_information == false )
      continue;

    $wpdb->insert( $section_table_name, [
      'post_id'  => $existing_course->ID,
      'title'    => $section_information['title'],
      'order'    => $section_iterator + 1
    ] );

    $inserted_section_id = $wpdb->insert_id;

    foreach( $section_information['lesson_data'] as $lesson_key => $lesson_information ) {
      if( $lesson_key == $section_information['lesson_count'] )
        break;

      $wpdb->insert( $lesson_table_name, [
        'section_id' => $inserted_section_id,
        'title'      => $lesson_information['title'],
        'content'    => $lesson_information['desc'],
        'order'      => $lesson_key + 1
      ] );

      $inserted_lesson_id = $wpdb->insert_id;

      foreach( $user_progress_list as $user_id => $user_progress ) {

        if( isset( $user_progress['lesson_checkbox_value'][$section_iterator][$lesson_key] )
          && $user_progress['lesson_checkbox_value'][$section_iterator][$lesson_key] == 'true' ) {

          $wpdb->insert( $lesson_completed_table_name, [
            'lesson_id' => $inserted_lesson_id,
            'user_id'   => $user_id
          ] );

        }

      }

    }
  }

  $position++;
}

echo '<p>' . __( "Imported Previous Content to the new structure introduced in 1.3.0.0", "wpep" ) . '</p>';
echo '<p>' . sprintf( __( "%s Courses have been migrated.", "wpep" ), "" . ( $position - 1 ) ) . '</p>';
echo '<p>' . __( "Please verify if all your courses have been properly migrated, if you encounter any problems, stop right here and contact support right away.", "wpep" ) . '</p>';
echo '<p><a class="button button-primary wpep-migration-continue">' . __( "Continue", "wpep" ) . '</a></p>';
